#version 120

#ifdef GL_ES
precision highp float;
#endif

uniform sampler2D tex0;
uniform float brightness;
uniform float contrast;
uniform float saturation;
uniform vec3 tint;
uniform float alphaCorrection;

void main(void)
{
	vec4 color = texture2D(tex0, vec2(gl_TexCoord[0]));
	
	// ignore alpha for now
    color.rgb /= color.a;

    // apply brightness
    color.rgb += brightness;
	
    // apply contrast
    color.rgb = ((color.rgb - 0.5) * max(contrast, 0.0)) + 0.5;
	
    // calculate brightness before tint
    float beforeBrightness = (color.rgb.x + color.rgb.y + color.rgb.z)/3.0;
	
    // apply color tint
    color.rgb *= tint;
	
    // restore brightness after tint
    float nowBrightness = (color.rgb.x + color.rgb.y + color.rgb.z)/3.0;
    color.rgb += (beforeBrightness - nowBrightness);
	
    // apply saturation
    const vec3 luminance = vec3(0.2125, 0.7154, 0.0721);
    vec3 intensity = vec3(dot(color.rgb, luminance));
    color.rgb = mix(intensity, color.rgb, saturation);

    // put alpha back in
    color.rgb *= color.a;
	
	// Do some alpha correction
	color.a -= alphaCorrection;
	if(color.a < 0)
		color.a = 0;
	
	gl_FragColor = color;
}