package application;

import game.GameHandler;

import java.io.File;

import org.lwjgl.LWJGLUtil;
import org.newdawn.slick.AppGameContainer;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.util.Log;

import application.exception.handler.ErrorHandler;
import application.properties.Properties;

/**
 * Main game class
 * 
 * @author Drag-On
 * 
 */
public class Application
{
	/**
	 * Main entry point
	 * 
	 * @param args
	 */
	public static void main(final String[] args)
	{
		// Read properties
		Properties.parseProperties();
		Properties.parseArguments(args);
		Log.setLogSystem(new ErrorHandler(Properties.loggingLevel));
		Log.info("Starting application.");

		// Set important properties
		System.setProperty(
				"org.lwjgl.librarypath",
				new File(new File(System.getProperty("user.dir"),
						"../GameLib/lib/lwjgl-2.9.0/native"), LWJGLUtil
						.getPlatformName()).getAbsolutePath());
		System.setProperty("net.java.games.input.librarypath",
				System.getProperty("org.lwjgl.librarypath"));

		try
		{
			// Open game window
			final GameHandler handler = GameHandler.getInstance();
			final AppGameContainer container = new AppGameContainer(handler);
			container.setShowFPS(Properties.showFPS);
			container.setDisplayMode(Properties.screenWidth,
					Properties.screenHeight, Properties.fullscreen);
			container.setMultiSample(Properties.multiSampling);
			container.setSmoothDeltas(Properties.smoothDeltas);
			container.setVSync(Properties.vSync);
			container.setAlwaysRender(Properties.alwaysRender);

			container.start();
		}
		catch (final SlickException e)
		{
			Log.error("Failed to initialize game window.", e);
		}
	}

}
