/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2012
 ***
 *** 
 ** GameEditor
 ** 	Drag-On
 ***
 *****************************************************/
package application;

import java.lang.reflect.InvocationTargetException;

import javax.swing.SwingUtilities;

import org.newdawn.slick.util.Log;

import application.exception.handler.ErrorHandler;
import application.properties.Properties;
import application.windows.MapWindow;
import application.windows.WindowManager;

/**
 * Main game editor class
 * 
 * @author Drag-On
 * 
 */
public class Application
{
	/**
	 * Main entry point
	 * 
	 * @param args
	 */
	public static void main(final String[] args)
	{
		try
		{
			// Read properties
			Properties.parseProperties();
			Properties.parseArguments(args);
			Log.setLogSystem(new ErrorHandler(Properties.loggingLevel));
			Log.info("Starting application.");

			// Create main window
			final MapWindow mapWnd = new MapWindow();

			// Initialize window manager
			WindowManager.setMainWindow(mapWnd);

			// Open main window
			mapWnd.open();
			mapWnd.run();
			mapWnd.exit();
		}
		catch (final Exception e)
		{
			try
			{
				SwingUtilities.invokeAndWait(new Runnable()
				{
					@Override
					public void run()
					{
						Log.error("Failed to initialize main window.", e);
					}
				});
			}
			catch (InvocationTargetException | InterruptedException e1)
			{
				e1.printStackTrace();
			}
		}
	}
}
