/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2012
 ***
 *** 
 ** GameEditor
 ** 	Drag-On
 ***
 *****************************************************/
package application.listeners;

import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

/**
 * This listener is used as a workaround for window resizing with GameLib
 * 
 * @author Drag-On
 * 
 */
public class ResizeListener implements ControlListener, Runnable
{
	/**
	 * Timestamp when the last event occurred
	 */
	private long lastEvent = 0;

	/**
	 * Composite that was resized (should always be the one GameLib is
	 * initialized in)
	 */
	private Composite comp;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void controlMoved(final ControlEvent e)
	{ // Not used here
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void controlResized(final ControlEvent e)
	{
		// Control is currently being resized, so disable it and wait some time
		// to do the actual resize operation. This is necessary because the
		// OpenGL context cannot be updated that often.
		lastEvent = System.currentTimeMillis();
		comp = (Composite) e.widget;
		comp.setEnabled(false);
		Display.getDefault().timerExec(500, this);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void run()
	{
		// If enough time has passed since the last resize
		if ((lastEvent + 500) < System.currentTimeMillis())
		{
			// Re-enable rendering
			comp.setEnabled(true);
		} else
		{
			// otherwise wait some more time
			Display.getDefault().timerExec(500, this);
		}
	}
}
