/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2012
 ***
 *** 
 ** GameEditor
 ** 	Drag-On
 ***
 *****************************************************/
package application.windows;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;

/**
 * This is a base class for dialog windows with 'OK' and 'Cancel' buttons
 * 
 * @author Drag-On
 * 
 */
public abstract class AbstractDialogWindow extends AbstractWindow
{
	/**
	 * Ok button
	 */
	protected Button okButton;

	/**
	 * Cancel button
	 */
	protected Button cancelButton;

	/**
	 * Constructor
	 * 
	 * @param parent
	 *            Parent shell or <code>null</code> if none
	 */
	protected AbstractDialogWindow(final Shell parent)
	{
		super(parent);
		// Add a dispose listener to get informed if the user closed the window
		// by clicking the 'x' button
		getShell().addDisposeListener(new DisposeListener()
		{
			@Override
			public void widgetDisposed(final DisposeEvent e)
			{
				if (okButton.getData() == null)
					onCancel(null);
			}
		});
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void createControls(final Composite mainComp)
	{
		final Composite comp = new Composite(mainComp, SWT.NONE);
		comp.setLayout(new GridLayout(2, true));
		comp.setLayoutData(new GridData(SWT.TRAIL, SWT.TOP, false, false));
		okButton = new Button(comp, SWT.PUSH);
		okButton.setText("OK");
		okButton.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
		okButton.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(final SelectionEvent e)
			{
				okButton.setData(true);
				onOK(e);
			}

			@Override
			public void widgetDefaultSelected(final SelectionEvent e)
			{
				widgetSelected(e);
			}
		});
		cancelButton = new Button(comp, SWT.PUSH);
		cancelButton.setText("Cancel");
		cancelButton.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false,
				false));
		cancelButton.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(final SelectionEvent e)
			{
				onCancel(e);
			}

			@Override
			public void widgetDefaultSelected(final SelectionEvent e)
			{
				widgetSelected(e);
			}
		});

		comp.layout();
		comp.pack();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void initSize(final Shell shell)
	{
		// Don't do anything over here as the size is calculated to fit the
		// window best
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected int getShellStyle()
	{
		return SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL;
	}

	/**
	 * This method is called when the window is closed via the 'Cancel' button
	 * or the 'x' in the corner
	 * 
	 * @param e
	 *            Event information or <code>null</code> if window was not
	 *            closed via the 'Cancel' button (e.g. the 'x' button or Alt+F4)
	 */
	protected abstract void onCancel(SelectionEvent e);

	/**
	 * This method is called when the window is closed via the 'OK'
	 * 
	 * @param e
	 *            Event information
	 */
	protected abstract void onOK(SelectionEvent e);
}
