/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2012
 ***
 *** 
 ** GameEditor
 ** 	Drag-On
 ***
 *****************************************************/
package application.windows;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

/**
 * Base class for all windows
 * 
 * @author Drag-On
 * 
 */
public abstract class AbstractWindow
{
	/**
	 * Main window shell
	 */
	private final Shell shell;

	/**
	 * Constructor
	 * 
	 * @param parent
	 *            Parent shell or <code>null</code> if none
	 */
	protected AbstractWindow(final Shell parent)
	{
		// Create main window shell
		shell = new Shell(parent, getShellStyle());
		shell.setLayout(new GridLayout());
		shell.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		shell.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		shell.setText(getTitle());

		// Add close listener to auto-call close event
		shell.addListener(SWT.Close, new Listener()
		{
			@Override
			public void handleEvent(final Event event)
			{
				close();
			}
		});

		final Composite comp = new Composite(shell, SWT.NONE);
		final GridLayout layout = new GridLayout();
		layout.marginBottom = 0;
		layout.marginHeight = 0;
		layout.marginLeft = 0;
		layout.marginRight = 0;
		layout.marginTop = 0;
		layout.marginWidth = 0;
		comp.setLayout(layout);
		comp.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		// Create contents
		createContents(comp);
		createControls(comp);

		comp.pack();
		comp.layout();
	}

	/**
	 * @return The style of the underlying shell
	 */
	protected int getShellStyle()
	{
		return SWT.DIALOG_TRIM | SWT.RESIZE;
	}

	/**
	 * Creates all window contents
	 * 
	 * @param mainComp
	 *            Main composite
	 */
	protected abstract void createContents(Composite mainComp);

	/**
	 * Creates the controls
	 * 
	 * @param mainComp
	 *            Main composite
	 */
	protected abstract void createControls(Composite mainComp);

	/**
	 * @return The title of the window as written in the title bar
	 */
	protected abstract String getTitle();

	/**
	 * Sets the size of this window
	 * 
	 * @param shell
	 *            Main window shell
	 */
	protected abstract void initSize(Shell shell);

	/**
	 * Opens the window
	 */
	public void open()
	{
		shell.layout();
		shell.pack();
		initSize(shell);
		shell.open();
		final Display display = shell.getDisplay();
		while (!shell.isDisposed())
		{
			if (!display.readAndDispatch())
				display.sleep();
		}
	}

	/**
	 * Closes the window
	 */
	protected void close()
	{
		WindowManager.close(this.getClass());
	}

	/**
	 * @return the shell
	 */
	protected Shell getShell()
	{
		return shell;
	}
}
