/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameEditor
 ** 	Drag-On
 ***
 *****************************************************/
package application.windows;

import game.resources.ResourceProvider;
import game.resources.ResourceType;
import game.states.map.ai.character.BasicCharacterAI;
import game.states.map.entity.BaseCharacter;
import game.states.map.entity.CharacterDirection;
import game.states.map.entity.CollidableGameEntity;
import game.states.map.entity.DrawableGameEntity;
import game.states.map.entity.NonPlayerCharacter;

import javax.swing.SwingUtilities;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;

/**
 * @author Drag-On
 * 
 */
public class CharacterEditorWindow extends AbstractWindow
{
	/**
	 * Add button
	 */
	private Button addButton;

	/**
	 * Delete button
	 */
	private Button delButton;

	/**
	 * Spinner for x position
	 */
	private Spinner xSpinner;

	/**
	 * Spinner for y position
	 */
	private Spinner ySpinner;

	/**
	 * Resource ID
	 */
	private Text resourceIDText;

	/**
	 * Combo box for selecting initial face direction
	 */
	private Combo directionCombo;

	/**
	 * Dialog ID
	 */
	private Text dialogIDText;

	/**
	 * Name of the character
	 */
	private Text nameText;

	/**
	 * Currently selected NPC or <code>null</code> in case none selected
	 */
	private DrawableGameEntity selectedEntity = null;

	/**
	 * Combo box for selecting the AI to use for this character
	 */
	private Combo aiCombo;

	/**
	 * Constructor
	 * 
	 * @param parent
	 *            Parent shell
	 */
	protected CharacterEditorWindow(Shell parent)
	{
		super(parent);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void createContents(Composite mainComp)
	{
		// Buttons
		final Group buttonGroup = new Group(mainComp, SWT.NONE);
		buttonGroup.setLayout(new GridLayout(2, false));
		buttonGroup.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		buttonGroup.setText("Character handles");
		addButton = new Button(buttonGroup, SWT.PUSH);
		addButton.setText("New");
		addButton.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				addNewCharacter();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});
		delButton = new Button(buttonGroup, SWT.PUSH);
		delButton.setText("Delete");
		delButton.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				deleteCharacter();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});
		Label resIdLabel = new Label(buttonGroup, SWT.NONE);
		resIdLabel.setText("Res. ID:");
		resourceIDText = new Text(buttonGroup, SWT.BORDER);
		resourceIDText.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true,
				false, 1, 1));
		resourceIDText.addKeyListener(new KeyListener()
		{
			@Override
			public void keyReleased(KeyEvent e)
			{
				if (selectedEntity != null
						&& (e.keyCode == SWT.CR || e.keyCode == SWT.LF))
				{
					final String resId = resourceIDText.getText().trim();
					if (ResourceProvider.getInstance().isKnown(resId,
							ResourceType.SPRITESHEET))
					{
						SwingUtilities.invokeLater(new Runnable()
						{
							@Override
							public void run()
							{
								ResourceProvider.getInstance().get(resId,
										ResourceType.SPRITESHEET, true);
								selectedEntity.setResourceId(resId);
							}
						});
						MapWindow.setDirty(true);
					}
				}
			}

			@Override
			public void keyPressed(KeyEvent e)
			{
			}
		});

		// Character data
		final Group characterGroup = new Group(mainComp, SWT.NONE);
		characterGroup.setLayout(new GridLayout(2, false));
		characterGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true,
				true));
		characterGroup.setText("Character data");
		Label labelX = new Label(characterGroup, SWT.NONE);
		labelX.setText("X:");
		xSpinner = new Spinner(characterGroup, SWT.BORDER);
		xSpinner.setMinimum(0);
		xSpinner.setMaximum(16000);
		xSpinner.setSelection(32);
		xSpinner.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false, 1,
				1));
		xSpinner.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (CharacterEditorWindow.this.selectedEntity != null)
				{
					CharacterEditorWindow.this.selectedEntity.setX(xSpinner
							.getSelection());
					MapWindow.setDirty(true);
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});
		Label labelY = new Label(characterGroup, SWT.NONE);
		labelY.setText("Y:");
		ySpinner = new Spinner(characterGroup, SWT.BORDER);
		ySpinner.setMinimum(0);
		ySpinner.setMaximum(16000);
		ySpinner.setSelection(32);
		ySpinner.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false, 1,
				1));
		ySpinner.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (CharacterEditorWindow.this.selectedEntity != null)
				{
					CharacterEditorWindow.this.selectedEntity.setY(ySpinner
							.getSelection());
					MapWindow.setDirty(true);
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});
		Label labelDirection = new Label(characterGroup, SWT.NONE);
		labelDirection.setText("Dir:");
		directionCombo = new Combo(characterGroup, SWT.BORDER);
		directionCombo.setItems(new String[] { "Down", "Left", "Right", "Up" });
		directionCombo.select(0);
		directionCombo.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true,
				false, 1, 1));
		directionCombo.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (selectedEntity != null
						&& selectedEntity instanceof BaseCharacter)
				{
					BaseCharacter chara = (BaseCharacter) selectedEntity;
					switch (directionCombo.getSelectionIndex())
					{
					case 0:
						chara.face(CharacterDirection.DOWN);
						break;
					case 1:
						chara.face(CharacterDirection.LEFT);
						break;
					case 2:
						chara.face(CharacterDirection.RIGHT);
						break;
					case 3:
						chara.face(CharacterDirection.UP);
						break;
					}
					MapWindow.setDirty(true);
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});
		Label dialogIdLabel = new Label(characterGroup, SWT.NONE);
		dialogIdLabel.setText("Dialog:");
		dialogIDText = new Text(characterGroup, SWT.BORDER);
		dialogIDText.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false,
				1, 1));
		dialogIDText.addKeyListener(new KeyListener()
		{
			@Override
			public void keyReleased(KeyEvent e)
			{
				if (selectedEntity != null
						&& selectedEntity instanceof NonPlayerCharacter
						&& (e.keyCode == SWT.CR || e.keyCode == SWT.LF))
				{
					String id = dialogIDText.getText().trim();
					if (id != "")
					{
						NonPlayerCharacter npc = (NonPlayerCharacter) selectedEntity;
						npc.setDialogID(dialogIDText.getText().trim());
						MapWindow.setDirty(true);
					}
				}
			}

			@Override
			public void keyPressed(KeyEvent e)
			{
			}
		});
		Label nameLabel = new Label(characterGroup, SWT.NONE);
		nameLabel.setText("Name:");
		nameText = new Text(characterGroup, SWT.BORDER);
		nameText.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false, 1,
				1));
		nameText.addKeyListener(new KeyListener()
		{
			@Override
			public void keyReleased(KeyEvent e)
			{
				if (selectedEntity != null
						&& selectedEntity instanceof NonPlayerCharacter)
				{
					String id = nameText.getText().trim();
					if (id != "")
					{
						NonPlayerCharacter npc = (NonPlayerCharacter) selectedEntity;
						npc.setName(nameText.getText().trim());
						MapWindow.setDirty(true);
					}
				}
			}

			@Override
			public void keyPressed(KeyEvent e)
			{
			}
		});
		Label labelAI = new Label(characterGroup, SWT.NONE);
		labelAI.setText("AI:");
		aiCombo = new Combo(characterGroup, SWT.BORDER);
		aiCombo.setItems(new String[] { "None", "SimpleAI" });
		aiCombo.select(0);
		aiCombo.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false, 1, 1));
		aiCombo.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (selectedEntity != null
						&& selectedEntity instanceof BaseCharacter)
				{
					BaseCharacter chara = (BaseCharacter) selectedEntity;
					switch (aiCombo.getSelectionIndex())
					{
					case 0:
						chara.setAi(null);
						break;
					case 1:
						chara.setAi(new BasicCharacterAI());
						break;
					}
				}
				MapWindow.setDirty(true);
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});

		updateControlsEnabledState();
	}

	/**
	 * Adds a new character to the map
	 */
	protected void addNewCharacter()
	{
		if (ResourceProvider.getInstance().isKnown(resourceIDText.getText(),
				ResourceType.SPRITESHEET))
		{
			final int x = xSpinner.getSelection();
			final int y = ySpinner.getSelection();
			final String resId = resourceIDText.getText().trim();
			if (ResourceProvider.getInstance().isKnown(resId,
					ResourceType.SPRITESHEET))
			{
				SwingUtilities.invokeLater(new Runnable()
				{
					@Override
					public void run()
					{
						// As the new character may need a new resource we need
						// to
						// load it in the AWT thread
						ResourceProvider.getInstance().get(resId,
								ResourceType.SPRITESHEET, true);
						final NonPlayerCharacter chara = new NonPlayerCharacter(
								x, y, resId, null);
						MapEditorToolWindow wnd = WindowManager
								.get(MapEditorToolWindow.class);
						wnd.getSelectedMapData().addDrawableEntity(chara);

						Display.getDefault().syncExec(new Runnable()
						{
							@Override
							public void run()
							{
								setSelectedEntity(chara);
							}
						});
					}
				});
				updateControlsEnabledState();

				// Update dirty state
				MapWindow.setDirty(true);
			}
		}
		else
		{
			MessageBox mb = new MessageBox(getShell());
			mb.setMessage("Please type the resource ID of the character graphic into the appropriate field.");
			mb.setText("Error");
			mb.open();
		}
	}

	/**
	 * Removes the currently selected character from map
	 */
	protected void deleteCharacter()
	{
		if (this.selectedEntity != null)
		{
			MapEditorToolWindow wnd = WindowManager
					.get(MapEditorToolWindow.class);
			wnd.getSelectedMapData().removeDrawableEntity(this.selectedEntity);
			this.selectedEntity = null;
			updateControlsEnabledState();

			// Update dirty state
			MapWindow.setDirty(true);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void createControls(Composite mainComp)
	{
		// There is nothing interesting over here. Really!
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected String getTitle()
	{
		return "Character Editor";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void initSize(Shell shell)
	{
		// shell.setSize(150, 285);
		shell.pack();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected int getShellStyle()
	{
		return SWT.DIALOG_TRIM | SWT.RESIZE | SWT.TOOL;
	}

	/**
	 * This methods updates the enabled states of all controls
	 */
	private void updateControlsEnabledState()
	{
		// Grey out controls if currently no map is selected
		MapEditorToolWindow mapToolWnd = WindowManager
				.get(MapEditorToolWindow.class);
		final boolean enabled = mapToolWnd.getSelectedMap() != null;
		addButton.setEnabled(enabled);
		delButton.setEnabled(enabled);
		xSpinner.setEnabled(enabled && selectedEntity != null);
		ySpinner.setEnabled(enabled && selectedEntity != null);
		directionCombo.setEnabled(enabled
				&& selectedEntity instanceof BaseCharacter);
		dialogIDText.setEnabled(enabled
				&& selectedEntity instanceof NonPlayerCharacter);
		nameText.setEnabled(enabled
				&& selectedEntity instanceof NonPlayerCharacter);
		aiCombo.setEnabled(enabled && selectedEntity instanceof BaseCharacter);
	}

	/**
	 * @return the selectedEntity
	 */
	public DrawableGameEntity getSelectedEntity()
	{
		return selectedEntity;
	}

	/**
	 * @param selectedEntity
	 *            the selectedEntity to set
	 */
	public void setSelectedEntity(DrawableGameEntity selectedEntity)
	{
		this.selectedEntity = selectedEntity;
		if (this.selectedEntity != null)
		{
			this.resourceIDText.setText(this.selectedEntity.getResourceId());
			this.xSpinner.setSelection(this.selectedEntity.getX());
			this.ySpinner.setSelection(this.selectedEntity.getY());
			if (this.selectedEntity instanceof BaseCharacter)
			{
				BaseCharacter chara = (BaseCharacter) this.selectedEntity;
				this.directionCombo.select(chara.getDirection().ordinal());
				if (chara.getAi() == null)
					this.aiCombo.select(0);
				else
					this.aiCombo.select(1);
			}
			if (this.selectedEntity instanceof NonPlayerCharacter)
			{
				NonPlayerCharacter chara = (NonPlayerCharacter) this.selectedEntity;
				this.dialogIDText.setText(chara.getDialogID() != null ? chara
						.getDialogID() : "");
				this.nameText.setText(chara.getName() != null ? chara.getName()
						: "");
			}
		}
		updateControlsEnabledState();
	}

	/**
	 * Changes the coordinates of the selected entity in case one is selected
	 * 
	 * @param x
	 *            New x
	 * @param y
	 *            New y
	 */
	public void changeSelectedCoordinates(int x, int y)
	{
		if (this.selectedEntity != null)
		{
			this.xSpinner.setSelection(x);
			this.ySpinner.setSelection(y);
			if (this.selectedEntity instanceof CollidableGameEntity)
			{
				// This will cause collision detection
				CollidableGameEntity e = (CollidableGameEntity) this.selectedEntity;
				e.moveTo(x, y);
			}
			else
			{
				this.selectedEntity.setX(x);
				this.selectedEntity.setY(y);
			}
			MapWindow.setDirty(true);
		}
	}
}
