/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameEditor
 ** 	Drag-On
 ***
 *****************************************************/
package application.windows;

import game.GameHandler;
import game.resources.TileData;
import game.states.MapState;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

/**
 * This window is used to select a section of the map an duplicate it to some
 * other position
 * 
 * @author Drag-On
 * 
 */
public class DuplicateTilesWindow extends AbstractDialogWindow
{
	/**
	 * Button for the upper left corner to copy from
	 */
	private Button fromPosLeft;

	/**
	 * Button for the lower right corner to copy from
	 */
	private Button fromPosRight;

	/**
	 * Button for the upper right coordinates to copy to
	 */
	private Button toPos;

	/**
	 * 0: From upper left 1: From lower right 2: To upper left
	 */
	private int selectionMode = 0;

	/**
	 * Constructor
	 * 
	 * @param parent
	 *            Parent shell
	 */
	protected DuplicateTilesWindow(Shell parent)
	{
		super(parent);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void createControls(final Composite mainComp)
	{
		super.createControls(mainComp);

		updateControlsEnabledState();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void createContents(Composite mainComp)
	{
		Label labelInfo = new Label(mainComp, SWT.NONE);
		labelInfo
				.setText("Beware: This will probably overwrite some sections of your map,\nkill your kittens or contribute in other ways to the rise of evil.\nAs it is not reversible it may be wise to save your work at first!");

		Group groupFrom = new Group(mainComp, SWT.NONE);
		groupFrom.setText("From");
		groupFrom.setLayout(new GridLayout(4, false));
		groupFrom.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		Label labelUpperLeft = new Label(groupFrom, SWT.NONE);
		labelUpperLeft.setText("Upper left:");
		fromPosLeft = new Button(groupFrom, SWT.NONE);
		fromPosLeft.setText("");
		fromPosLeft.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		fromPosLeft.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				selectionMode = 0;
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});
		Label labelLowerRight = new Label(groupFrom, SWT.NONE);
		labelLowerRight.setText("Lower right:");
		fromPosRight = new Button(groupFrom, SWT.NONE);
		fromPosRight.setText("");
		fromPosRight
				.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		fromPosRight.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				selectionMode = 1;
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});
		Group groupTo = new Group(mainComp, SWT.NONE);
		groupTo.setText("To");
		groupTo.setLayout(new GridLayout(2, false));
		groupTo.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		Label labelTo = new Label(groupTo, SWT.NONE);
		labelTo.setText("Upper left:");
		toPos = new Button(groupTo, SWT.NONE);
		toPos.setText("");
		toPos.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		toPos.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				selectionMode = 2;
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void onCancel(SelectionEvent e)
	{
		close();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void onOK(SelectionEvent e)
	{
		MapState mapState = (MapState) GameHandler.getInstance().getState(
				GameHandler.STATE_MAP);

		int layers = mapState.getMapData().getAmountOfLayers();
		int width = (int) fromPosRight.getData("x")
				- (int) fromPosLeft.getData("x");
		int height = (int) fromPosRight.getData("y")
				- (int) fromPosLeft.getData("y");
		int fromX = (int) fromPosLeft.getData("x");
		int fromY = (int) fromPosLeft.getData("y");
		int toX = (int) toPos.getData("x");
		int toY = (int) toPos.getData("y");
		int xDir = width > 0 ? 1 : -1;
		int yDir = height > 0 ? 1 : -1;

		for (int x = 0; x != width + xDir; x += xDir)
		{
			for (int y = 0; y != height + yDir; y += yDir)
			{
				for (int l = 0; l < layers; l++)
				{
					TileData tile = mapState.getMapData().getTile(fromX + x,
							fromY + y, l);
					if (tile != null)
						mapState.getMapData().setTile(toX + x, toY + y, l,
								tile.getX(), tile.getY());
					else
						mapState.getMapData().removeTile(toX + x, toY + y, l);
				}
			}
		}

		MapWindow.setDirty(true);

		close();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected String getTitle()
	{
		return "Duplicate map section";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected int getShellStyle()
	{
		return SWT.DIALOG_TRIM;
	}

	/**
	 * This methods updates the enabled states of all controls
	 */
	private void updateControlsEnabledState()
	{
		// Grey out controls if currently no map is selected
		final boolean enabled = fromPosLeft.getData("x") != null
				&& fromPosRight.getData("x") != null
				&& toPos.getData("x") != null;
		okButton.setEnabled(enabled);
	}

	/**
	 * 
	 * @param x
	 * @param y
	 */
	public void setSelectedPos(int x, int y)
	{
		switch (this.selectionMode)
		{
		case 0:
			this.fromPosLeft.setData("x", x);
			this.fromPosLeft.setData("y", y);
			this.fromPosLeft.setText("(" + x + "|" + y + ")");
			break;
		case 1:
			this.fromPosRight.setData("x", x);
			this.fromPosRight.setData("y", y);
			this.fromPosRight.setText("(" + x + "|" + y + ")");
			break;
		case 2:
			this.toPos.setData("x", x);
			this.toPos.setData("y", y);
			this.toPos.setText("(" + x + "|" + y + ")");
			break;
		}
		updateControlsEnabledState();
	}
}
