/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameEditor
 ** 	Drag-On
 ***
 *****************************************************/
package application.windows;

import game.resources.ResourceProvider;
import game.resources.ResourceType;
import game.states.map.entity.BaseCharacter;
import game.states.map.entity.CharacterDirection;
import game.states.map.entity.CollidableGameEntity;
import game.states.map.entity.DrawableGameEntity;
import game.states.map.entity.EnemyCharacter;

import javax.swing.SwingUtilities;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;

/**
 * @author Drag-On
 * 
 */
public class EnemyEditorWindow extends AbstractWindow
{
	/**
	 * Currently selected entity or <code>null</code> in case none selected
	 */
	private DrawableGameEntity selectedEntity = null;

	/**
	 * Add button
	 */
	private Button addButton;

	/**
	 * Delete button
	 */
	private Button delButton;

	/**
	 * Spinner for x position
	 */
	private Spinner xSpinner;

	/**
	 * Spinner for y position
	 */
	private Spinner ySpinner;

	/**
	 * Resource ID
	 */
	private Text resourceIDText;

	/**
	 * Combo box for selecting initial face direction
	 */
	private Combo directionCombo;

	/**
	 * Resource ID of melee weapon
	 */
	private Text meleeResIdText;

	/**
	 * Resource ID of ranged weapon
	 */
	private Text rangedResIdText;

	/**
	 * Resource ID of some other item
	 */
	private Text otherItemResIdText;

	/**
	 * Spinner for amount of items
	 */
	private Spinner amountOfItemsSpinner;

	/**
	 * Spinner for base health value
	 */
	private Spinner healthBaseSpinner;

	/**
	 * Spinner for base attack value
	 */
	private Spinner attackBaseSpinner;

	/**
	 * Spinner for base defense value
	 */
	private Spinner defenseBaseSpinner;

	/**
	 * Constructor
	 * 
	 * @param parent
	 *            Parent shell
	 */
	protected EnemyEditorWindow(Shell parent)
	{
		super(parent);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void createContents(Composite mainComp)
	{
		// Buttons
		final Group buttonGroup = new Group(mainComp, SWT.NONE);
		buttonGroup.setLayout(new GridLayout(2, false));
		buttonGroup.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		buttonGroup.setText("Enemy handles");
		addButton = new Button(buttonGroup, SWT.PUSH);
		addButton.setText("New");
		addButton.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				addNewEnemy();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});
		delButton = new Button(buttonGroup, SWT.PUSH);
		delButton.setText("Delete");
		delButton.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				deleteEnemy();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});
		Label resIdLabel = new Label(buttonGroup, SWT.NONE);
		resIdLabel.setText("Res. ID:");
		resourceIDText = new Text(buttonGroup, SWT.BORDER);
		resourceIDText.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true,
				false, 1, 1));
		resourceIDText.addKeyListener(new KeyListener()
		{
			@Override
			public void keyReleased(KeyEvent e)
			{
				if (selectedEntity != null
						&& (e.keyCode == SWT.CR || e.keyCode == SWT.LF))
				{
					// selectedEntity.setAdditionalData(resourceIDText.getText()
					// .trim());
					MapWindow.setDirty(true);
				}
			}

			@Override
			public void keyPressed(KeyEvent e)
			{
			}
		});

		// Character data
		final Group characterGroup = new Group(mainComp, SWT.NONE);
		characterGroup.setLayout(new GridLayout(2, false));
		characterGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true,
				true));
		characterGroup.setText("Character data");
		Label labelX = new Label(characterGroup, SWT.NONE);
		labelX.setText("X:");
		xSpinner = new Spinner(characterGroup, SWT.BORDER);
		xSpinner.setMinimum(0);
		xSpinner.setMaximum(16000);
		xSpinner.setSelection(32);
		xSpinner.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false, 1,
				1));
		xSpinner.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (EnemyEditorWindow.this.selectedEntity != null)
				{
					EnemyEditorWindow.this.selectedEntity.setX(xSpinner
							.getSelection());
					MapWindow.setDirty(true);
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});
		Label labelY = new Label(characterGroup, SWT.NONE);
		labelY.setText("Y:");
		ySpinner = new Spinner(characterGroup, SWT.BORDER);
		ySpinner.setMinimum(0);
		ySpinner.setMaximum(16000);
		ySpinner.setSelection(32);
		ySpinner.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false, 1,
				1));
		ySpinner.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (EnemyEditorWindow.this.selectedEntity != null)
				{
					EnemyEditorWindow.this.selectedEntity.setY(ySpinner
							.getSelection());
					MapWindow.setDirty(true);
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});
		Label labelDirection = new Label(characterGroup, SWT.NONE);
		labelDirection.setText("Dir:");
		directionCombo = new Combo(characterGroup, SWT.BORDER);
		directionCombo.setItems(new String[] { "Down", "Left", "Right", "Up" });
		directionCombo.select(0);
		directionCombo.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true,
				false, 1, 1));
		directionCombo.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (selectedEntity != null
						&& selectedEntity instanceof BaseCharacter)
				{
					BaseCharacter chara = (BaseCharacter) selectedEntity;
					switch (directionCombo.getSelectionIndex())
					{
					case 0:
						chara.face(CharacterDirection.DOWN);
						break;
					case 1:
						chara.face(CharacterDirection.LEFT);
						break;
					case 2:
						chara.face(CharacterDirection.RIGHT);
						break;
					case 3:
						chara.face(CharacterDirection.UP);
						break;
					}
					MapWindow.setDirty(true);
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});

		// Equipment
		final Group enemyGroup = new Group(mainComp, SWT.NONE);
		enemyGroup.setLayout(new GridLayout(2, false));
		enemyGroup.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		enemyGroup.setText("Equipment");
		Label meleeResLabel = new Label(enemyGroup, SWT.NONE);
		meleeResLabel.setText("Melee Weapon:");
		meleeResIdText = new Text(enemyGroup, SWT.BORDER);
		meleeResIdText.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true,
				false, 1, 1));
		meleeResIdText.addKeyListener(new KeyListener()
		{
			@Override
			public void keyReleased(KeyEvent e)
			{
				if (selectedEntity != null
						&& selectedEntity instanceof EnemyCharacter
						&& (e.keyCode == SWT.CR || e.keyCode == SWT.LF))
				{
					if (ResourceProvider.getInstance().isKnown(
							meleeResIdText.getText(), ResourceType.ITEM))
					{
						EnemyCharacter enemy = ((EnemyCharacter) selectedEntity);
						String itemResId = meleeResIdText.getText();
						if (!enemy.getInventory().hasItem(itemResId))
						{
							enemy.getInventory().addItem(itemResId);
						}
						enemy.getInventory().setMeleeWeapon(
								enemy.getInventory().getItem(itemResId));
						MapWindow.setDirty(true);
					}
					else
					{
						MessageBox mb = new MessageBox(getShell());
						mb.setMessage("Please type the resource ID of the melee weapon into the appropriate field.");
						mb.setText("Error");
						mb.open();
					}
				}
			}

			@Override
			public void keyPressed(KeyEvent e)
			{
			}
		});
		Label rangedResLabel = new Label(enemyGroup, SWT.NONE);
		rangedResLabel.setText("Ranged Weapon:");
		rangedResIdText = new Text(enemyGroup, SWT.BORDER);
		rangedResIdText.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true,
				false, 1, 1));
		rangedResIdText.addKeyListener(new KeyListener()
		{
			@Override
			public void keyReleased(KeyEvent e)
			{
				if (selectedEntity != null
						&& selectedEntity instanceof EnemyCharacter
						&& (e.keyCode == SWT.CR || e.keyCode == SWT.LF))
				{
					if (ResourceProvider.getInstance().isKnown(
							rangedResIdText.getText(), ResourceType.ITEM))
					{
						EnemyCharacter enemy = ((EnemyCharacter) selectedEntity);
						String itemResId = rangedResIdText.getText();
						if (!enemy.getInventory().hasItem(itemResId))
						{
							enemy.getInventory().addItem(itemResId);
						}
						enemy.getInventory().setRangedWeapon(
								enemy.getInventory().getItem(itemResId));
						MapWindow.setDirty(true);
					}
					else
					{
						MessageBox mb = new MessageBox(getShell());
						mb.setMessage("Please type the resource ID of the ranged weapon into the appropriate field.");
						mb.setText("Error");
						mb.open();
					}
				}
			}

			@Override
			public void keyPressed(KeyEvent e)
			{
			}
		});
		otherItemResIdText = new Text(enemyGroup, SWT.BORDER);
		otherItemResIdText.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true,
				false, 1, 1));
		otherItemResIdText.addKeyListener(new KeyListener()
		{
			@Override
			public void keyReleased(KeyEvent e)
			{
				if (selectedEntity != null
						&& selectedEntity instanceof EnemyCharacter
						&& (e.keyCode == SWT.CR || e.keyCode == SWT.LF))
				{
					if (ResourceProvider.getInstance().isKnown(
							otherItemResIdText.getText(), ResourceType.ITEM))
					{
						EnemyCharacter enemy = ((EnemyCharacter) selectedEntity);
						String itemResId = otherItemResIdText.getText();
						amountOfItemsSpinner.setSelection(enemy.getInventory()
								.getAmountOf(itemResId));
					}
					else
					{
						MessageBox mb = new MessageBox(getShell());
						mb.setMessage("Please type the resource ID of an item into the appropriate field.");
						mb.setText("Error");
						mb.open();
					}
				}
			}

			@Override
			public void keyPressed(KeyEvent e)
			{
			}
		});
		amountOfItemsSpinner = new Spinner(enemyGroup, SWT.BORDER);
		amountOfItemsSpinner.setMinimum(0);
		amountOfItemsSpinner.setMaximum(10);
		amountOfItemsSpinner.setSelection(0);
		amountOfItemsSpinner.setLayoutData(new GridData(SWT.FILL, SWT.TOP,
				true, false, 1, 1));
		amountOfItemsSpinner.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (selectedEntity != null
						&& selectedEntity instanceof EnemyCharacter)
				{
					if (ResourceProvider.getInstance().isKnown(
							otherItemResIdText.getText(), ResourceType.ITEM))
					{
						EnemyCharacter enemy = ((EnemyCharacter) selectedEntity);
						String itemResId = otherItemResIdText.getText();
						int amount = amountOfItemsSpinner.getSelection()
								- enemy.getInventory().getAmountOf(itemResId);
						if (amount > 0)
						{
							enemy.getInventory().addItem(itemResId, amount);
							MapWindow.setDirty(true);
						}
						else if (amount < 0)
						{
							enemy.getInventory().removeItem(itemResId, -amount);
							MapWindow.setDirty(true);
						}
					}
					else
					{
						MessageBox mb = new MessageBox(getShell());
						mb.setMessage("Please type the resource ID of an item into the appropriate field.");
						mb.setText("Error");
						mb.open();
					}
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});

		// Strength
		final Group strengthGroup = new Group(mainComp, SWT.NONE);
		strengthGroup.setLayout(new GridLayout(2, false));
		strengthGroup
				.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		strengthGroup.setText("Strength");
		Label healthBaseLabel = new Label(strengthGroup, SWT.NONE);
		healthBaseLabel.setText("Health Base:");
		healthBaseSpinner = new Spinner(strengthGroup, SWT.BORDER);
		healthBaseSpinner.setMinimum(0);
		healthBaseSpinner.setMaximum(1000);
		healthBaseSpinner.setSelection(100);
		healthBaseSpinner.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true,
				false, 1, 1));
		healthBaseSpinner.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (selectedEntity != null
						&& selectedEntity instanceof EnemyCharacter)
				{

					EnemyCharacter enemy = ((EnemyCharacter) selectedEntity);
					enemy.setHealthBase(healthBaseSpinner.getSelection());
					MapWindow.setDirty(true);
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});
		Label attackBaseLabel = new Label(strengthGroup, SWT.NONE);
		attackBaseLabel.setText("Attack Base:");
		attackBaseSpinner = new Spinner(strengthGroup, SWT.BORDER);
		attackBaseSpinner.setMinimum(0);
		attackBaseSpinner.setMaximum(1000);
		attackBaseSpinner.setSelection(100);
		attackBaseSpinner.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true,
				false, 1, 1));
		attackBaseSpinner.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (selectedEntity != null
						&& selectedEntity instanceof EnemyCharacter)
				{

					EnemyCharacter enemy = ((EnemyCharacter) selectedEntity);
					enemy.setAttackBase(attackBaseSpinner.getSelection());
					MapWindow.setDirty(true);
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});
		Label defenseBaseLabel = new Label(strengthGroup, SWT.NONE);
		defenseBaseLabel.setText("Defense Base:");
		defenseBaseSpinner = new Spinner(strengthGroup, SWT.BORDER);
		defenseBaseSpinner.setMinimum(0);
		defenseBaseSpinner.setMaximum(1000);
		defenseBaseSpinner.setSelection(100);
		defenseBaseSpinner.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true,
				false, 1, 1));
		defenseBaseSpinner.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (selectedEntity != null
						&& selectedEntity instanceof EnemyCharacter)
				{

					EnemyCharacter enemy = ((EnemyCharacter) selectedEntity);
					enemy.setDefenseBase(defenseBaseSpinner.getSelection());
					MapWindow.setDirty(true);
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});
		updateControlsEnabledState();
	}

	/**
	 * Adds a new enemy to the map
	 */
	protected void addNewEnemy()
	{
		if (ResourceProvider.getInstance().isKnown(resourceIDText.getText(),
				ResourceType.SPRITESHEET))
		{
			final int x = xSpinner.getSelection();
			final int y = ySpinner.getSelection();
			final String resId = resourceIDText.getText().trim();
			SwingUtilities.invokeLater(new Runnable()
			{
				@Override
				public void run()
				{
					// As the new character may need a new resource we need to
					// load it in the AWT thread
					final EnemyCharacter chara = new EnemyCharacter(x, y, resId);
					MapEditorToolWindow wnd = WindowManager
							.get(MapEditorToolWindow.class);
					wnd.getSelectedMapData().addDrawableEntity(chara);

					Display.getDefault().syncExec(new Runnable()
					{
						@Override
						public void run()
						{
							setSelectedEntity(chara);
						}
					});
				}
			});
			updateControlsEnabledState();

			// Update dirty state
			MapWindow.setDirty(true);
		}
		else
		{
			MessageBox mb = new MessageBox(getShell());
			mb.setMessage("Please type the resource ID of the enemy graphic into the appropriate field.");
			mb.setText("Error");
			mb.open();
		}
	}

	/**
	 * Removes the currently selected enemy from map
	 */
	protected void deleteEnemy()
	{
		if (this.selectedEntity != null)
		{
			MapEditorToolWindow wnd = WindowManager
					.get(MapEditorToolWindow.class);
			wnd.getSelectedMapData().removeDrawableEntity(this.selectedEntity);
			this.selectedEntity = null;
			updateControlsEnabledState();

			// Update dirty state
			MapWindow.setDirty(true);
		}
	}

	/**
	 * This methods updates the enabled states of all controls
	 */
	private void updateControlsEnabledState()
	{
		// Grey out controls if currently no map is selected
		MapEditorToolWindow mapToolWnd = WindowManager
				.get(MapEditorToolWindow.class);
		final boolean enabled = mapToolWnd.getSelectedMap() != null;
		addButton.setEnabled(enabled);
		delButton.setEnabled(enabled);
		xSpinner.setEnabled(enabled && selectedEntity != null);
		ySpinner.setEnabled(enabled && selectedEntity != null);
		directionCombo.setEnabled(enabled
				&& selectedEntity instanceof BaseCharacter);
		meleeResIdText.setEnabled(enabled
				&& selectedEntity instanceof EnemyCharacter);
		rangedResIdText.setEnabled(enabled
				&& selectedEntity instanceof EnemyCharacter);
		otherItemResIdText.setEnabled(enabled
				&& selectedEntity instanceof EnemyCharacter);
		amountOfItemsSpinner.setEnabled(enabled
				&& selectedEntity instanceof EnemyCharacter);
		healthBaseSpinner.setEnabled(enabled
				&& selectedEntity instanceof EnemyCharacter);
		attackBaseSpinner.setEnabled(enabled
				&& selectedEntity instanceof EnemyCharacter);
		defenseBaseSpinner.setEnabled(enabled
				&& selectedEntity instanceof EnemyCharacter);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void createControls(Composite mainComp)
	{
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected String getTitle()
	{
		return "Enemy Editor";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void initSize(Shell shell)
	{
		shell.pack();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected int getShellStyle()
	{
		return SWT.DIALOG_TRIM | SWT.RESIZE | SWT.TOOL;
	}

	/**
	 * @return the selectedEntity
	 */
	public DrawableGameEntity getSelectedEntity()
	{
		return selectedEntity;
	}

	/**
	 * @param selectedEntity
	 *            the selectedEntity to set
	 */
	public void setSelectedEntity(DrawableGameEntity selectedEntity)
	{
		this.selectedEntity = selectedEntity;
		if (this.selectedEntity != null)
		{
			this.resourceIDText.setText(this.selectedEntity.getResourceId());
			this.xSpinner.setSelection(this.selectedEntity.getX());
			this.ySpinner.setSelection(this.selectedEntity.getY());
			if (this.selectedEntity instanceof BaseCharacter)
			{
				BaseCharacter chara = (BaseCharacter) this.selectedEntity;
				this.directionCombo.select(chara.getDirection().ordinal());
			}
			if (this.selectedEntity instanceof EnemyCharacter)
			{
				EnemyCharacter chara = (EnemyCharacter) this.selectedEntity;
				if (chara.getInventory().getMeleeWeapon() != null)
					this.meleeResIdText.setText(chara.getInventory()
							.getMeleeWeapon().getResourceID());
				else
					this.meleeResIdText.setText("");
				if (chara.getInventory().getRangedWeapon() != null)
					this.rangedResIdText.setText(chara.getInventory()
							.getRangedWeapon().getResourceID());
				else
					this.rangedResIdText.setText("");
				this.otherItemResIdText.setText("");
				this.amountOfItemsSpinner.setSelection(0);
				this.healthBaseSpinner.setSelection(chara.getHealthBase());
				this.attackBaseSpinner.setSelection(chara.getAttackBase());
				this.defenseBaseSpinner.setSelection(chara.getDefenseBase());
			}
		}
		updateControlsEnabledState();
	}

	/**
	 * Changes the coordinates of the selected entity in case one is selected
	 * 
	 * @param x
	 *            New x
	 * @param y
	 *            New y
	 */
	public void changeSelectedCoordinates(int x, int y)
	{
		if (this.selectedEntity != null)
		{
			this.xSpinner.setSelection(x);
			this.ySpinner.setSelection(y);
			if (this.selectedEntity instanceof CollidableGameEntity)
			{
				// This will cause collision detection
				CollidableGameEntity e = (CollidableGameEntity) this.selectedEntity;
				e.moveTo(x, y);
			}
			else
			{
				this.selectedEntity.setX(x);
				this.selectedEntity.setY(y);
			}

			// Update dirty state
			MapWindow.setDirty(true);
		}
	}
}
