/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameEditor
 ** 	Drag-On
 ***
 *****************************************************/
package application.windows;

import game.states.map.entity.TriggerZoneEntity;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;

/**
 * This window contains all data about the trigger zones of a map
 * 
 * @author Drag-On
 * 
 */
public class TriggerEditorWindow extends AbstractWindow
{
	/**
	 * Add button
	 */
	private Button addButton;

	/**
	 * Delete button
	 */
	private Button delButton;

	/**
	 * Spinner for x position
	 */
	private Spinner xSpinner;

	/**
	 * Spinner for y position
	 */
	private Spinner ySpinner;

	/**
	 * Spinner for width
	 */
	private Spinner wSpinner;

	/**
	 * Spinner for height
	 */
	private Spinner hSpinner;

	/**
	 * Currently selected trigger entity
	 */
	private TriggerZoneEntity selectedEntity = null;

	/**
	 * Qualified name of the selected or about to be created trigger
	 */
	private Text triggerIdentifier;

	/**
	 * Contains additional data that may be passed to the trigger
	 */
	private Text additionalData;

	/**
	 * Constructor
	 * 
	 * @param parent
	 *            Parent window
	 */
	protected TriggerEditorWindow(Shell parent)
	{
		super(parent);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void createContents(Composite mainComp)
	{
		// Buttons
		final Group buttonGroup = new Group(mainComp, SWT.NONE);
		buttonGroup.setLayout(new GridLayout(2, false));
		buttonGroup.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		buttonGroup.setText("Trigger handles");
		addButton = new Button(buttonGroup, SWT.PUSH);
		addButton.setText("New");
		addButton.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				addNewTrigger();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});
		delButton = new Button(buttonGroup, SWT.PUSH);
		delButton.setText("Delete");
		delButton.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				deleteTrigger();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});
		triggerIdentifier = new Text(buttonGroup, SWT.BORDER);
		triggerIdentifier.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true,
				false, 2, 1));
		Label additionalDataLabel = new Label(buttonGroup, SWT.NONE);
		additionalDataLabel.setText("Add. Data:");
		additionalData = new Text(buttonGroup, SWT.BORDER);
		additionalData.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true,
				false, 1, 1));
		additionalData.addKeyListener(new KeyListener()
		{
			@Override
			public void keyReleased(KeyEvent e)
			{
				if (selectedEntity != null
						&& (e.keyCode == SWT.CR || e.keyCode == SWT.LF))
				{
					selectedEntity.setAdditionalData(additionalData.getText()
							.trim());
					MapWindow.setDirty(true);
				}
			}

			@Override
			public void keyPressed(KeyEvent e)
			{
			}
		});

		// Trigger data
		final Group triggerGroup = new Group(mainComp, SWT.NONE);
		triggerGroup.setLayout(new GridLayout(2, false));
		triggerGroup
				.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		triggerGroup.setText("Trigger data");
		Label labelX = new Label(triggerGroup, SWT.NONE);
		labelX.setText("X:");
		xSpinner = new Spinner(triggerGroup, SWT.BORDER);
		xSpinner.setMinimum(0);
		xSpinner.setMaximum(16000);
		xSpinner.setSelection(32);
		xSpinner.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false, 1,
				1));
		xSpinner.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (TriggerEditorWindow.this.selectedEntity != null)
				{
					TriggerEditorWindow.this.selectedEntity.setX(xSpinner
							.getSelection());
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});
		Label labelY = new Label(triggerGroup, SWT.NONE);
		labelY.setText("Y:");
		ySpinner = new Spinner(triggerGroup, SWT.BORDER);
		ySpinner.setMinimum(0);
		ySpinner.setMaximum(16000);
		ySpinner.setSelection(32);
		ySpinner.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false, 1,
				1));
		ySpinner.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (TriggerEditorWindow.this.selectedEntity != null)
				{
					TriggerEditorWindow.this.selectedEntity.setY(ySpinner
							.getSelection());
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});
		Label labelW = new Label(triggerGroup, SWT.NONE);
		labelW.setText("W:");
		wSpinner = new Spinner(triggerGroup, SWT.BORDER);
		wSpinner.setMinimum(32);
		wSpinner.setMaximum(16000);
		wSpinner.setSelection(32);
		wSpinner.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false, 1,
				1));
		wSpinner.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (TriggerEditorWindow.this.selectedEntity != null)
				{
					TriggerEditorWindow.this.selectedEntity.setWidth(wSpinner
							.getSelection());
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});
		Label labelH = new Label(triggerGroup, SWT.NONE);
		labelH.setText("H:");
		hSpinner = new Spinner(triggerGroup, SWT.BORDER);
		hSpinner.setMinimum(32);
		hSpinner.setMaximum(16000);
		hSpinner.setSelection(32);
		hSpinner.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false, 1,
				1));
		hSpinner.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (TriggerEditorWindow.this.selectedEntity != null)
				{
					TriggerEditorWindow.this.selectedEntity.setHeight(hSpinner
							.getSelection());
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
				widgetSelected(e);
			}
		});

		// Update controls enabled state
		updateControlsEnabledState();
	}

	/**
	 * Creates a new trigger zone and adds it to the map
	 */
	protected void addNewTrigger()
	{
		try
		{
			Class<?> clazz = Class.forName(triggerIdentifier.getText());
			TriggerZoneEntity trigger = (TriggerZoneEntity) clazz
					.getDeclaredConstructors()[0].newInstance(
					xSpinner.getSelection(), ySpinner.getSelection(),
					wSpinner.getSelection(), hSpinner.getSelection());
			MapEditorToolWindow wnd = WindowManager
					.get(MapEditorToolWindow.class);
			trigger.setAdditionalData(this.additionalData.getText().trim()
					.length() > 0 ? this.additionalData.getText().trim() : null);
			wnd.getSelectedMapData().addTriggerZone(trigger);
			this.selectedEntity = trigger;
			updateControlsEnabledState();

			// Update dirty state
			MapWindow.setDirty(true);
		}
		catch (ClassNotFoundException | InstantiationException
				| IllegalAccessException | IllegalArgumentException
				| InvocationTargetException | SecurityException e)
		{
			MessageBox mb = new MessageBox(getShell());
			mb.setMessage("The identifier in the text box is not correct or doesn't belong to a TriggerZoneEntity.");
			mb.setText("Error");
			mb.open();
		}
	}

	/**
	 * Deletes the currently selected trigger if any
	 */
	protected void deleteTrigger()
	{
		if (this.selectedEntity != null)
		{
			MapEditorToolWindow wnd = WindowManager
					.get(MapEditorToolWindow.class);
			wnd.getSelectedMapData().removeTriggerZone(this.selectedEntity);
			this.selectedEntity = null;
			updateControlsEnabledState();

			// Update dirty state
			MapWindow.setDirty(true);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void createControls(Composite mainComp)
	{
		// No controls needed
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected String getTitle()
	{
		return "Trigger zones";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void initSize(Shell shell)
	{
		shell.setSize(150, 285);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected int getShellStyle()
	{
		return SWT.DIALOG_TRIM | SWT.RESIZE | SWT.TOOL;
	}

	/**
	 * This methods updates the enabled states of all controls
	 */
	private void updateControlsEnabledState()
	{
		// Grey out controls if currently no map is selected
		MapEditorToolWindow mapToolWnd = WindowManager
				.get(MapEditorToolWindow.class);
		final boolean enabled = mapToolWnd.getSelectedMap() != null;
		addButton.setEnabled(enabled);
		delButton.setEnabled(enabled);
		xSpinner.setEnabled(enabled && selectedEntity != null);
		ySpinner.setEnabled(enabled && selectedEntity != null);
		wSpinner.setEnabled(enabled && selectedEntity != null);
		hSpinner.setEnabled(enabled && selectedEntity != null);
	}

	/**
	 * @return the selectedEntity
	 */
	public TriggerZoneEntity getSelectedEntity()
	{
		return selectedEntity;
	}

	/**
	 * @param selectedEntity
	 *            the selectedEntity to set
	 */
	public void setSelectedEntity(TriggerZoneEntity selectedEntity)
	{
		this.selectedEntity = selectedEntity;
		if (this.selectedEntity != null)
		{
			this.triggerIdentifier.setText(this.selectedEntity.getClass()
					.getName());
			if (this.selectedEntity.getAdditionalData() != null)
				this.additionalData.setText(this.selectedEntity
						.getAdditionalData());
			else
				this.additionalData.setText("");
			this.xSpinner.setSelection(this.selectedEntity.getX());
			this.ySpinner.setSelection(this.selectedEntity.getY());
			this.wSpinner.setSelection(this.selectedEntity.getWidth());
			this.hSpinner.setSelection(this.selectedEntity.getHeight());
		}
		updateControlsEnabledState();
	}

	/**
	 * Changes the coordinates of the selected entity in case one is selected
	 * 
	 * @param x
	 *            New x
	 * @param y
	 *            New y
	 */
	public void changeSelectedCoordinates(int x, int y)
	{
		if (this.selectedEntity != null)
		{
			this.xSpinner.setSelection(x);
			this.ySpinner.setSelection(y);
			this.selectedEntity.setX(x);
			this.selectedEntity.setY(y);

			// Update dirty state
			MapWindow.setDirty(true);
		}
	}
}
