/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2012
 ***
 *** 
 ** GameEditor
 ** 	Drag-On
 ***
 *****************************************************/
package application.windows;

import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;

import javax.swing.SwingUtilities;

import org.eclipse.swt.widgets.Shell;
import org.newdawn.slick.util.Log;

/**
 * This is a simple window manager. It maintains all open windows and provides
 * access to them. If a window tried to get is not known by the manager, it will
 * be created on the fly.
 * 
 * @author Drag-On
 * 
 */
public class WindowManager
{
	/**
	 * Main window
	 */
	private static MapWindow mainWindow;

	/**
	 * Map of all open windows
	 */
	private static final Map<String, AbstractWindow> windows = new HashMap<String, AbstractWindow>();

	/**
	 * Provides access to the allready opened windows or constructs new ones, in
	 * case none of the passed type has already been openend
	 * 
	 * windowClass Class object of the window to open
	 * 
	 * @return The window fitting the passed type
	 */
	@SuppressWarnings("unchecked")
	public static <T extends AbstractWindow> T get(
			final Class<? extends AbstractWindow> windowClass)
	{
		if (windows.containsKey(windowClass.getName()))
			return (T) windows.get(windowClass.getName());
		else
		{
			final T wnd = createWindow(windowClass, mainWindow.getShell());
			windows.put(windowClass.getName(), wnd);
			return wnd;
		}
	}

	/**
	 * Checks, if the window of the passed type is currently open
	 * 
	 * @param windowClass
	 *            Window to check
	 * @return <code>true</code> if window is currently open, otherwise
	 *         <code>false</code>
	 */
	public static boolean isOpen(
			final Class<? extends AbstractWindow> windowClass)
	{
		return windows.containsKey(windowClass.getName());
	}

	/**
	 * Closes the window of the passed type if one is open
	 * 
	 * @param windowClass
	 *            Class type of the window to close
	 */
	public static void close(final Class<? extends AbstractWindow> windowClass)
	{
		if (windows.containsKey(windowClass.getName()))
		{
			final AbstractWindow wnd = windows.get(windowClass.getName());
			if (!wnd.getShell().isDisposed())
				// wnd.getShell().close();
				wnd.getShell().dispose();
			windows.remove(windowClass.getName());
		}
	}

	/**
	 * Creates a new instance of the window with the passed class type.
	 * 
	 * @param windowClass
	 *            Class object of the window to construct
	 * @param parent
	 *            Parent shell or <code>null</code> if no parent
	 * @return The freshly created window or <code>null</code> if something went
	 *         wrong.
	 */
	@SuppressWarnings("unchecked")
	private static <T extends AbstractWindow> T createWindow(
			final Class<? extends AbstractWindow> windowClass,
			final Shell parent)
	{
		try
		{
			return (T) windowClass.getDeclaredConstructor(Shell.class)
					.newInstance(parent);
		}
		catch (final InstantiationException | IllegalAccessException
				| IllegalArgumentException | InvocationTargetException
				| NoSuchMethodException | SecurityException e)
		{
			SwingUtilities.invokeLater(new Runnable()
			{
				@Override
				public void run()
				{
					Log.error(
							"Unable to instanciate window "
									+ windowClass.getName() + ".", e);
				}
			});
		}
		return null;
	}

	/**
	 * @param mainWindow
	 *            the main window to set
	 */
	public static void setMainWindow(final MapWindow mainWindow)
	{
		WindowManager.mainWindow = mainWindow;
	}

	/**
	 * @return The main window
	 */
	public static MapWindow getMainWindow()
	{
		return mainWindow;
	}
}
