/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2012
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package application.event;

import java.util.EventObject;

/**
 * This class describes all kinds of mouse events
 * 
 * @author Drag-On
 * 
 */
public class MouseEvent extends EventObject
{
	/**
	 * Serial Version UID
	 */
	private static final long serialVersionUID = 6953841271233457184L;

	/**
	 * Button, that has been pressed, starting with 0
	 */
	private int button = -1;

	/**
	 * Mouse cursor x coordinate
	 */
	private int x = -1;

	/**
	 * Mouse cursor y coordinate
	 */
	private int y = -1;

	/**
	 * Old mouse cursor x coordinate
	 */
	private int oldX = -1;

	/**
	 * Old mouse cursor y coordinate
	 */
	private int oldY = -1;

	/**
	 * Amount of clicks
	 */
	private int clickCount = 0;

	/**
	 * How far the wheel was crolled
	 */
	private int wheelChange = 0;

	/**
	 * Constructor
	 * 
	 * @param source
	 *            Initial source object
	 * @param x
	 *            X coordinate
	 * @param y
	 *            Y coordinate
	 */
	public MouseEvent(final Object source, final int x, final int y)
	{
		super(source);
		this.x = x;
		this.y = y;
	}

	/**
	 * Constructor
	 * 
	 * @param source
	 *            Initial source object
	 * @param x
	 *            X coordinate
	 * @param y
	 *            Y coordinate
	 * @param button
	 *            Mouse button that has been pressed
	 * @param clickCount
	 *            Amount of clicks
	 */
	public MouseEvent(final Object source, final int x, final int y,
			final int button, final int clickCount)
	{
		super(source);
		this.x = x;
		this.y = y;
		this.button = button;
		this.clickCount = clickCount;
	}

	/**
	 * Constructor
	 * 
	 * @param source
	 *            Initial source object
	 * @param x
	 *            X coordinate
	 * @param y
	 *            Y coordinate
	 * @param oldX
	 *            Old x coordinate
	 * @param oldY
	 *            oldY coordinate
	 * @param button
	 *            Mouse button that has been pressed
	 */
	public MouseEvent(final Object source, final int x, final int y,
			final int oldX, final int oldY, final int button)
	{
		super(source);
		this.x = x;
		this.y = y;
		this.oldX = oldX;
		this.oldY = oldY;
		this.button = button;
	}

	/**
	 * Constructor
	 * 
	 * @param source
	 *            Initial source object
	 * @param wheelChange
	 *            How far the wheel has been scrolled
	 */
	public MouseEvent(final Object source, final int wheelChange)
	{
		super(source);
		this.wheelChange = wheelChange;
	}

	/**
	 * @return the button
	 */
	public int getButton()
	{
		return button;
	}

	/**
	 * @return the x
	 */
	public int getX()
	{
		return x;
	}

	/**
	 * @return the y
	 */
	public int getY()
	{
		return y;
	}

	/**
	 * @return the oldX
	 */
	public int getOldX()
	{
		return oldX;
	}

	/**
	 * @return the oldY
	 */
	public int getOldY()
	{
		return oldY;
	}

	/**
	 * @return the clickCount
	 */
	public int getClickCount()
	{
		return clickCount;
	}

	/**
	 * @return the wheelChange
	 */
	public int getWheelChange()
	{
		return wheelChange;
	}
}
