/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2012
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package application.exception.handler;

import javax.swing.JOptionPane;

import org.newdawn.slick.util.LogSystem;

/**
 * This class serves as an exception handler
 * 
 * @author Drag-On
 * 
 */
public class ErrorHandler implements LogSystem
{
	/**
	 * Constructor
	 * 
	 * @param logLevel
	 *            Log level to use
	 */
	public ErrorHandler(final SeverityLevel logLevel)
	{
		Logfile.getInstance().setLogLevel(logLevel);
	}

	/**
	 * Shows a simple message box
	 * 
	 * @param title
	 *            Messagebox title
	 * @param msg
	 *            Message
	 */
	public static void showMessageBox(final String title, final String msg)
	{
		JOptionPane.showMessageDialog(null, msg, title,
				JOptionPane.INFORMATION_MESSAGE);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void error(final String message, final Throwable e)
	{
		if (message != null)
			System.out.println(message);
		Logfile.getInstance().log(SeverityLevel.ERROR, e, message);

		String msg = message;
		if (e != null)
		{
			msg += "\n" + e.getLocalizedMessage();
			e.printStackTrace();
		}

		if (message != null)
			JOptionPane.showMessageDialog(null, msg, "Exception occurred",
					JOptionPane.ERROR_MESSAGE);

		System.exit(0);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void error(final Throwable e)
	{
		error(null, e);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void error(final String message)
	{
		error(message, null);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void warn(final String message)
	{
		warn(message, null);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void warn(final String message, final Throwable e)
	{
		System.out.println(message);
		if (e != null)
			e.printStackTrace();
		Logfile.getInstance().log(SeverityLevel.WARNING, e, message);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void info(final String message)
	{
		Logfile.getInstance().log(SeverityLevel.INFO, message);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void debug(final String message)
	{
		Logfile.getInstance().log(SeverityLevel.DEBUG, message);
	}
}
