/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2012
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package application.properties;

import java.io.FileInputStream;
import java.io.IOException;
import java.lang.reflect.Field;

import org.lwjgl.util.Point;
import org.newdawn.slick.Input;
import org.newdawn.slick.util.Log;

import application.exception.handler.SeverityLevel;

/**
 * This class holds all game specific properties
 * 
 * @author Drag-On
 * 
 */
public class Properties
{
	/*****************************************************************/
	/*********************** Render properties ***********************/
	/*****************************************************************/

	/**
	 * Amount of multisampling passes
	 */
	public static Integer multiSampling = 0;

	/**
	 * Fullscreen flag
	 */
	public static Boolean fullscreen = false;

	/**
	 * Toggles vertical synchronisation
	 */
	public static Boolean vSync = false;

	/**
	 * Enables smoothing of time between frames.<br>
	 * Useful when the frame rate gets reduced a lot leading to rounding issues
	 * with delta values.<br>
	 * Should most likely be on when using vSync.
	 */
	public static Boolean smoothDeltas = false;

	/**
	 * Attempts to limit frame rate to this value. Negative values indicate no
	 * limit.
	 */
	public static Integer targetFrameRate = -1;

	/**
	 * Minimum amount of time that has to pass until update is called again
	 */
	public static Integer minimumLogicUpdateInterval = 1;

	/**
	 * Screen resolution: width
	 */
	public static Integer screenWidth = 800;

	/**
	 * Screen resolution: height
	 */
	public static Integer screenHeight = 600;

	/**
	 * Flag determining if the game should try to use shaders.
	 */
	public static Boolean useShaders = true;

	/**
	 * Flag determining if the FPS should be drawn
	 */
	public static Boolean showFPS = false;

	/**
	 * Flag indicating that the game window is rendered even if it's not the
	 * focus window
	 */
	public static Boolean alwaysRender = false;

	/*****************************************************************/
	/************************* Map constants *************************/
	/*****************************************************************/

	/**
	 * Amount of pixels in a tile in each direction
	 */
	public static Integer tileSize = 32;

	/*****************************************************************/
	/********************** Gameplay properties **********************/
	/*****************************************************************/

	/**
	 * ID of the map to start game with
	 */
	public static Integer startMap = 0;

	/**
	 * Player coordinates to start game with in tiles
	 */
	public static Point startPos = new Point(1, 1);

	/**
	 * Language to start the game in
	 */
	public static String language = "de";

	/**
	 * In immersive mode some things such as the HUD are not displayed
	 */
	public static Boolean immersiveMode = false;

	/**
	 * Volume to play audio at
	 */
	public static Float audioVolume = 1.0f;

	/**
	 * Flag indicating if the survey should be opened in web browser on exit
	 */
	public static Boolean showSurveyOnExit = true;

	/**
	 * URL of the survey to open
	 */
	public static String surveyURL = "www.survey.rpg-projekt.de";

	/*****************************************************************/
	/***************************** Paths *****************************/
	/*****************************************************************/

	/**
	 * Folder, where maps are stored
	 */
	public static String mapFolder = "Maps";

	/**
	 * Folder, where maps are stored
	 */
	public static String saveGameFolder = "SaveGames";

	/**
	 * Base path, where resources are supposed to be. Relative to run folder.
	 */
	public static String resourceBasePath = "";

	/*****************************************************************/
	/************************ Debug properties ***********************/
	/*****************************************************************/

	/**
	 * Defines how much the logfile is allowed to print out
	 */
	public static SeverityLevel loggingLevel = SeverityLevel.INFO;

	/**
	 * Enables / disables debug mode
	 */
	public static Boolean debugMode = false;

	/*****************************************************************/
	/*********************** Editor properties ***********************/
	/*****************************************************************/

	/**
	 * Must be enabled when game is running in editor
	 */
	public static Boolean editor = false;

	/**
	 * Amount of steps that can be undone
	 */
	public static Integer historySize = 20;

	/*****************************************************************/
	/************************* Key assignment ************************/
	/*****************************************************************/

	/**
	 * Key to exit windows, go to menu etc.
	 */
	public static Integer keyReturn = Input.KEY_ESCAPE;

	/**
	 * Key to take a screenshot
	 */
	public static Integer keyScreenshot = Input.KEY_SYSRQ;

	/**
	 * Key to pause the game
	 */
	public static Integer keyPause = Input.KEY_PAUSE;

	/**
	 * Key for moving upwards
	 */
	public static Integer keyMoveUp = Input.KEY_W;

	/**
	 * Key for moving left
	 */
	public static Integer keyMoveLeft = Input.KEY_A;

	/**
	 * Key for moving down
	 */
	public static Integer keyMoveDown = Input.KEY_S;

	/**
	 * Key for moving right
	 */
	public static Integer keyMoveRight = Input.KEY_D;

	/**
	 * Key to trigger actions
	 */
	public static Integer keyAction = Input.KEY_SPACE;

	/**
	 * Key to trigger ranged attacks
	 */
	public static Integer keyAttackRanged = Input.MOUSE_LEFT_BUTTON;

	/**
	 * Key to trigger melee attacks
	 */
	public static Integer keyAttackMelee = Input.MOUSE_RIGHT_BUTTON;

	/**
	 * Key to activate questlog
	 */
	public static Integer keyQuestlog = Input.KEY_L;

	/**
	 * Key to open inventory
	 */
	public static Integer keyInventory = Input.KEY_TAB;

	/**
	 * Key to open inventory
	 */
	public static Integer keyCharacterStats = Input.KEY_C;

	/**
	 * Parses the properties file
	 */
	public static void parseProperties()
	{
		// Read properties file
		final java.util.Properties properties = new java.util.Properties();
		try
		{
			properties.load(new FileInputStream("Properties.txt"));
		}
		catch (final IOException e)
		{
			Log.error("Error while reading properties from file.", e);
		}

		Class<Properties> propClazz = Properties.class;
		Field[] fields = propClazz.getDeclaredFields();
		for (Field f : fields)
		{
			// read out the properties
			String fieldName = f.getName();
			if (properties.containsKey(fieldName))
			{
				assignValue(f, properties.getProperty(fieldName));
			}
		}
	}

	/**
	 * Parses the arguments passed to the game
	 * 
	 * @param args
	 *            The arguments to parse
	 */
	public static void parseArguments(final String[] args)
	{
		final Class<Properties> aClass = Properties.class;
		Field field = null;

		for (String s : args)
		{
			if (s.startsWith("-"))
			{
				s = s.substring(1);
				try
				{
					field = aClass.getField(s);
				}
				catch (final NoSuchFieldException e)
				{
					Log.error("Error while parsing arguments.", e);
				}
				catch (final SecurityException e)
				{
					Log.error("Error while parsing arguments.", e);
				}
			}
			else if (field != null)
			{
				assignValue(field, s);
			}
		}
	}

	/**
	 * Assigns the passed value to the earlier passed field via reflections
	 * 
	 * @param field
	 *            Field to edit
	 * @param value
	 *            New value in it's string representation
	 */
	private static void assignValue(final Field field, final String value)
	{
		try
		{
			if (field.getType().equals(Integer.class))
			{
				field.set(Properties.class, Integer.parseInt(value));
			}
			else if (field.getType().equals(Boolean.class))
			{
				field.set(Properties.class, Boolean.parseBoolean(value));
			}
			else if (field.getType().equals(String.class))
			{
				field.set(Properties.class, value);
			}
			else if (field.getType().equals(Float.class))
			{
				field.set(Properties.class, Float.parseFloat(value));
			}
			else if (field.getType().equals(SeverityLevel.class))
			{
				field.set(Properties.class, SeverityLevel.valueOf(value));
			}
			else if (field.getType().equals(Point.class))
			{
				String coords[] = value.split(",");
				field.set(Properties.class, new Point(
						Integer.decode(coords[0]), Integer.decode(coords[1])));
			}
		}
		catch (final NumberFormatException e)
		{
			Log.error("Error while parsing arguments.", e);
		}
		catch (final IllegalArgumentException e)
		{
			Log.error("Error while parsing arguments.", e);
		}
		catch (final IllegalAccessException e)
		{
			Log.error("Error while parsing arguments.", e);
		}

	}
}
