/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package data.geometry;

/**
 * This class represents a point without extension in 3D space
 * 
 * @author Drag-On
 * 
 */
public class Point3D
{
	/**
	 * Point coordinates
	 */
	private int x, y, z;

	/**
	 * Constructor<br>
	 * Initializes the point with zeros.
	 */
	public Point3D()
	{
		this(0, 0, 0);
	}

	/**
	 * Constructor
	 * 
	 * @param x
	 *            X coordinate
	 * @param y
	 *            Y coordinate
	 * @param z
	 *            Z coordinate
	 */
	public Point3D(final int x, final int y, final int z)
	{
		this.x = x;
		this.y = y;
		this.z = z;
	}

	/**
	 * Constructor
	 * 
	 * @param p
	 *            Point to copy into this point
	 */
	public Point3D(final Point3D p)
	{
		this.x = p.x;
		this.y = p.y;
		this.z = p.z;
	}

	/**
	 * @return the x
	 */
	public int getX()
	{
		return x;
	}

	/**
	 * @param x
	 *            the x to set
	 */
	public void setX(final int x)
	{
		this.x = x;
	}

	/**
	 * @return the y
	 */
	public int getY()
	{
		return y;
	}

	/**
	 * @param y
	 *            the y to set
	 */
	public void setY(final int y)
	{
		this.y = y;
	}

	/**
	 * @return the z
	 */
	public int getZ()
	{
		return z;
	}

	/**
	 * @param z
	 *            the z to set
	 */
	public void setZ(final int z)
	{
		this.z = z;
	}

	/**
	 * Checks if two Point3Ds are equal in terms of their content coordinates
	 * {@inheritDoc}
	 */
	@Override
	public boolean equals(final Object obj)
	{
		if (this == obj)
			return true;
		if (obj instanceof Point3D)
		{
			Point3D pObj = (Point3D) obj;
			return (this.x == pObj.x && this.y == pObj.y && this.z == pObj.z);
		}
		return false;
	}

	/**
	 * Calculates a unique hash code for this Point3D based on the coordinate
	 * values {@inheritDoc}
	 */
	@Override
	public int hashCode()
	{
		int hash = 42; // Some starting point; 42 is always a good choice ;)
		int multiplier = 59; // Small prime number

		// hash = hash * multiplier + super.hashCode();
		hash = hash * multiplier + x;
		hash = hash * multiplier + y;
		hash = hash * multiplier + z;

		return hash;
	}
}
