/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2012
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.managers;

import org.newdawn.slick.SlickException;

/**
 * This class is the generic representation of a game resource. All specific
 * resources need to subclass this so they can be managed by the
 * {@link ResourceManager}
 * 
 * @author Drag-On
 * 
 */
public abstract class Resource
{
	/**
	 * Path to the file this resource represents
	 */
	private final String path;

	/**
	 * Constructor
	 * 
	 * @param path
	 *            Path to the resource
	 */
	public Resource(final String path)
	{
		this.path = path;
	}

	/**
	 * Copy-Constructor
	 * 
	 * @param other
	 *            Some other resource of the same type
	 */
	public Resource(final Resource other)
	{
		this.path = other.path;
	}

	/**
	 * @return the path
	 */
	public String getPath()
	{
		return path;
	}

	/**
	 * Checks if this resource has already been loaded
	 * 
	 * @return <code>true</code> in case this resource has already been loaded,
	 *         otherwise <code>false</code>
	 */
	public abstract boolean isLoaded();

	/**
	 * Actually loads this resource into memory.
	 */
	public abstract void load() throws SlickException;

	/**
	 * Frees all native memory used by this resource.
	 */
	public abstract void unload() throws SlickException;

	/**
	 * Copies all data from some other resource to this one. This does not
	 * include native resources!
	 * 
	 * @param other
	 *            Resource to copy data from
	 */
	public abstract void copyData(Resource other);
}
