/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2012
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.resources;

import game.managers.Resource;

import org.newdawn.slick.Font;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.UnicodeFont;
import org.newdawn.slick.font.effects.ColorEffect;

/**
 * This resource represents a TrueTypeFont
 * 
 * @author Drag-On
 * 
 */
public class FontResource extends Resource
{
	/**
	 * Slick font resource
	 */
	private UnicodeFont font;

	/**
	 * Slick font resource bold
	 */
	private UnicodeFont fontBold;

	/**
	 * Slick font resource italic
	 */
	private UnicodeFont fontItalic;

	/**
	 * Slick font resource bold italic
	 */
	private UnicodeFont fontBoldItalic;

	/**
	 * Font size
	 */
	private int size;

	/**
	 * Constructor
	 * 
	 * @param name
	 *            Font name
	 */
	public FontResource(final String name, final int size)
	{
		super(name);
		this.size = size;
	}

	/**
	 * Copy-Constructor
	 * 
	 * @param other
	 *            Some other resource of the same type
	 */
	public FontResource(final FontResource other)
	{
		super(other);
		this.size = other.size;
		this.font = other.font;
		this.fontBold = other.fontBold;
		this.fontBoldItalic = other.fontBoldItalic;
		this.fontItalic = other.fontItalic;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean isLoaded()
	{
		return font != null;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void load() throws SlickException
	{
		font = new UnicodeFont(getPath(), size, false, false);
		font.addAsciiGlyphs();
		font.addGlyphs("äöüÄÖÜß");
		font.getEffects().add(new ColorEffect(java.awt.Color.WHITE));
		font.loadGlyphs();
		fontBold = new UnicodeFont(getPath(), size, true, false);
		fontBold.addAsciiGlyphs();
		fontBold.addGlyphs("äöüÄÖÜß");
		fontBold.getEffects().add(new ColorEffect(java.awt.Color.WHITE));
		fontBold.loadGlyphs();
		fontItalic = new UnicodeFont(getPath(), size, false, true);
		fontItalic.addAsciiGlyphs();
		fontItalic.addGlyphs("äöüÄÖÜß");
		fontItalic.getEffects().add(new ColorEffect(java.awt.Color.WHITE));
		fontItalic.loadGlyphs();
		fontBoldItalic = new UnicodeFont(getPath(), size, true, true);
		fontBoldItalic.addAsciiGlyphs();
		fontBoldItalic.addGlyphs("äöüÄÖÜß");
		fontBoldItalic.getEffects().add(new ColorEffect(java.awt.Color.WHITE));
		fontBoldItalic.loadGlyphs();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void unload() throws SlickException
	{
		if (isLoaded())
		{
			font.destroy();
			fontBold.destroy();
			fontBoldItalic.destroy();
			fontItalic.destroy();
			font = fontBold = fontBoldItalic = fontItalic = null;
		}
	}

	/**
	 * @return The font in the chosen typeface
	 */
	public Font getFont(final boolean bold, final boolean italic)
	{
		if (bold && italic)
			return fontBoldItalic;
		else if (bold)
			return fontBold;
		else if (italic)
			return fontItalic;
		else
			return font;
	}

	/**
	 * @return the size
	 */
	public int getSize()
	{
		return size;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void copyData(final Resource other)
	{
		if (other instanceof FontResource)
			this.size = ((FontResource) other).size;
	}
}
