/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		 \u00a9 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.resources;

import game.managers.Resource;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

import org.newdawn.slick.SlickException;
import org.newdawn.slick.util.Log;

import application.properties.Properties;

/**
 * @author Drag-On
 * 
 */
public class InventoryItemResource extends Resource
{
	/**
	 * File name
	 */
	private String fileName;

	/**
	 * Properties object
	 */
	private java.util.Properties properties;

	/**
	 * Constructor
	 * 
	 * @param path
	 *            Path to the file
	 * @param fileName
	 *            Filename
	 */
	public InventoryItemResource(final String path, final String fileName)
	{
		super(path);
		this.fileName = fileName;
	}

	/**
	 * Copy constructor
	 * 
	 * @param other
	 *            Other instance to copy
	 */
	public InventoryItemResource(final InventoryItemResource other)
	{
		super(other);
		this.fileName = other.fileName;
		this.properties = other.properties;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean isLoaded()
	{
		return this.properties != null;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void load() throws SlickException
	{
		properties = new java.util.Properties();
		try
		{
			FileInputStream stream = new FileInputStream(getPath()
					+ File.separator + Properties.language + File.separator
					+ "item" + File.separator + this.fileName);
			properties.load(stream);
			stream.close();
		}
		catch (final IOException e)
		{
			Log.error("Error while reading item properties from file.", e);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void unload() throws SlickException
	{
		// No native resources to free
	}

	/**
	 * @return the name
	 */
	public final String getName()
	{
		return this.properties.getProperty("name", "");
	}

	/**
	 * @return the description
	 */
	public final String getDescription()
	{
		return this.properties.getProperty("description", "");
	}

	/**
	 * @return the type
	 */
	public final String getType()
	{
		return this.properties.getProperty("type", "");
	}

	/**
	 * Provides access to type-specific parameters
	 * 
	 * @param param
	 *            Parameter name
	 * @return The parameter with the specified name in it's string
	 *         representation
	 */
	public final String getParameter(final String param)
	{
		return this.properties.getProperty(param, "");
	}

	/**
	 * Provides access to type-specific integer parameters
	 * 
	 * @param param
	 *            Parameter name
	 * @return The parameter value with the specified name or 0 in case the
	 *         requested key is not specified
	 * @throws NumberFormatException
	 *             In case the parameter with the specified name can not be read
	 *             as a integer
	 */
	public final int getParameterAsInt(final String param)
			throws NumberFormatException
	{
		if (this.properties.containsKey(param))
			return Integer.parseInt(this.properties.getProperty(param, "0"));
		else
			return 0;
	}

	/**
	 * @return <code>true</code> in case this is a consumable item, otherwise
	 *         <code>false</code>
	 */
	public boolean isConsumable()
	{
		String type = getType().replace("$", "");
		return type.equals("consumable");
	}

	/**
	 * @return <code>true</code> in case this is a equippable item, otherwise
	 *         <code>false</code>
	 */
	public boolean isEquippable()
	{
		String type = getType().replace("$", "");
		return type.equals("meleeWeapon") || type.equals("rangedWeapon")
				|| type.equals("armor");
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void copyData(final Resource other)
	{
		if (other instanceof InventoryItemResource)
		{
			this.fileName = ((InventoryItemResource) other).fileName;
			this.properties = ((InventoryItemResource) other).properties;
		}
	}
}
