/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.resources;

import game.managers.Resource;

import java.io.File;

import org.newdawn.slick.SlickException;
import org.newdawn.slick.opengl.shader.ShaderProgram;
import org.newdawn.slick.util.Log;

/**
 * This resource represents a shader program consisting of a vertex- and a
 * fragment-shader ("pixel-shader")
 * 
 * @author Drag-On
 * 
 */
public class ShaderResource extends Resource
{
	/**
	 * File name of the vertex shader to use
	 */
	private String vertexShaderFile;

	/**
	 * File name of the fragement shader to use
	 */
	private String fragmentShaderFile;

	/**
	 * Shader object
	 */
	private ShaderProgram shader;

	/**
	 * Constructor
	 * 
	 * @param path
	 *            Path of the folder the shaders are in
	 * @param vertexShaderName
	 *            File name of the vertex shader to use
	 * @param fragmentShaderName
	 *            File name of the fragment shader to use
	 */
	public ShaderResource(final String path, final String vertexShaderName,
			final String fragmentShaderName)
	{
		super(path);
		this.vertexShaderFile = vertexShaderName;
		this.fragmentShaderFile = fragmentShaderName;
	}

	/**
	 * Copy-Constructor
	 * 
	 * @param other
	 *            Some other resource of the same type
	 */
	public ShaderResource(final ShaderResource other)
	{
		super(other);
		this.shader = other.shader;
		this.fragmentShaderFile = other.fragmentShaderFile;
		this.vertexShaderFile = other.vertexShaderFile;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean isLoaded()
	{
		return this.shader != null;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void load() throws SlickException
	{
		// At first check if shaders are supported
		if (!ShaderProgram.isSupported())
		{
			Log.warn("Shaders are not supported on this machine. The game will run anyway but it won't look as nice as it could.");
			return;
		}
		try
		{
			this.shader = ShaderProgram.loadProgram(getPath() + File.separator
					+ vertexShaderFile, getPath() + File.separator
					+ fragmentShaderFile);
		}
		catch (SlickException e)
		{
			Log.error("Something went wrong while loading the shader "
					+ vertexShaderFile + " and " + fragmentShaderFile + ".", e);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void unload() throws SlickException
	{
		if (isLoaded())
			shader.release();
		shader = null;
	}

	/**
	 * @return the shader
	 */
	public ShaderProgram getShader()
	{
		return shader;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void copyData(final Resource other)
	{
		if (other instanceof QuestResource)
		{
			this.fragmentShaderFile = ((ShaderResource) other).fragmentShaderFile;
			this.vertexShaderFile = ((ShaderResource) other).vertexShaderFile;
		}
	}
}
