/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2012
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.resources;

import game.managers.Resource;

import org.newdawn.slick.Image;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.SpriteSheet;

/**
 * This resource represents a GUI skin
 * 
 * @author Drag-On
 * 
 */
public class SkinResource extends Resource
{
	/**
	 * Spritesheet used for this skin
	 */
	private SpriteSheet spriteSheet = null;

	/**
	 * Constructor
	 * 
	 * @param path
	 *            Path to the skin's file on hard disk
	 */
	public SkinResource(final String path)
	{
		super(path);
	}

	/**
	 * Copy-Constructor
	 * 
	 * @param other
	 *            Some other resource of the same type
	 */
	public SkinResource(final SkinResource other)
	{
		super(other);
		this.spriteSheet = other.spriteSheet;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean isLoaded()
	{
		return spriteSheet != null;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void load() throws SlickException
	{
		final Image img = new Image(getPath());
		this.spriteSheet = new SpriteSheet(img, 32, 32);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void unload() throws SlickException
	{
		if (isLoaded())
			spriteSheet.destroy();
		spriteSheet = null;
	}

	/**
	 * @return The used spriteSheet
	 */
	public SpriteSheet getSpriteSheet()
	{
		return spriteSheet;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void copyData(final Resource other)
	{
	}
}
