/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.resources;

import game.managers.Resource;

import java.io.IOException;
import java.lang.reflect.Field;

import org.newdawn.slick.SlickException;
import org.newdawn.slick.Sound;
import org.newdawn.slick.loading.LoadingList;
import org.newdawn.slick.openal.DeferredSound;
import org.newdawn.slick.util.Log;

/**
 * A resource representing a sound
 * 
 * @author Drag-On
 * 
 */
public class SoundResource extends Resource
{
	/**
	 * Actual sound handle
	 */
	private Sound sound;

	/**
	 * Constructor
	 * 
	 * @param path
	 *            Path to the resource file
	 */
	public SoundResource(final String path)
	{
		super(path);
	}

	/**
	 * Copy-Constructor
	 * 
	 * @param other
	 *            Some other resource of the same type
	 */
	public SoundResource(final SoundResource other)
	{
		super(other);
		this.sound = other.sound;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean isLoaded()
	{
		return this.sound != null;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void load() throws SlickException
	{
		this.sound = new Sound(getPath());

		// Slick won't load deferred sounds by itself (bug?) so we need this
		// nasty hack to do it manually
		if (LoadingList.isDeferredLoading())
		{
			Class<Sound> clazz = Sound.class;
			try
			{
				Field soundField = clazz.getDeclaredField("sound");
				soundField.setAccessible(true);
				DeferredSound snd = (DeferredSound) soundField.get(this.sound);
				snd.load();
			}
			catch (NoSuchFieldException | SecurityException
					| IllegalArgumentException | IllegalAccessException
					| IOException e)
			{
				Log.error(
						"Unable to access private field of sound "
								+ this.getPath(), e);
			}
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void unload() throws SlickException
	{
		if (isLoaded())
			sound.release();
		sound = null;
	}

	/**
	 * @return the sound
	 */
	public Sound getSound()
	{
		return sound;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void copyData(final Resource other)
	{
	}
}
