/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2012
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.resources;

import game.managers.Resource;

import org.newdawn.slick.Image;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.SpriteSheet;

/**
 * A general spritesheet resource which can be used for multiple things
 * 
 * @author Drag-On
 * 
 */
public class SpritesheetResource extends Resource
{
	/**
	 * Spritesheet
	 */
	private SpriteSheet spriteSheet = null;

	/**
	 * Amount of horizontal tiles
	 */
	private int width = 4;

	/**
	 * Amount of vertical tiles
	 */
	private int height = 4;

	/**
	 * Constructor
	 * 
	 * @param path
	 *            Path to the resource
	 * @param width
	 *            Amount of horizontal tiles
	 * @param height
	 *            Amount of vertical tiles
	 */
	public SpritesheetResource(final String path, final int width,
			final int height)
	{
		super(path);
		this.width = width;
		this.height = height;
	}

	/**
	 * Copy-Constructor
	 * 
	 * @param other
	 *            Some other resource of the same type
	 */
	public SpritesheetResource(final SpritesheetResource other)
	{
		super(other);
		this.width = other.getWidth();
		this.height = other.getHeight();
		this.spriteSheet = other.spriteSheet;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean isLoaded()
	{
		return spriteSheet != null;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void load() throws SlickException
	{
		final Image img = new Image(getPath());
		final int width = img.getWidth() / getWidth();
		final int height = img.getHeight() / getHeight();

		this.spriteSheet = new SpriteSheet(img, width, height);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void unload() throws SlickException
	{
		if (isLoaded())
			this.spriteSheet.destroy();
		this.spriteSheet = null;
	}

	/**
	 * @return The amount of frames of this character sheet
	 */
	public int getWidth()
	{
		return width;
	}

	/**
	 * @return The amount of directions of this character sheet
	 */
	public int getHeight()
	{
		return height;
	}

	/**
	 * Calculates the width of one tile
	 * 
	 * @return The width of one tile in pixels if the spritesheet has already
	 *         been loaded, otherwise -1
	 */
	public int getTileWidth()
	{
		return spriteSheet != null ? spriteSheet.getWidth() / getWidth() : -1;
	}

	/**
	 * Calculates the height of one tile
	 * 
	 * @return the height of one tile in pixels if the spritesheet has already
	 *         been loaded, otherwise -1
	 */
	public int getTileHeight()
	{
		return spriteSheet != null ? spriteSheet.getHeight() / getHeight() : -1;
	}

	/**
	 * @return The used spriteSheet
	 */
	public SpriteSheet getSpriteSheet()
	{
		return spriteSheet;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void copyData(final Resource other)
	{
	}
}
