/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2012
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.resources;

import game.managers.Resource;

import org.newdawn.slick.Image;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.SpriteSheet;

import application.properties.Properties;

/**
 * This class respresents a tileset resource
 * 
 * @author Drag-On
 * 
 */
public class TilesetResource extends Resource
{
	/**
	 * The actual spritesheet for this tileset
	 */
	private SpriteSheet spriteSheet = null;

	/**
	 * Information about the tiles of this tileset
	 */
	private TileData[][] tileMetaData;

	/**
	 * Constructor
	 * 
	 * @param path
	 *            Path to the resource
	 * @param tileMetaData
	 *            Meta information about the tiles on this tileset
	 */
	public TilesetResource(final String path, final TileData[][] tileMetaData)
	{
		super(path);
		this.tileMetaData = tileMetaData;
	}

	/**
	 * Copy-Constructor
	 * 
	 * @param other
	 *            Some other resource of the same type
	 */
	public TilesetResource(final TilesetResource other)
	{
		super(other);
		this.spriteSheet = other.spriteSheet;
		this.tileMetaData = other.tileMetaData;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean isLoaded()
	{
		return spriteSheet != null;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void load() throws SlickException
	{
		final Image img = new Image(getPath());
		this.spriteSheet = new SpriteSheet(img, Properties.tileSize,
				Properties.tileSize);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void unload() throws SlickException
	{
		if (isLoaded())
			spriteSheet.destroy();
		spriteSheet = null;
	}

	/**
	 * @return the tileMetaData
	 */
	public TileData[][] getTileData()
	{
		return tileMetaData;
	}

	/**
	 * @return The used spriteSheet
	 */
	public SpriteSheet getSpriteSheet()
	{
		return spriteSheet;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void copyData(final Resource other)
	{
		if (other instanceof TilesetResource)
		{
			this.tileMetaData = ((TilesetResource) other).tileMetaData;
		}
	}
}
