/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		 \u00a9 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.save;

import game.states.map.entity.DrawableGameEntity;
import game.states.variables.GameVariables;

import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * The data that is written to harddisk in case the user decided to save the
 * game
 * 
 * @author Drag-On
 * 
 */
public class SaveGameData implements Serializable, Comparable<SaveGameData>
{
	/**
	 * Serial Version UID
	 */
	private static final long serialVersionUID = 8951722009116582957L;

	/**
	 * The ID of the map the player was on
	 */
	private int mapID;

	/**
	 * The name of the map the player was on
	 */
	private String mapName;

	/**
	 * The time of day when the user saved the game
	 */
	private Date time;

	/**
	 * State of game variables
	 */
	private GameVariables gameVariables;

	/**
	 * The game entities in their last state
	 */
	private final Map<Integer, Set<DrawableGameEntity>> entities;

	/**
	 * Constructor
	 * 
	 * @param mapID
	 *            ID of the map the player was on
	 * @param mapName
	 *            The name of the map the player was on
	 * @param gameVariables
	 *            State of game variables
	 * @param entities
	 *            The game entities in their last state
	 */
	public SaveGameData(final int mapID, final String mapName,
			final GameVariables gameVariables,
			final Set<DrawableGameEntity> entities)
	{
		this.entities = new HashMap<Integer, Set<DrawableGameEntity>>();
		update(mapID, mapName, gameVariables, entities);
	}

	/**
	 * Updates the save state (i.e. on map change)
	 * 
	 * @param mapID
	 *            ID of the map the player was on
	 * @param mapName
	 *            The name of the map the player was on
	 * @param gameVariables
	 *            State of game variables
	 * @param entities
	 *            The game entities in their last state
	 */
	public void update(final int mapID, final String mapName,
			final GameVariables gameVariables,
			final Set<DrawableGameEntity> entities)
	{
		this.mapID = mapID;
		this.mapName = mapName;
		this.time = new Date();
		this.gameVariables = gameVariables;
		this.entities.put(mapID, entities);
	}

	/**
	 * @return the mapID
	 */
	public int getMapID()
	{
		return mapID;
	}

	/**
	 * @return the mapName
	 */
	public String getMapName()
	{
		return mapName;
	}

	/**
	 * @return the time
	 */
	public Date getTime()
	{
		return time;
	}

	/**
	 * @return the gameVariables
	 */
	public GameVariables getGameVariables()
	{
		return gameVariables;
	}

	/**
	 * @return the entities
	 */
	public Set<DrawableGameEntity> getEntities(final Integer mapID)
	{
		return entities.get(mapID);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int compareTo(final SaveGameData other)
	{
		return -this.getTime().compareTo(other.getTime());
	}
}
