/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		 \u00a9 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states;

import game.GameHandler;
import game.resources.ResourceProvider;
import game.resources.ResourceType;
import game.resources.TextResource;
import game.states.map.entity.PlayerCharacter;
import game.widget.Composite;
import game.widget.TextWidget;
import game.widget.Widget;

import org.lwjgl.util.Rectangle;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.StateBasedGame;

import application.properties.Properties;

/**
 * @author Drag-On
 * 
 */
public class CharacterStatsMenuState extends AbstractGameState
{
	/**
	 * Main composite
	 */
	private Composite mainComp;

	/**
	 * Health points text widget
	 */
	private TextWidget healthPointText;

	/**
	 * Attack points text widget
	 */
	private TextWidget attackText;

	/**
	 * Defense points text widget
	 */
	private TextWidget defenseText;

	/**
	 * Charisma points text widget
	 */
	private TextWidget charismaText;

	/**
	 * Sanity points text widget
	 */
	private TextWidget sanityText;

	/**
	 * Flag indicating if the character stats have changed since last opened
	 */
	private boolean dirty = false;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void init(final GameContainer container, final StateBasedGame game)
			throws SlickException
	{
		// Request important resources so they are loaded
		ResourceProvider.getInstance().get("TextHeading", ResourceType.FONT);
		ResourceProvider.getInstance().get("TextNormal", ResourceType.FONT);
		ResourceProvider.getInstance().get("TextSmall", ResourceType.FONT);
		ResourceProvider.getInstance().get("defaultSkin", ResourceType.SKIN);
		ResourceProvider.getInstance().get("MainTexts", ResourceType.TEXT);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void firstInit(final GameContainer container,
			final StateBasedGame game)
	{
		// Initialize main menu
		int w = 300;
		int h = 170;
		int x = container.getWidth() / 2 - w / 2;
		int y = container.getHeight() / 2 - h / 2;
		Rectangle bounds = new Rectangle(x, y, w, h);
		mainComp = new Composite(bounds, "TextHeading", "defaultSkin", 2,
				Widget.NONE);
		TextResource texts = ResourceProvider.getInstance().get("MainTexts",
				ResourceType.TEXT);

		new TextWidget(this.mainComp, texts.getText("healthpoints") + ": ", 1,
				Widget.NONE);
		healthPointText = new TextWidget(this.mainComp, "", 1, Widget.RIGHT);
		new TextWidget(this.mainComp, texts.getText("attack") + ": ", 1,
				Widget.NONE);
		attackText = new TextWidget(this.mainComp, "", 1, Widget.RIGHT);
		new TextWidget(this.mainComp, texts.getText("defense") + ": ", 1,
				Widget.NONE);
		defenseText = new TextWidget(this.mainComp, "", 1, Widget.RIGHT);
		new TextWidget(this.mainComp, texts.getText("charisma") + ": ", 1,
				Widget.NONE);
		charismaText = new TextWidget(this.mainComp, "", 1, Widget.RIGHT);
		new TextWidget(this.mainComp, texts.getText("sanity") + ": ", 1,
				Widget.NONE);
		sanityText = new TextWidget(this.mainComp, "", 1, Widget.RIGHT);

		updateValues();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void enter(final GameContainer container, final StateBasedGame game)
			throws SlickException
	{
		super.enter(container, game);

		updateValues();

		setDirty(false);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void render(final GameContainer container,
			final StateBasedGame game, final Graphics g) throws SlickException
	{
		// Render map in background
		GameHandler.getInstance().getState(GameHandler.STATE_MAP)
				.render(container, game, g);

		mainComp.render(container, game, g);
		mainComp.postRenderState(container, g);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void update(final GameContainer container,
			final StateBasedGame game, final int delta) throws SlickException
	{
		mainComp.update(container, game, delta);

		// Go back to map state in case the return key has been pressed
		if (container.getInput().isKeyPressed(Properties.keyReturn)
				|| container.getInput().isKeyPressed(
						Properties.keyCharacterStats)
				|| container.getInput().isMousePressed(1))
		{
			game.enterState(GameHandler.STATE_MAP);
		}

		GameHandler.getInstance().getState(GameHandler.STATE_MAP)
				.update(container, game, delta);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int getID()
	{
		return GameHandler.STATE_CHARACTER_STATS_SCREEN;
	}

	/**
	 * Updates the values to display
	 */
	private void updateValues()
	{
		MapState mapState = (MapState) GameHandler.getInstance().getState(
				GameHandler.STATE_MAP);
		PlayerCharacter player = mapState.getMapData().getPlayerCharacter();

		healthPointText.setText(player.getCurHealth() + "/"
				+ player.getHealthBase());
		int attackBonusMelee = player.getInventory().getMeleeWeapon() != null ? player
				.getInventory().getMeleeWeapon().getResource()
				.getParameterAsInt("attackBonus")
				: 0;
		int attackBonusRanged = player.getInventory().getRangedWeapon() != null ? player
				.getInventory().getRangedWeapon().getResource()
				.getParameterAsInt("attackBonus")
				: 0;
		attackText.setText(String.valueOf(player.getAttackBase()) + " *+ "
				+ attackBonusMelee + " | " + attackBonusRanged + "*");
		int defenseBonus = player.getInventory().getArmor() != null ? player
				.getInventory().getArmor().getResource()
				.getParameterAsInt("defenseBonus") : 0;
		defenseText.setText(String.valueOf(player.getDefenseBase()) + " *+ "
				+ defenseBonus + "%*");
		charismaText.setText(String.valueOf(player.getCharisma()));
		sanityText.setText(String.valueOf(player.getSanity()));
	}

	/**
	 * @return the dirty
	 */
	public boolean isDirty()
	{
		return dirty;
	}

	/**
	 * @param dirty
	 *            the dirty to set
	 */
	public void setDirty(final boolean dirty)
	{
		this.dirty = dirty;
	}
}
