/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		 \u00a9 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states;

import game.GameHandler;
import game.resources.FontResource;
import game.resources.ResourceProvider;
import game.resources.ResourceType;
import game.widget.Composite;
import game.widget.TextWidget;
import game.widget.Widget;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;

import org.lwjgl.util.Rectangle;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.state.transition.FadeInTransition;
import org.newdawn.slick.state.transition.FadeOutTransition;
import org.newdawn.slick.util.Log;

import application.properties.Properties;

/**
 * This game state shows the credits
 * 
 * @author Drag-On
 * 
 */
public class EndCreditsState extends AbstractGameState
{
	/**
	 * Credits lines
	 */
	private String credits;

	/**
	 * Composite containing the text
	 */
	private Composite comp;

	/**
	 * Last y coordinate
	 */
	private float lastY;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void init(final GameContainer container, final StateBasedGame game)
			throws SlickException
	{
		// Read credits file
		BufferedReader br = null;
		try
		{
			String path = Properties.resourceBasePath + ".." + File.separator
					+ "Game" + File.separator + "lang" + File.separator
					+ Properties.language + File.separator + "Credits.txt";
			br = new BufferedReader(new InputStreamReader(new FileInputStream(
					path), "utf-8"));
			StringBuilder sb = new StringBuilder();
			String line = br.readLine();

			while (line != null)
			{
				sb.append(line);
				sb.append("\n");
				line = br.readLine();
			}
			credits = sb.toString();
		}
		catch (IOException e)
		{
			Log.error("Error while reading credits file.", e);
		}
		finally
		{
			try
			{
				br.close();
			}
			catch (IOException e)
			{
				Log.error("Error while reading credits file.", e);
			}
		}

		// Remove weird char in the beginning
		credits = credits.substring(1);

		FontResource fontRes = ResourceProvider.getInstance().get(
				"TextHeading", ResourceType.FONT, true);
		int height = fontRes.getFont(false, false).getHeight(credits);
		this.comp = new Composite(new Rectangle(0, container.getHeight(),
				container.getWidth(), height), "TextHeading", "defaultSkin", 1,
				Widget.NO_BG);
		new TextWidget(comp, credits, 1, Widget.CENTER);
		this.lastY = this.comp.getBounds().getY();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void render(final GameContainer container,
			final StateBasedGame game, final Graphics g) throws SlickException
	{
		this.comp.render(container, game, g);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void update(final GameContainer container,
			final StateBasedGame game, final int delta) throws SlickException
	{
		this.comp.update(container, game, delta);
		this.comp.getBounds().setY((int) (this.lastY -= (delta / 20.0f)));

		if (container.getInput().isKeyPressed(Properties.keyAction)
				|| container.getInput().isMousePressed(
						Properties.keyAttackMelee)
				|| container.getInput().isMousePressed(
						Properties.keyAttackRanged)
				|| container.getInput().isKeyPressed(Properties.keyReturn)
				|| this.lastY + this.comp.getBounds().getHeight() <= 0)
		{
			GameHandler.getInstance().enterState(GameHandler.STATE_MAIN_MENU,
					new FadeOutTransition(), new FadeInTransition());
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int getID()
	{
		return GameHandler.STATE_END_CREDITS;
	}

}
