/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		 \u00a9 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states;

import game.GameHandler;
import game.audio.AudioPlayer;
import game.resources.FontResource;
import game.resources.ResourceProvider;
import game.resources.ResourceType;
import game.resources.TextResource;

import org.newdawn.slick.Font;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.state.transition.FadeInTransition;
import org.newdawn.slick.state.transition.FadeOutTransition;

import application.properties.Properties;

/**
 * The game state that is viewed in case the player loses the game
 * 
 * @author Drag-On
 * 
 */
public class GameOverState extends AbstractGameState
{
	/**
	 * {@inheritDoc}
	 */
	@Override
	public void init(final GameContainer container, final StateBasedGame game)
			throws SlickException
	{
		// Request resources
		ResourceProvider.getInstance().get("TextHeading", ResourceType.FONT);
		ResourceProvider.getInstance().get("MainTexts", ResourceType.TEXT);
		ResourceProvider.getInstance().get("GameOver", ResourceType.MUSIC);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void enter(final GameContainer container, final StateBasedGame game)
			throws SlickException
	{
		super.enter(container, game);

		// Stop all currently playing sounds
		AudioPlayer.stopAllSFX();
		AudioPlayer.stopMusic();

		AudioPlayer.loopMusic("GameOver");
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void render(final GameContainer container,
			final StateBasedGame game, final Graphics g) throws SlickException
	{
		FontResource fontRes = ResourceProvider.getInstance().get(
				"TextHeading", ResourceType.FONT);
		Font heading = fontRes.getFont(false, true);
		fontRes = ResourceProvider.getInstance().get("TextNormal",
				ResourceType.FONT);
		Font normal = fontRes.getFont(false, false);

		TextResource textRes = ResourceProvider.getInstance().get("MainTexts",
				ResourceType.TEXT);
		String gameOver = textRes.getText("gameOver");
		String gameOverSub = textRes.getText("gameOverSub");

		int x = container.getWidth() / 2 - heading.getWidth(gameOver) / 2;
		int y = container.getHeight() / 2;

		heading.drawString(x, y, gameOver);
		normal.drawString(x, y + heading.getLineHeight(), gameOverSub);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void update(final GameContainer container,
			final StateBasedGame game, final int delta) throws SlickException
	{
		if (container.getInput().isKeyPressed(Properties.keyReturn)
				|| container.getInput().isKeyPressed(Properties.keyAction)
				|| container.getInput().isMousePressed(0))
			game.enterState(GameHandler.STATE_MAIN_MENU,
					new FadeOutTransition(), new FadeInTransition());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int getID()
	{
		return GameHandler.STATE_GAME_OVER;
	}

}
