/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		 \u00a9 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states;

import game.GameHandler;
import game.audio.AudioPlayer;
import game.managers.Resource;
import game.resources.FontResource;
import game.resources.ResourceProvider;
import game.resources.ResourceType;
import game.resources.TextResource;

import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.GameState;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.state.transition.Transition;

/**
 * @author Drag-On
 * 
 */
public class LoadingState extends AbstractGameState
{
	/**
	 * State to move to after all resources have been loaded
	 */
	private GameState nextState;

	/**
	 * Transition to use when moving into the next state
	 */
	private Transition inTransition = null;

	/**
	 * Transition to use when leaving this state
	 */
	private Transition outTransition = null;

	/**
	 * The resource that was last loaded
	 */
	private Resource lastLoadedResource;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void init(final GameContainer container, final StateBasedGame game)
			throws SlickException
	{
		this.nextState = GameHandler.getInstance().getState(
				GameHandler.STATE_MAIN_MENU);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void render(final GameContainer container,
			final StateBasedGame game, final Graphics g) throws SlickException
	{
		int barX = (container.getWidth() - 400) / 2;
		int barY = container.getHeight() - 200;
		int barW = 400;
		int barH = 50;
		int left = ResourceProvider.getInstance().getLoadingListSize();

		// Draw border
		g.setColor(Color.white);
		g.setLineWidth(2.0f);
		g.drawRect(barX, barY, barW, barH);

		// Draw bar
		g.fillRect(barX, barY, barW / (left + 1), barH);

		// Draw loading text
		String text = "Loading...";
		TextResource textRes = ResourceProvider.getInstance().get("MainTexts",
				ResourceType.TEXT);
		if (textRes.isLoaded())
			text = textRes.getText("loading");

		FontResource fontResource = ResourceProvider.getInstance().get(
				"TextNormal", ResourceType.FONT, true);
		int height = fontResource.getFont(true, true).getHeight(text);
		fontResource.getFont(true, true).drawString(barX, barY - height - 3,
				text);

		if (this.lastLoadedResource != null)
		{
			// Draw resource name
			String path = this.lastLoadedResource.getPath();
			int pathBegin = path.length() > 20 ? path.length() - 20 : 0;
			fontResource.getFont(false, false).drawString(barX, barY + barH,
					"..." + path.substring(pathBegin));
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void update(final GameContainer container,
			final StateBasedGame game, final int delta)
	{
		if (ResourceProvider.getInstance().isLoadingNeeded())
		{
			lastLoadedResource = ResourceProvider.getInstance()
					.loadNextResource();
		}
		else
		{
			game.enterState(this.nextState.getID(), this.outTransition,
					this.inTransition);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void leave(final GameContainer container, final StateBasedGame game)
			throws SlickException
	{
		// Stop all currently playing audio
		AudioPlayer.stopAllSFX();
		AudioPlayer.stopMusic();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int getID()
	{
		return GameHandler.STATE_LOADING;
	}

	/**
	 * @return the nextState
	 */
	public GameState getNextState()
	{
		return nextState;
	}

	/**
	 * @param nextState
	 *            the nextState to set
	 */
	public void setNextState(final GameState nextState)
	{
		this.nextState = nextState;
	}

	/**
	 * @return the inTransition
	 */
	public Transition getInTransition()
	{
		return inTransition;
	}

	/**
	 * @param inTransition
	 *            the inTransition to set
	 */
	public void setInTransition(final Transition inTransition)
	{
		this.inTransition = inTransition;
	}

	/**
	 * @return the outTransition
	 */
	public Transition getOutTransition()
	{
		return outTransition;
	}

	/**
	 * @param outTransition
	 *            the outTransition to set
	 */
	public void setOutTransition(final Transition outTransition)
	{
		this.outTransition = outTransition;
	}

}
