/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		 \u00a9 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states;

import game.GameHandler;
import game.resources.ResourceProvider;
import game.resources.ResourceType;
import game.resources.TextResource;
import game.widget.ButtonWidget;
import game.widget.Composite;
import game.widget.Widget;
import game.widget.event.SelectionEvent;
import game.widget.event.SelectionEventListener;

import org.lwjgl.util.Rectangle;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.state.transition.FadeInTransition;
import org.newdawn.slick.state.transition.FadeOutTransition;

import application.properties.Properties;

/**
 * @author Drag-On
 * 
 */
public class PauseMenuState extends AbstractGameState
{
	/**
	 * Menu composite
	 */
	private Composite menuComp;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void init(final GameContainer container, final StateBasedGame game)
			throws SlickException
	{
		// Request important resources so they are being loaded
		ResourceProvider.getInstance().get("TextHeading", ResourceType.FONT);
		ResourceProvider.getInstance().get("TextNormal", ResourceType.FONT);
		ResourceProvider.getInstance().get("TextSmall", ResourceType.FONT);
		ResourceProvider.getInstance().get("defaultSkin", ResourceType.SKIN);
		ResourceProvider.getInstance().get("MainTexts", ResourceType.TEXT);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void firstInit(final GameContainer container,
			final StateBasedGame game)
	{
		// Initialize main menu
		menuComp = new Composite(new Rectangle(container.getWidth() / 2 - 80,
				container.getHeight() / 2 - 69, 160, 138), "TextHeading",
				"defaultSkin", 1, Widget.NONE);
		TextResource texts = ResourceProvider.getInstance().get("MainTexts",
				ResourceType.TEXT);

		// Continue game
		final ButtonWidget buttonContinue = new ButtonWidget(menuComp,
				texts.getText("continue"), 1, Widget.CENTER);
		buttonContinue.setToolTipText(texts.getText("continueTooltip"));
		buttonContinue.addSelectionEventListener(new SelectionEventListener()
		{
			@Override
			public void selected(final SelectionEvent evt)
			{
				game.enterState(GameHandler.STATE_MAP);
			}
		});

		// Save game
		final ButtonWidget buttonSaveGame = new ButtonWidget(menuComp,
				texts.getText("saveGame"), 1, Widget.CENTER);
		buttonSaveGame.setToolTipText(texts.getText("saveGameTooltip"));
		buttonSaveGame.addSelectionEventListener(new SelectionEventListener()
		{
			@Override
			public void selected(final SelectionEvent evt)
			{
				// Open save menu
				SaveGameMenuState saveState = (SaveGameMenuState) game
						.getState(GameHandler.STATE_SAVE_GAME);
				saveState.setLastState(getID());
				saveState.setLoadGame(false);
				game.enterState(GameHandler.STATE_SAVE_GAME,
						new FadeOutTransition(), new FadeInTransition());
			}
		});

		// Load game
		final ButtonWidget buttonLoadGame = new ButtonWidget(menuComp,
				texts.getText("loadGame"), 1, Widget.CENTER);
		buttonLoadGame.setToolTipText(texts.getText("loadGameTooltip"));
		buttonLoadGame.addSelectionEventListener(new SelectionEventListener()
		{
			@Override
			public void selected(final SelectionEvent evt)
			{
				// Enter load game state
				SaveGameMenuState saveState = (SaveGameMenuState) game
						.getState(GameHandler.STATE_SAVE_GAME);
				saveState.setLastState(getID());
				saveState.setLoadGame(true);
				game.enterState(GameHandler.STATE_SAVE_GAME,
						new FadeOutTransition(), new FadeInTransition());
			}
		});

		// Back to main menu
		final ButtonWidget buttonBackToMain = new ButtonWidget(menuComp,
				texts.getText("backToMain"), 1, Widget.CENTER);
		buttonBackToMain.setToolTipText(texts.getText("backToMainTooltip"));
		buttonBackToMain.addSelectionEventListener(new SelectionEventListener()
		{
			@Override
			public void selected(final SelectionEvent evt)
			{
				game.enterState(GameHandler.STATE_MAIN_MENU,
						new FadeOutTransition(), new FadeInTransition());
			}
		});
		menuComp.layout();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void render(final GameContainer container,
			final StateBasedGame game, final Graphics g) throws SlickException
	{
		// Render map in background
		GameHandler.getInstance().getState(GameHandler.STATE_MAP)
				.render(container, game, g);

		// Render composite
		if (menuComp != null)
		{
			menuComp.render(container, game, g);
			menuComp.postRenderState(container, g);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void update(final GameContainer container,
			final StateBasedGame game, final int delta) throws SlickException
	{
		if (this.menuComp != null)
		{
			this.menuComp.update(container, game, delta);
		}

		if (container.getInput().isKeyPressed(Properties.keyReturn)
				|| container.getInput().isMousePressed(1))
		{
			game.enterState(GameHandler.STATE_MAP);
		}

		GameHandler.getInstance().getState(GameHandler.STATE_MAP)
				.update(container, game, delta);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int getID()
	{
		return GameHandler.STATE_PAUSE_MENU;
	}

}
