/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states;

import game.GameHandler;
import game.resources.QuestResource;
import game.resources.ResourceProvider;
import game.resources.ResourceType;
import game.resources.TextResource;
import game.states.questlog.Questlog;
import game.states.questlog.QuestlogItem;
import game.widget.Composite;
import game.widget.ListItemWidget;
import game.widget.ListWidget;
import game.widget.TextWidget;
import game.widget.Widget;
import game.widget.event.SelectionEvent;
import game.widget.event.SelectionEventListener;

import java.util.Iterator;

import org.lwjgl.util.Rectangle;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.StateBasedGame;

import application.properties.Properties;

/**
 * This state is used when the player opens the quest log
 * 
 * @author Drag-On
 * 
 */
public class QuestlogState extends AbstractGameState
{
	/**
	 * Composite used for the left menu bar
	 */
	private ListWidget leftMenuBar;

	/**
	 * Quest data composite
	 */
	private Composite questTitleComp;

	/**
	 * Quest data composite
	 */
	private Composite questDescriptionComp;

	/**
	 * Quest data composite
	 */
	private Composite questStagesComp;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void init(final GameContainer container, final StateBasedGame game)
			throws SlickException
	{
		// Request important resources so they are loaded
		ResourceProvider.getInstance().get("TextHeading", ResourceType.FONT);
		ResourceProvider.getInstance().get("TextNormal", ResourceType.FONT);
		ResourceProvider.getInstance().get("TextSmall", ResourceType.FONT);
		ResourceProvider.getInstance().get("defaultSkin", ResourceType.SKIN);
		ResourceProvider.getInstance().get("MainTexts", ResourceType.TEXT);

		// Request quests
		if (this.getQuestlog() != null)
		{
			Iterator<QuestlogItem> it = getQuestlog().getQuests().values()
					.iterator();
			while (it.hasNext())
			{
				ResourceProvider.getInstance().get(it.next().getQuestID(),
						ResourceType.QUEST, true);
			}
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void enter(final GameContainer container, final StateBasedGame game)
			throws SlickException
	{
		// Initialize widgets
		int x = 100;
		int y = 100;
		int w = container.getWidth() / 3;
		int h = container.getHeight() - 2 * y;

		TextResource texts = ResourceProvider.getInstance().get("MainTexts",
				ResourceType.TEXT, true);

		// Create and fill the left menu bar
		this.leftMenuBar = new ListWidget(new Rectangle(x, y, w, h),
				"TextNormal", "defaultSkin", new int[] { 1 }, Widget.NONE);
		Iterator<QuestlogItem> it = getQuestlog().getQuests().values()
				.iterator();
		while (it.hasNext())
		{
			QuestlogItem item = it.next();
			if (!item.isFinished())
			{
				String name = texts.replaceKeys(item.getName());
				ListItemWidget itemWidget = new ListItemWidget(leftMenuBar,
						name, Widget.NONE);
				itemWidget.setData(item.getQuestID());
				itemWidget
						.addSelectionEventListener(new SelectionEventListener()
						{
							@Override
							public void selected(final SelectionEvent evt)
							{
								onQuestSelection(evt, container);
							}
						});
			}
		}

		QuestlogItem firstQuest = getQuestlog().getQuests().values().iterator()
				.next();
		createQuestInfoComp(firstQuest.getQuestID());
	}

	/**
	 * Creates the quest info composites
	 * 
	 * @param questID
	 *            Resource ID of the quest
	 */
	private void createQuestInfoComp(final String questID)
	{
		QuestResource questRes = ResourceProvider.getInstance().get(questID,
				ResourceType.QUEST);
		QuestlogItem quest = getQuestlog().getQuests().get(questID);
		int x = 100 + GameHandler.getInstance().getContainer().getWidth() / 3;
		int y = 100;
		int w = 2 * GameHandler.getInstance().getContainer().getWidth() / 3 - 2
				* y;
		int h = GameHandler.getInstance().getContainer().getHeight() - 2 * y;

		TextResource texts = ResourceProvider.getInstance().get("MainTexts",
				ResourceType.TEXT, true);

		// Name composite
		if (this.questTitleComp != null)
			this.questTitleComp.dispose();
		this.questTitleComp = new Composite(new Rectangle(x, y, w, 42),
				"TextHeading", "defaultSkin", 1, Widget.NONE);
		new TextWidget(questTitleComp, "*" + quest.getName() + "*", 1,
				Widget.WORD_WRAP);
		this.questTitleComp.layout();

		// Quest description
		if (questDescriptionComp != null)
			this.questDescriptionComp.dispose();
		this.questDescriptionComp = new Composite(new Rectangle(x, y + 42, w,
				h / 2 - 42), "TextNormal", "defaultSkin", 1, Widget.NONE);
		new TextWidget(questDescriptionComp, quest.getDescription(), 1,
				Widget.WORD_WRAP);
		this.questDescriptionComp.layout();

		// Quest stages
		if (questStagesComp != null)
			this.questStagesComp.dispose();
		this.questStagesComp = new Composite(new Rectangle(x, y + h / 2, w,
				h / 2), "TextNormal", "defaultSkin", 1, Widget.NONE);
		new TextWidget(questStagesComp, "*" + texts.getText("openTargets")
				+ "*", 1, Widget.WORD_WRAP);
		String openTargets = "";
		for (String id : quest.getOpen())
		{
			openTargets += texts.replaceKeys(questRes.getQuestStage(id))
					+ "\n\n";
		}
		new TextWidget(questStagesComp, openTargets, 1, Widget.WORD_WRAP);
		new TextWidget(questStagesComp, "*" + texts.getText("closedTargets")
				+ "*", 1, Widget.WORD_WRAP);
		String closedTargets = "";
		for (String id : quest.getClosed())
		{
			closedTargets += texts.replaceKeys(questRes.getQuestStage(id))
					+ "\n\n";
		}
		new TextWidget(questStagesComp, closedTargets, 1, Widget.WORD_WRAP);
		this.questStagesComp.layout();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void leave(final GameContainer container, final StateBasedGame game)
			throws SlickException
	{
		this.leftMenuBar.dispose();
		this.questTitleComp.dispose();

		container.getInput().clearControlPressedRecord();
		container.getInput().clearKeyPressedRecord();
		container.getInput().clearMousePressedRecord();
	}

	/**
	 * Called when a new quest has been selected
	 * 
	 * @param evt
	 *            Event context
	 * @param container
	 *            Game container
	 */
	protected void onQuestSelection(final SelectionEvent evt,
			final GameContainer container)
	{
		String questID = (String) ((Widget) evt.getSource()).getData();
		QuestlogItem quest = getQuestlog().getQuests().get(questID);

		createQuestInfoComp(quest.getQuestID());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void render(final GameContainer container,
			final StateBasedGame game, final Graphics g) throws SlickException
	{
		// Render map in background
		game.getState(GameHandler.STATE_MAP).render(container, game, g);

		if (this.leftMenuBar != null)
		{
			this.leftMenuBar.render(container, game, g);
			this.leftMenuBar.postRenderState(container, g);
		}
		if (this.questTitleComp != null)
		{
			this.questTitleComp.render(container, game, g);
			this.questTitleComp.postRenderState(container, g);
		}
		if (this.questDescriptionComp != null)
		{
			this.questDescriptionComp.render(container, game, g);
			this.questDescriptionComp.postRenderState(container, g);
		}
		if (this.questStagesComp != null)
		{
			this.questStagesComp.render(container, game, g);
			this.questStagesComp.postRenderState(container, g);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void update(final GameContainer container,
			final StateBasedGame game, final int delta) throws SlickException
	{
		// Go back to map state in case the return key has been pressed
		if (container.getInput().isKeyPressed(Properties.keyReturn)
				|| container.getInput().isKeyPressed(Properties.keyQuestlog)
				|| container.getInput().isMousePressed(1))
		{
			game.enterState(GameHandler.STATE_MAP);
		}

		GameHandler.getInstance().getState(GameHandler.STATE_MAP)
				.update(container, game, delta);

		// Update composites
		if (this.leftMenuBar != null)
			this.leftMenuBar.update(container, game, delta);
		if (this.questTitleComp != null)
			this.questTitleComp.update(container, game, delta);
		if (this.questDescriptionComp != null)
			this.questDescriptionComp.update(container, game, delta);
		if (this.questStagesComp != null)
			this.questStagesComp.update(container, game, delta);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int getID()
	{
		return GameHandler.STATE_QUESTLOG;
	}

	/**
	 * @return The questlog or <code>null</code> in case it has not been
	 *         initialized yet
	 */
	private Questlog getQuestlog()
	{
		MapState map = (MapState) GameHandler.getInstance().getState(
				GameHandler.STATE_MAP);
		if (map.getMapData().getUID() >= 0)
			return map.getMapData().getPlayerCharacter().getQuestlog();
		else
			return null;
	}
}
