/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		 \u00a9 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states;

import game.GameHandler;
import game.resources.ResourceProvider;
import game.resources.ResourceType;
import game.resources.TextResource;
import game.save.SaveGameData;
import game.widget.ButtonWidget;
import game.widget.Composite;
import game.widget.ListItemWidget;
import game.widget.ListWidget;
import game.widget.Widget;
import game.widget.event.SelectionEvent;
import game.widget.event.SelectionEventListener;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.lwjgl.util.Rectangle;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.state.transition.FadeInTransition;
import org.newdawn.slick.state.transition.FadeOutTransition;
import org.newdawn.slick.util.Log;

import application.properties.Properties;

/**
 * This game state is used in case the player opened the save menu
 * 
 * @author Drag-On
 * 
 */
public class SaveGameMenuState extends AbstractGameState
{
	/**
	 * Widget to display all savegames
	 */
	private ListWidget saveGameList;

	/**
	 * Flag indicating if this is used for loading or saving a game
	 */
	private boolean loadGame = false;

	/**
	 * State that is called when the user decided to go back
	 */
	private int lastState = GameHandler.STATE_MAIN_MENU;

	/**
	 * Composite holding the buttons
	 */
	private Composite buttonsComposite;

	/**
	 * Button that is labeled as "save" or "load", depending on state
	 */
	private ButtonWidget saveLoadButton;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void init(final GameContainer container, final StateBasedGame game)
			throws SlickException
	{
		ResourceProvider.getInstance().get("MainTexts", ResourceType.TEXT);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void firstInit(final GameContainer container,
			final StateBasedGame game)
	{
		super.firstInit(container, game);

		TextResource textRes = ResourceProvider.getInstance().get("MainTexts",
				ResourceType.TEXT);

		int x = 100;
		int y = 100;
		int w = container.getWidth() - 2 * x;
		int h = container.getHeight() - 2 * y;
		Rectangle bounds = new Rectangle(x, y, w, h);
		this.saveGameList = new ListWidget(bounds, "TextNormal", "defaultSkin",
				new int[] { 1 }, Widget.CENTER);
		buttonsComposite = new Composite(new Rectangle(x + w - 200, y + h, 200,
				42), "TextNormal", "defaultSkin", 2, Widget.CENTER);
		String actionButtonText;
		if (this.loadGame)
			actionButtonText = textRes.getText("load");
		else
			actionButtonText = textRes.getText("save");
		saveLoadButton = new ButtonWidget(buttonsComposite, actionButtonText,
				1, Widget.CENTER);
		saveLoadButton.addSelectionEventListener(new SelectionEventListener()
		{
			@Override
			public void selected(final SelectionEvent evt)
			{
				if (SaveGameMenuState.this.loadGame)
				{
					if (SaveGameMenuState.this.saveGameList.getLines() > 0
							&& SaveGameMenuState.this.saveGameList
									.getMarkedLines() != null)
					{
						int index = SaveGameMenuState.this.saveGameList
								.getMarkedLines().get(0);
						loadGame((SaveGameData) SaveGameMenuState.this.saveGameList
								.getItem(0, index).getData());
					}
				}
				else
					saveGame();
			}
		});
		ButtonWidget cancelButton = new ButtonWidget(buttonsComposite,
				textRes.getText("cancel"), 1, Widget.CENTER);
		cancelButton.addSelectionEventListener(new SelectionEventListener()
		{
			@Override
			public void selected(final SelectionEvent evt)
			{
				GameHandler.getInstance().enterState(
						SaveGameMenuState.this.lastState,
						new FadeOutTransition(), new FadeInTransition());
			}
		});
	}

	/**
	 * Saves the game to hard disk
	 */
	protected void saveGame()
	{
		MapState mapState = (MapState) GameHandler.getInstance().getState(
				GameHandler.STATE_MAP);
		mapState.getMapData().updateSaveGame();
		SaveGameData saveGame = mapState.getMapData().getSaveGame();

		try
		{
			final FileOutputStream fOut = new FileOutputStream(
					Properties.saveGameFolder
							+ File.separator
							+ DateFormat.getInstance()
									.format(saveGame.getTime())
									.replaceAll("\\W+", "") + ".save");
			final ObjectOutputStream objOut = new ObjectOutputStream(fOut);
			objOut.writeObject(saveGame);
			objOut.close();
			fOut.close();
		}
		catch (final IOException e)
		{
			Log.error("Unable to serialize save game.", e);
		}

		GameHandler.getInstance().enterState(SaveGameMenuState.this.lastState,
				new FadeOutTransition(), new FadeInTransition());
	}

	/**
	 * Loads a game from hard disk
	 * 
	 * @param saveGame
	 *            Save game object to load
	 */
	protected void loadGame(final SaveGameData saveGame)
	{
		MapState mapState = (MapState) GameHandler.getInstance().getState(
				GameHandler.STATE_MAP);
		mapState.getMapData().loadSaveGame(saveGame);

		GameHandler.getInstance().enterState(GameHandler.STATE_MAP,
				new FadeOutTransition(), new FadeInTransition());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void enter(final GameContainer container, final StateBasedGame game)
			throws SlickException
	{
		super.enter(container, game);

		initSaveGameList();
		this.saveGameList.layout();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void render(final GameContainer container,
			final StateBasedGame game, final Graphics g) throws SlickException
	{
		this.saveGameList.render(container, game, g);
		this.buttonsComposite.render(container, game, g);
		this.saveGameList.postRenderState(container, g);
		this.buttonsComposite.postRenderState(container, g);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void update(final GameContainer container,
			final StateBasedGame game, final int delta) throws SlickException
	{
		this.saveGameList.update(container, game, delta);
		this.buttonsComposite.update(container, game, delta);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int getID()
	{
		return GameHandler.STATE_SAVE_GAME;
	}

	/**
	 * @return the loadGame
	 */
	public boolean isLoadGame()
	{
		return loadGame;
	}

	/**
	 * @param loadGame
	 *            the loadGame to set
	 */
	public void setLoadGame(final boolean loadGame)
	{
		this.loadGame = loadGame;

		if (this.saveLoadButton != null)
		{
			TextResource textRes = ResourceProvider.getInstance().get(
					"MainTexts", ResourceType.TEXT);
			if (this.loadGame)
				saveLoadButton.setText(textRes.getText("load"));
			else
				saveLoadButton.setText(textRes.getText("save"));
		}

	}

	/**
	 * @return the lastState
	 */
	public int getLastState()
	{
		return lastState;
	}

	/**
	 * @param lastState
	 *            the lastState to set
	 */
	public void setLastState(final int lastState)
	{
		this.lastState = lastState;
	}

	/**
	 * Initializes and fills the list
	 */
	private void initSaveGameList()
	{
		// Clear everything that may be in there already
		this.saveGameList.clear();

		// Load save games from hard disk
		File dir = new File(Properties.saveGameFolder);
		if (!dir.exists() || !dir.isDirectory())
			dir.mkdir();
		File[] files = dir.listFiles();
		List<SaveGameData> saveGames = new ArrayList<SaveGameData>();
		for (File file : files)
		{
			int index = file.getName().lastIndexOf('.');
			// Check if it's a save game file
			if (file.getName().substring(index + 1).equals("save"))
			{
				// Read save game data from from disk
				FileInputStream fIn;
				SaveGameData saveGameData = null;
				try
				{
					fIn = new FileInputStream(file.getAbsolutePath());
					final ObjectInputStream objIn = new ObjectInputStream(fIn);
					final Object obj = objIn.readObject();
					if (obj instanceof SaveGameData)
					{
						saveGameData = (SaveGameData) obj;
						saveGames.add(saveGameData);
					}
					objIn.close();
					fIn.close();
				}
				catch (IOException | ClassNotFoundException e)
				{
					Log.error("Unable to load save " + file.getName() + ".", e);
				}
			}
		}

		// Sort save games by date
		Collections.sort(saveGames);

		for (SaveGameData saveGameData : saveGames)
		{
			if (saveGameData != null)
			{
				String name = saveGameData.getMapName()
						+ " / "
						+ DateFormat.getInstance().format(
								saveGameData.getTime());
				ListItemWidget item = new ListItemWidget(saveGameList, name,
						Widget.CENTER);
				item.setData(saveGameData);
				if (this.loadGame)
				{
					item.addSelectionEventListener(new SelectionEventListener()
					{
						@Override
						public void selected(final SelectionEvent evt)
						{
							// TODO: Add save / Load game options
							SaveGameMenuState.this.saveGameList
									.clearLineMarks();
							SaveGameMenuState.this.saveGameList
									.toggleLineMark(evt.getY());
						}
					});
				}
			}
		}
	}
}
