/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		 \u00a9 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.inventory;

import game.resources.InventoryItemResource;
import game.resources.ResourceProvider;
import game.resources.ResourceType;

import java.io.Serializable;

/**
 * A collectible item which can be put into the player's inventory
 * 
 * @author Drag-On
 * 
 */
public class InventoryItem implements Serializable
{
	/**
	 * Serial Version UID
	 */
	private static final long serialVersionUID = -7913069560419010555L;

	/**
	 * The amounts of items of this kind the player has
	 */
	private int amount;

	/**
	 * Resource ID of the item
	 */
	private final String resourceID;

	/**
	 * Constructor
	 * 
	 * @param resourceId
	 *            Resource ID of the item that is represented by this object
	 */
	public InventoryItem(final String resourceId)
	{
		this.resourceID = resourceId;
		this.amount = 1;
	}

	/**
	 * @return the amount
	 */
	public int getAmount()
	{
		return amount;
	}

	/**
	 * Adds some items of this type to the inventory
	 * 
	 * @param amount
	 *            Amount of items to add
	 */
	public void add(final int amount)
	{
		this.amount += amount;
	}

	/**
	 * Removes some items of this type from the inventory
	 * 
	 * @param amount
	 *            Amount of items to remove
	 */
	public void remove(final int amount)
	{
		this.amount -= amount;
	}

	/**
	 * @return the resourceID
	 */
	public String getResourceID()
	{
		return resourceID;
	}

	/**
	 * Provides access to the item resource
	 * 
	 * @return The item resource that corresponds to this item
	 */
	public InventoryItemResource getResource()
	{
		return ResourceProvider.getInstance().get(resourceID,
				ResourceType.ITEM, true);
	}
}
