/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		 \u00a9 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map;

import game.GameHandler;
import game.resources.FontResource;
import game.resources.ResourceProvider;
import game.resources.ResourceType;
import game.resources.SpritesheetResource;
import game.states.CharacterStatsMenuState;
import game.states.MapState;

import org.lwjgl.util.Rectangle;
import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.StateBasedGame;

import application.properties.Properties;

/**
 * The user interface showing live points. It's drawn on top of the map.
 * 
 * @author Drag-On
 * 
 */
public class GUI
{
	/**
	 * Current menu y value
	 */
	private float menuY = -70;

	/**
	 * Update everything
	 * 
	 * @param container
	 *            The game container
	 * @param game
	 *            Game object
	 * @param delta
	 *            Delta time since last update
	 * @throws SlickException
	 */
	public void update(final GameContainer container,
			final StateBasedGame game, final int delta) throws SlickException
	{
		// Animate menu
		if (container.getInput().getMouseY() < 80
				&& game.getCurrentStateID() == GameHandler.STATE_MAP)
		{
			if (this.menuY < 0)
				this.menuY += delta;
			if (this.menuY > 0)
				this.menuY = 0;
		}
		else
		{
			if (this.menuY >= -70)
				this.menuY -= delta;
			if (this.menuY < -70)
				this.menuY = -70;
		}

		int menuHeight = 73;
		int menuWidth = 382;
		int x = container.getWidth() / 2 - menuWidth / 2;
		int amountOfItems = Properties.immersiveMode ? 3 : 4;
		int itemSpace = (int) (menuWidth / (float) amountOfItems);
		// Mainmenu
		Rectangle rect = new Rectangle(x, 0, itemSpace, menuHeight);
		if (rect.contains(container.getInput().getMouseX(), container
				.getInput().getMouseY())
				&& container.getInput().isMouseButtonDown(0))
		{
			GameHandler.getInstance().enterState(GameHandler.STATE_PAUSE_MENU,
					null, null);
		}
		// Inventory
		rect.setX(x + itemSpace);
		if (rect.contains(container.getInput().getMouseX(), container
				.getInput().getMouseY())
				&& container.getInput().isMouseButtonDown(0))
		{
			GameHandler.getInstance().enterState(GameHandler.STATE_INVENTORY,
					null, null);
		}
		// Questlog
		rect.setX(x + 2 * itemSpace);
		if (rect.contains(container.getInput().getMouseX(), container
				.getInput().getMouseY())
				&& container.getInput().isMouseButtonDown(0)
				&& !((MapState) GameHandler.getInstance().getState(
						GameHandler.STATE_MAP)).getMapData()
						.getPlayerCharacter().getQuestlog().getQuests()
						.isEmpty())
		{
			GameHandler.getInstance().enterState(GameHandler.STATE_QUESTLOG,
					null, null);
		}
		// Character stats
		if (!Properties.immersiveMode)
		{
			rect.setX(x + 3 * itemSpace);
			if (rect.contains(container.getInput().getMouseX(), container
					.getInput().getMouseY())
					&& container.getInput().isMouseButtonDown(0))
			{
				GameHandler.getInstance().enterState(
						GameHandler.STATE_CHARACTER_STATS_SCREEN, null, null);
			}
		}
	}

	/**
	 * Render the GUI
	 * 
	 * @param container
	 *            The game container
	 * @param game
	 *            Game object
	 * @param g
	 *            Graphics context
	 * @throws SlickException
	 */
	public void render(final GameContainer container,
			final StateBasedGame game, final Graphics g) throws SlickException
	{
		MapState mapState = (MapState) GameHandler.getInstance().getState(
				GameHandler.STATE_MAP);

		boolean spriteFreed = false;

		// Render menu bar
		SpritesheetResource resMenu = ResourceProvider.getInstance().get(
				"TopMenu", ResourceType.SPRITESHEET);
		resMenu.getSpriteSheet().startUse();
		int menuHeight = 73;
		int menuWidth = 382;
		int y = (int) this.menuY;
		int x = container.getWidth() / 2 - menuWidth / 2;
		resMenu.getSpriteSheet().drawEmbedded(x, y, x + menuWidth,
				y + menuHeight, 0, 0, menuWidth, menuHeight);
		int amountOfItems = Properties.immersiveMode ? 3 : 4;
		int itemSpace = (int) (menuWidth / (float) amountOfItems);
		// Mainmenu
		y = (int) this.menuY - 10;
		Rectangle rect = new Rectangle(x, 0, itemSpace, menuHeight);
		if (rect.contains(container.getInput().getMouseX(), container
				.getInput().getMouseY())
				&& game.getCurrentStateID() == GameHandler.STATE_MAP)
			y += 10;
		resMenu.getSpriteSheet().drawEmbedded(x, y, x + 85, y + 99, 213, 74,
				298, 173);
		// Inventory
		y = (int) this.menuY - 10;
		rect.setX(x + itemSpace);
		if (rect.contains(container.getInput().getMouseX(), container
				.getInput().getMouseY())
				&& game.getCurrentStateID() == GameHandler.STATE_MAP)
			y += 10;
		resMenu.getSpriteSheet().drawEmbedded(x + itemSpace, y,
				x + itemSpace + 103, y + 99, 110, 74, 213, 173);
		// Questlog
		y = (int) this.menuY - 10;
		rect.setX(x + 2 * itemSpace);
		if (rect.contains(container.getInput().getMouseX(), container
				.getInput().getMouseY())
				&& game.getCurrentStateID() == GameHandler.STATE_MAP)
			y += 10;
		resMenu.getSpriteSheet().drawEmbedded(x + 2 * itemSpace, y,
				x + 2 * itemSpace + 88, y + 99, 293, 74, 381, 173);
		// Character stats
		if (!Properties.immersiveMode)
		{
			y = (int) this.menuY - 10;
			rect.setX(x + 3 * itemSpace);
			if (rect.contains(container.getInput().getMouseX(), container
					.getInput().getMouseY())
					&& game.getCurrentStateID() == GameHandler.STATE_MAP)
				y += 10;
			resMenu.getSpriteSheet().drawEmbedded(x + 3 * itemSpace, y,
					x + 3 * itemSpace + 112, y + 99, 0, 74, 112, 173);

			// Draw exclamation mark in case the character stats have been
			// changed
			if (((CharacterStatsMenuState) game
					.getState(GameHandler.STATE_CHARACTER_STATS_SCREEN))
					.isDirty())
			{
				resMenu.getSpriteSheet().endUse();
				spriteFreed = true;

				FontResource font = ResourceProvider.getInstance().get(
						"TextBig", ResourceType.FONT, true);
				font.getFont(false, false).drawString(x + 3 * itemSpace + 25,
						y + 40, "!");
			}
		}

		if (!spriteFreed)
		{
			resMenu.getSpriteSheet().endUse();
			spriteFreed = true;
		}

		// Render health bar
		if (!Properties.immersiveMode)
		{
			if (mapState.getMapData().getPlayerCharacter() != null)
			{
				SpritesheetResource res = ResourceProvider.getInstance().get(
						"Healthbar", ResourceType.SPRITESHEET);
				res.getSpriteSheet().startUse();

				// Health bar
				int barHeight = 28;
				int barMaxWidth = 145;
				int barWidth = (int) (barMaxWidth * ((float) mapState
						.getMapData().getPlayerCharacter().getCurHealth() / mapState
						.getMapData().getPlayerCharacter().getHealthBase()));
				x = 10;// container.getWidth() / 2 - barMaxWidth / 2;
				y = 10;

				// g.setColor(Color.red);
				// g.fillRoundRect(x, y, barWidth, barHeight, 10)
				res.getSpriteSheet().drawEmbedded(x + 2, y + 2,
						x + barWidth - 1, y + barHeight + 5, 8, 37, 153, 65);

				// Health bar frame
				res.getSpriteSheet().renderInUse(x - 6, y, 0, 0);

				res.getSpriteSheet().endUse();

				// Attack cooldown
				y += barHeight + 5;
				barHeight = 3;
				barMaxWidth = 100;
				barWidth = (int) (barMaxWidth * ((float) mapState.getMapData()
						.getPlayerCharacter().getCurAttackCooldown() / mapState
						.getMapData().getPlayerCharacter()
						.getAttackCooldownBase()));
				x = 10 + barWidth / 4;// container.getWidth() / 2 - barWidth /
										// 2;

				g.setColor(Color.orange);
				g.fillRoundRect(x, y, barWidth, barHeight, 1);
			}
		}
		else
		{
			// In immersive mode render only red border if hurt
			float healthLevel = mapState.getMapData().getPlayerCharacter()
					.getHealthLevel();

			SpritesheetResource res = ResourceProvider.getInstance().get(
					"Hurt", ResourceType.SPRITESHEET);

			res.getSpriteSheet().setAlpha(1 - (healthLevel / 100));
			res.getSpriteSheet().draw(0, 0, container.getWidth(),
					container.getHeight());
		}
	}
}
