/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		 \u00a9 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.ai.character;

import java.util.Random;

import application.properties.Properties;

/**
 * A simple AI implementation intended to be used for NPCs. This AI makes the
 * character move around randomly close to it's starting position.
 * 
 * @author Drag-On
 * 
 */
public class BasicCharacterAI implements ICharacterAI
{
	/**
	 * Serial Version UID
	 */
	private static final long serialVersionUID = -7519993839882252074L;

	/**
	 * Character that "owns" this AI
	 */
	private IAICharacter owner;

	/**
	 * Start coordinates
	 */
	private int startX, startY;

	/**
	 * Random generator
	 */
	private Random rand;

	/**
	 * Stores how much time needs to pass until next move
	 */
	private int nextUpdateTime = 0;

	/**
	 * Time that has passed since last update
	 */
	private int curTime = 0;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void init(final IAICharacter character)
	{
		// Store owner
		this.owner = character;

		// Store start coordinates so the character doesn't move away too far
		this.startX = character.getFootX();
		this.startY = character.getFootY();

		// Create random generator
		this.rand = new Random();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void update(final int delta)
	{
		if (this.curTime >= this.nextUpdateTime)
		{
			this.curTime = 0;
			this.nextUpdateTime = this.rand.nextInt(5000) + 3000;

			// Generate some coordinates to move to in case the character is not
			// already moving
			if (!this.owner.isMoving())
			{
				int radius = 4;
				int xOff = this.rand.nextInt(2 * radius) - radius;
				int yOff = this.rand.nextInt(2 * radius) - radius;
				this.owner.findPath(this.startX + xOff * Properties.tileSize,
						this.startY + yOff * Properties.tileSize);
			}
		}
		else
		{
			this.curTime += delta;
		}
	}
}
