/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		 \u00a9 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.ai.character;

/**
 * This interface defines the methods that need to be implemented by a character
 * that can be controlled by an AI. It's not to be confused with
 * {@link ICharacterAI} which defines the methods of an actual AI
 * implementation.
 * 
 * @author Drag-On
 * 
 */
public interface IAICharacter
{
	/**
	 * @return X coordinate of the character on map
	 */
	public int getFootX();

	/**
	 * @return Y coordinate of the character on map
	 */
	public int getFootY();

	/**
	 * Indicates if the character is currently moving
	 * 
	 * @return <code>true</code> in case the character is currently moving,
	 *         otherwise <code>false</code>
	 */
	public boolean isMoving();

	/**
	 * Makes the character move from it's current position to the passed
	 * coordinates finding an appropriate path on it's own.
	 * 
	 * @param x
	 *            X map coordinate to move to
	 * @param y
	 *            Y map coordinate to move to
	 */
	public void findPath(final int x, final int y);
}
