/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		 \u00a9 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.ai.character;

/**
 * An AI state needs to implement the methods defined in this interface.
 * 
 * @author Drag-On
 * 
 */
public interface ICharacterAIState
{
	/**
	 * Initializes the AI state
	 * 
	 * @param character
	 *            The character that is controlled by this AI instance
	 * @param ai
	 *            Main AI object
	 */
	public void init(IAICharacter character, AbstractStateBasedCharacterAI ai);

	/**
	 * This method calculates in what extent this state is capable of handling
	 * the situation the passed character is currently in.
	 * 
	 * @param character
	 *            Character to handle
	 * @param ai
	 *            Main AI object
	 * @return The score of this AI state to handle the character's current
	 *         situation
	 */
	public int getCompetenceScore(IAICharacter character,
			AbstractStateBasedCharacterAI ai);

	/**
	 * Updates this AI state
	 * 
	 * @param character
	 *            Character to handle
	 * @param ai
	 *            Main AI object
	 * @param delta
	 *            Time since last update
	 */
	public void update(IAICharacter character,
			AbstractStateBasedCharacterAI ai, final int delta);
}
