/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		 \u00a9 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.ai.character.enemy;

import game.states.map.ai.character.IAICharacter;

import java.util.List;

/**
 * A special enemy character
 * 
 * @author Drag-On
 * 
 */
public interface AbstractEnemyAICharacter extends IAICharacter
{
	/**
	 * Gets the current health as a percentage value
	 * 
	 * @return Current health level of the character on a scale from 0 to 100.
	 */
	public int getHealthLevel();

	/**
	 * Provides a list of all currently spotted enemies
	 * 
	 * @return A list containing all spotted enemies
	 */
	public List<AbstractEnemyAICharacter> getSpottedEnemies();

	/**
	 * @return <code>true</code> in case the character is able to attack melee,
	 *         otherwise <code>false</code>.
	 */
	public boolean canAttackMelee();

	/**
	 * @return <code>true</code> in case the character is able to attack ranged,
	 *         otherwise <code>false</code>.
	 */
	public boolean canAttackRanged();

	/**
	 * Starts a melee attack
	 */
	public void attackMelee();

	/**
	 * Fires a ranged attack to the passed coordinates
	 * 
	 * @param x
	 *            X map coordinate
	 * @param y
	 *            Y map coordinate
	 */
	public void attackRanged(int x, int y);

	/**
	 * @return The max distance to start a melee attack
	 */
	public int getMeleeDistance();

	/**
	 * @return The max distance to start a ranged attack
	 */
	public int getRangedDistance();

	/**
	 * Checks if the line of sight between this character and the passed
	 * coordinates is not blocked
	 * 
	 * @param x
	 *            X map coordinate
	 * @param y
	 *            Y map coordinate
	 * @return <code>true</code> in case there are no obstacles between this
	 *         character and the passed coordinates, otherwise
	 *         <code>false</code>
	 */
	public boolean checkLineOfSight(int x, int y);

	/**
	 * @return <code>true</code> in case there is any possibility to heal the
	 *         character
	 */
	public boolean canHeal();

	/**
	 * Executes the most appropriate method to heal the character
	 */
	public void heal();
}
