/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		 \u00a9 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.ai.character.enemy;

import game.states.map.ai.character.AbstractStateBasedCharacterAI;
import game.states.map.ai.character.IAICharacter;
import game.states.map.ai.character.ICharacterAIState;
import game.states.map.entity.PlayerCharacter;

import java.util.List;

import application.properties.Properties;

/**
 * The AI used for enemies.
 * 
 * @author Drag-On
 * 
 */
public class EnemyAI extends AbstractStateBasedCharacterAI
{
	/**
	 * Serial Version UID
	 */
	private static final long serialVersionUID = -8858714930143364052L;

	/**
	 * The currently engaged enemy or <code>null</code> if none
	 */
	private AbstractEnemyAICharacter engagedEnemy = null;

	/**
	 * Distance a enemy can "see"
	 */
	public static final int enemySightDistance = 12 * Properties.tileSize;

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void initStatesList(final List<ICharacterAIState> knownStates)
	{
		knownStates.add(new EnemyAIStateAttack());
		knownStates.add(new EnemyAIStateRetreat());
		knownStates.add(new EnemyAIStateExplore());
	}

	/**
	 * @return the engagedEnemy
	 */
	public AbstractEnemyAICharacter getEngagedEnemy()
	{
		return engagedEnemy;
	}

	/**
	 * @param engagedEnemy
	 *            the engagedEnemy to set
	 */
	public void setEngagedEnemy(final AbstractEnemyAICharacter engagedEnemy)
	{
		if (this.engagedEnemy instanceof PlayerCharacter
				&& !(engagedEnemy instanceof PlayerCharacter))
		{
			((PlayerCharacter) this.engagedEnemy).stopEngage();
		}
		else if (engagedEnemy instanceof PlayerCharacter)
		{
			((PlayerCharacter) engagedEnemy).engage();
		}
		this.engagedEnemy = engagedEnemy;
	}

	/**
	 * Calculates the squared distance between to characters
	 * 
	 * @param char1
	 *            First character
	 * @param char2
	 *            Second character
	 * @return Squared distance between both characters
	 */
	public static int squareDist(final IAICharacter char1,
			final IAICharacter char2)
	{
		int a = char1.getFootX() - char2.getFootX();
		int b = char1.getFootY() - char2.getFootY();
		return a * a + b * b;
	}
}
