/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		 \u00a9 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.ai.character.enemy;

import java.util.List;
import java.util.Random;

/**
 * Makes the character explore it's environment and search for enemies.
 * 
 * @author Drag-On
 * 
 */
public class EnemyAIStateExplore extends AbstractEnemyAIState
{
	/**
	 * Serial Version UID
	 */
	private static final long serialVersionUID = 5217303925305964792L;

	/**
	 * Random generator
	 */
	private Random rand;

	/**
	 * Stores how much time needs to pass until next move
	 */
	private int nextUpdateTime = 0;

	/**
	 * Time that has passed since last update
	 */
	private int curTime = 0;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void init(final AbstractEnemyAICharacter character, final EnemyAI ai)
	{
		// Create random generator
		this.rand = new Random();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int getCompetenceScore(final AbstractEnemyAICharacter character,
			final EnemyAI ai)
	{
		if (ai.getEngagedEnemy() != null)
			return 0;
		// Depending on health, the character is less likely to explore in case
		// it is wounded
		return (int) ((-1 / 48000f) * Math.pow(character.getHealthLevel(), 3)
				+ (-1 / 400f) * Math.pow(character.getHealthLevel(), 2) + (127 / 120f)
				* character.getHealthLevel());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void update(final AbstractEnemyAICharacter character,
			final EnemyAI ai, final int delta)
	{
		if (!character.isMoving() && this.curTime >= this.nextUpdateTime)
		{
			this.curTime = 0;
			this.nextUpdateTime = this.rand.nextInt(5000) + 1000;

			// Generate some coordinates to move to in case the character is not
			// already moving
			if (!character.isMoving())
			{
				int x = this.rand.nextInt(320) - 160 + ai.getStartX();
				int y = this.rand.nextInt(320) - 160 + ai.getStartY();
				character.findPath(x, y);
			}
		}
		else
		{
			this.curTime += delta;

			List<AbstractEnemyAICharacter> spottedEnemies = character
					.getSpottedEnemies();
			if (spottedEnemies != null && !spottedEnemies.isEmpty())
			{
				// Engage the closest enemy
				int closestDist = Integer.MAX_VALUE;
				AbstractEnemyAICharacter closest = null;
				for (AbstractEnemyAICharacter enemy : spottedEnemies)
				{
					int dist = EnemyAI.squareDist(character, enemy);
					if (dist < closestDist)
					{
						closestDist = dist;
						closest = enemy;
					}
				}
				ai.setEngagedEnemy(closest);
			}
		}
	}
}
