/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		 \u00a9 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.ai.character.enemy;

import java.util.Random;

/**
 * This state makes the enemy retreat and refresh
 * 
 * @author Drag-On
 * 
 */
public class EnemyAIStateRetreat extends AbstractEnemyAIState
{
	/**
	 * Serial Version UID
	 */
	private static final long serialVersionUID = 1421440388738133282L;

	/**
	 * Random generator
	 */
	private Random rand;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void init(final AbstractEnemyAICharacter character, final EnemyAI ai)
	{
		// Create random generator
		this.rand = new Random();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int getCompetenceScore(final AbstractEnemyAICharacter character,
			final EnemyAI ai)
	{
		// The lower the health value, the higher the score of this state
		if (character.getHealthLevel() > 0 && character.canHeal())
			return (int) ((-1 / 192000.0f)
					* Math.pow(character.getHealthLevel(), 4) + (7 / 4800.0f)
					* Math.pow(character.getHealthLevel(), 3) + (-71 / 480.0f)
					* Math.pow(character.getHealthLevel(), 2) + (65 / 12.0f)
					* character.getHealthLevel());
		else
			return 0;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void update(final AbstractEnemyAICharacter character,
			final EnemyAI ai, final int delta)
	{
		// Don't intercept any movements
		if (character.isMoving())
			return;

		// Try to heal
		if (character.canHeal())
		{
			character.heal();
		}

		// Move to a position in the opposite direction of the enemy
		AbstractEnemyAICharacter enemy = ai.getEngagedEnemy();
		if (enemy != null)
		{
			int xDiff = enemy.getFootX() - character.getFootX();
			int yDiff = enemy.getFootY() - character.getFootY();
			int xDir = xDiff > 0 ? 1 : -1;
			int yDir = yDiff > 0 ? 1 : -1;

			int xPos = character.getFootX() + rand.nextInt(128) * xDir;
			int yPos = character.getFootY() + rand.nextInt(128) * yDir;
			character.findPath(xPos, yPos);
			ai.setEngagedEnemy(null);
		}
	}
}
