/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.ai.pathfinding;

import java.util.List;

import org.lwjgl.util.Point;

/**
 * This interface defines a specific pathfinding algorithm and is used by the
 * {@link PathfindingManager}
 * 
 * @author Drag-On
 * 
 */
public interface IPathfinder
{
	/**
	 * Calculates the path from <code>start</code> to <code>end</code>.
	 * 
	 * @param start
	 *            Start point
	 * @param end
	 *            Target point
	 * @return A list containing the steps to do to move from <code>start</code>
	 *         to <code>end</code> in the right order or <code>null</code> in
	 *         case there is no valid path to the target node.
	 */
	public List<Point> findPath(Point start, Point end);

	/**
	 * Is called when the underlying map has changed
	 * 
	 * @param map
	 *            The map to get data about the environment from
	 */
	public void update(IPathfindingMap map);
}
