/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.ai.pathfinding;

/**
 * Interface that must be implemented by maps that use pathfinding from the
 * {@link PathfindingManager}
 * 
 * @author Drag-On
 * 
 */
public interface IPathfindingMap
{
	/**
	 * Checks if the node with the passed coordinates is passable
	 * 
	 * @param x
	 *            X coordinate to check
	 * @param y
	 *            Y coordinate to check
	 * @return <code>true</code> in case the specified node is passable
	 *         otherwise <code>false</code>.
	 */
	public boolean isPassable(int x, int y);

	/**
	 * Returns the penalty for the node with the passed coordinates. This
	 * influences the probability of entities moving onto this node.
	 * 
	 * @param x
	 *            X node coordinate
	 * @param y
	 *            Y node coordinate
	 * @return The penalty to move to the node with the passed coordinates
	 */
	public int getNodePenalty(int x, int y);

	/**
	 * @return Horizontal amount of pathfinding nodes
	 */
	public int getWidth();

	/**
	 * @return Vertical amount of pathfinding ndoes
	 */
	public int getHeight();
}
