/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.ai.pathfinding;

import java.util.LinkedList;
import java.util.List;

import org.lwjgl.util.Point;

/**
 * The PathfindingManager is intended to do all the pathfinding. It needs to be
 * updated regularly as it distributes each pathfinding request on one update
 * cycle in order to achieve a uniform processor load.
 * 
 * @author Drag-On
 * 
 */
public class PathfindingManager
{
	/**
	 * Actual pathfinding implementation
	 */
	private final IPathfinder pathfinder;

	/**
	 * Holds all current Pathfinding requests, oldest first
	 */
	private final List<PathfindingRequest> requests;

	/**
	 * Constructor
	 * 
	 * @param map
	 *            Reference to map object
	 */
	public PathfindingManager(final IPathfindingMap map)
	{
		this.pathfinder = new AStarPathfinder(map);
		this.requests = new LinkedList<PathfindingRequest>();
	}

	/**
	 * Updates the pathfinding
	 */
	public void update()
	{
		// If there are pending pathfinding requests execute the first one
		if (!requests.isEmpty())
		{
			PathfindingRequest first = requests.get(0);
			List<Point> path = pathfinder.findPath(new Point(first.getStartX(),
					first.getStartY()),
					new Point(first.getTargetX(), first.getTargetY()));
			first.getObject().setPath(path);
			requests.remove(0);
		}
	}

	/**
	 * Adds a new request for pathfinding
	 * 
	 * @param request
	 *            Request data
	 */
	public void addRequest(final PathfindingRequest request)
	{
		this.requests.add(0, request);
	}
}
