/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		 \u00a9 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.data;

import game.GameHandler;
import game.resources.ResourceProvider;
import game.resources.ResourceType;
import game.resources.TextResource;
import game.widget.Composite;
import game.widget.TextWidget;
import game.widget.Widget;

import org.lwjgl.util.Rectangle;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.state.StateBasedGame;

/**
 * This class handles an info text that is displayed on screen
 * 
 * @author Drag-On
 * 
 */
public class InfoMessage
{
	/**
	 * Compsite to display the text in
	 */
	private final Composite comp;

	/**
	 * Text widget
	 */
	private final TextWidget textWidget;

	/**
	 * Destination y coordinate
	 */
	private final int toY;

	/**
	 * Current y coordinate. This is necessary because the composite coordinate
	 * is only saved as an integer
	 */
	private float curY;

	/**
	 * Moving speed
	 */
	private static final float speed = 0.02f;

	/**
	 * Constructor
	 * 
	 * @param msg
	 *            Text to display
	 * @param yOffset
	 *            Amount of pixels to offset the start position of the message
	 *            (e.g. when there are more than one message on screen)
	 */
	public InfoMessage(final String msg, final int yOffset)
	{
		TextResource textRes = ResourceProvider.getInstance().get("MainTexts",
				ResourceType.TEXT);
		int w = GameHandler.getInstance().getContainer().getWidth();
		int h = GameHandler.getInstance().getContainer().getHeight();
		this.curY = h / 2 + yOffset;
		this.comp = new Composite(new Rectangle(5, (int) curY, w, h),
				"TextSmall", "defaultSkin", 1, Widget.NO_BG);
		this.toY = h / 2 - 100 + yOffset;
		textWidget = new TextWidget(comp, textRes.replaceKeys(msg), 1,
				Widget.WORD_WRAP);
		this.textWidget.getTextColor().a = 0;
	}

	/**
	 * Update
	 * 
	 * @param gc
	 * @param state
	 * @param delta
	 */
	public void update(final GameContainer gc, final StateBasedGame state,
			final int delta)
	{
		if (!isFinished())
		{
			this.curY -= (InfoMessage.speed * delta);

			if (this.curY <= this.toY + 40)
			{
				float alphaPos = curY - this.toY;
				this.textWidget.getTextColor().a = alphaPos / 40.0f;
			}
			else if (this.curY >= this.toY + 60)
			{
				float alphaPos = this.toY + 100 - curY;
				this.textWidget.getTextColor().a = alphaPos / 40.0f;
			}

			this.comp.getBounds().setY((int) this.curY);
			this.comp.update(gc, state, delta);
		}
	}

	/**
	 * Render everything to screen
	 * 
	 * @param container
	 * @param game
	 * @param g
	 */
	public void render(final GameContainer container,
			final StateBasedGame game, final Graphics g)
	{
		if (!isFinished())
		{
			this.comp.render(container, game, g);
		}
	}

	/**
	 * @return Difference to default start position of info messages
	 */
	public int getStartPosDifference()
	{
		int h = GameHandler.getInstance().getContainer().getHeight();
		return (h / 2) - this.comp.getBounds().getY();
	}

	/**
	 * @return The height of this message
	 */
	public int getHeight()
	{
		return this.textWidget.getTextHeight();
	}

	/**
	 * @return <code>true</code> in case the info message can be deleted,
	 *         otherwise <code>false</code>
	 */
	public boolean isFinished()
	{
		return this.comp.getBounds().getY() <= this.toY;
	}
}
