/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.dialog;

import game.GameHandler;

import java.util.List;

import org.newdawn.slick.util.Log;

import bsh.EvalError;
import bsh.Interpreter;

/**
 * This is an abstract dialog node which has a condition
 * 
 * @author Drag-On
 * 
 */
public abstract class AbstractDialogConditionalNode extends AbstractDialogNode
{
	/**
	 * Condition to execute dialog
	 */
	private String condition = "true";

	/**
	 * Constructor
	 * 
	 * @param childNodes
	 *            Child nodes
	 * @param condition
	 *            Condition that has to be met
	 */
	public AbstractDialogConditionalNode(final List<IDialogNode> childNodes,
			final String condition)
	{
		super(childNodes);
		this.condition = condition;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean canExecute()
	{
		if (this.condition != null && this.condition.trim() != "")
		{
			// Interpret condition
			Interpreter interpreter = GameHandler.getInterpreter();
			boolean value = false;
			try
			{
				interpreter.set("player", getMap().getPlayerCharacter());
				interpreter.set("inventory", getMap().getPlayerCharacter()
						.getInventory());
				interpreter.set("questlog", getQuestlog());
				interpreter.set("map", getMap());
				interpreter.set("variables", getMap().getGameVariables());
				if (getNPC() != null)
					interpreter.set("npc", getNPC());
				value = (boolean) interpreter.eval(this.condition);
			}
			catch (EvalError e)
			{
				Log.error("Error while parsing condition of dialog node: "
						+ this.condition, e);
			}
			return value;
		}
		else
			return true;
	}
}
