/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.dialog;

import game.GameHandler;
import game.states.MapState;
import game.states.map.MapData;
import game.states.map.entity.NonPlayerCharacter;
import game.states.questlog.Questlog;

import java.util.List;

/**
 * @author Drag-On
 * 
 */
public abstract class AbstractDialogNode implements IDialogNode
{
	/**
	 * A list containing all choices of this selection node
	 */
	protected List<IDialogNode> children = null;

	/**
	 * Parent dialog node
	 */
	private IDialogNode parent = null;

	/**
	 * NPC this dialog node belongs to
	 */
	private NonPlayerCharacter npc = null;

	/**
	 * Constructor
	 * 
	 * @param children
	 *            Child nodes
	 */
	public AbstractDialogNode(final List<IDialogNode> children)
	{
		this.children = children;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void init(final NonPlayerCharacter npc)
	{
		// No initialization necessary unless there are variables which need to
		// have a certain state before the dialog gets executed

		if (npc != null)
			this.npc = npc;

		GameHandler.getInstance().getContainer().getInput()
				.clearKeyPressedRecord();
		GameHandler.getInstance().getContainer().getInput()
				.clearControlPressedRecord();
		GameHandler.getInstance().getContainer().getInput()
				.clearMousePressedRecord();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean isFinished()
	{
		return !canExecute();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public List<IDialogNode> getChildNodes()
	{
		return children;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public IDialogNode getParent()
	{
		return parent;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setParent(final IDialogNode parent)
	{
		this.parent = parent;
	}

	/**
	 * @return the npc
	 */
	@Override
	public NonPlayerCharacter getNPC()
	{
		return npc;
	}

	/**
	 * Convenience method to get the map data
	 * 
	 * @return The current map
	 */
	protected MapData getMap()
	{
		return ((MapState) game.GameHandler.getInstance().getState(
				GameHandler.STATE_MAP)).getMapData();
	}

	/**
	 * Convenience method to get the questlog
	 * 
	 * @return The current questlog
	 */
	protected Questlog getQuestlog()
	{
		return getMap().getPlayerCharacter().getQuestlog();
	}
}
