/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.dialog;

import game.GameHandler;
import game.states.map.entity.NonPlayerCharacter;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.util.Log;

import bsh.EvalError;
import bsh.Interpreter;

/**
 * This is a dialog command node which has no visual representation but
 * instantly executes some code
 * 
 * @author Drag-On
 * 
 */
public class DialogCommandNode extends AbstractDialogNode
{
	/**
	 * String containing the commands
	 */
	private String command = "";

	/**
	 * Indicating the command has been processed
	 */
	private boolean finished = false;

	/**
	 * Constructor
	 * 
	 * @param command
	 *            Commands to execute
	 */
	public DialogCommandNode(final String command)
	{
		super(null);
		this.command = command;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void init(final NonPlayerCharacter npc)
	{
		super.init(getNPC());

		this.finished = false;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void update(final GameContainer container,
			final StateBasedGame game, final int delta)
	{
		if (!this.finished)
		{
			// Execute commands
			Interpreter interpreter = GameHandler.getInterpreter();
			try
			{
				interpreter.set("player", getMap().getPlayerCharacter());
				interpreter.set("inventory", getMap().getPlayerCharacter()
						.getInventory());
				interpreter.set("questlog", getQuestlog());
				if (getNPC() != null)
					interpreter.set("npc", getNPC());
				interpreter.set("map", getMap());
				interpreter.set("variables", getMap().getGameVariables());
				interpreter.eval(this.command);
				this.finished = true;
			}
			catch (EvalError e)
			{
				Log.error("Error while parsing command of dialog node: "
						+ this.command, e);
			}
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void render(final GameContainer container,
			final StateBasedGame game, final Graphics g)
	{
		// Nothing to do over here
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean canExecute()
	{
		return true;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean isFinished()
	{
		return this.finished;
	}
}
