/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.dialog;

import java.util.List;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.state.StateBasedGame;

/**
 * This class represents the root node of a dialog which is only used for
 * checking the main dialog condition
 * 
 * @author Drag-On
 * 
 */
public class DialogRootNode extends AbstractDialogConditionalNode
{
	/**
	 * Constructor
	 * 
	 * @param condition
	 *            Condition that must be met to execute the dialog
	 * @param nextNodes
	 *            Next dialog nodes to execute after this one
	 */
	public DialogRootNode(final String condition,
			final List<IDialogNode> nextNodes)
	{
		super(nextNodes, condition);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void update(final GameContainer container,
			final StateBasedGame game, final int delta)
	{
		// Root nodes do not need to be updated because they finish instantly
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void render(final GameContainer container,
			final StateBasedGame game, final Graphics g)
	{
		// Don't do anything over here as root nodes do not have any visible
		// information
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean isFinished()
	{
		// Root nodes finish instantly as they do not contain any visible
		// information
		return true;
	}
}
