/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.dialog;

import game.GameHandler;
import game.resources.ResourceProvider;
import game.resources.ResourceType;
import game.resources.SpritesheetResource;
import game.resources.TextResource;
import game.states.map.entity.NonPlayerCharacter;
import game.widget.ButtonWidget;
import game.widget.Composite;
import game.widget.Widget;
import game.widget.event.SelectionEvent;
import game.widget.event.SelectionEventListener;

import java.util.List;

import org.lwjgl.util.Rectangle;
import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.util.Log;

/**
 * This class implements a dialog node that allows the palyer to select a
 * certain answer
 * 
 * @author Drag-On
 * 
 */
public class DialogSelectionNode extends AbstractDialogNode
{
	/**
	 * Composite to display the message in
	 */
	private Composite comp = null;

	/**
	 * ID of the choice the player has clicked on or negative numbers in case no
	 * decision has been made yet
	 */
	private int finished = -1;

	/**
	 * Constructor
	 * 
	 * @param children
	 *            List of possible choices
	 */
	public DialogSelectionNode(final List<IDialogNode> children)
	{
		super(children);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void init(final NonPlayerCharacter npc)
	{
		super.init(npc);
		this.finished = -1;
		initComp();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void update(final GameContainer container,
			final StateBasedGame game, final int delta)
	{
		// Update composite
		this.comp.update(container, game, delta);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void render(final GameContainer container,
			final StateBasedGame game, final Graphics g)
	{
		// Draw background
		SpritesheetResource backResource = ResourceProvider.getInstance().get(
				"DialogBG", ResourceType.SPRITESHEET);
		int width = 600;// backResource.getSpriteSheet().getWidth();
		int height = backResource.getSpriteSheet().getHeight();
		int x = (GameHandler.getInstance().getContainer().getWidth() - width) / 2;
		int y = GameHandler.getInstance().getContainer().getHeight() - height;
		backResource.getSpriteSheet().draw(x, y);

		this.comp.render(container, game, g);
		this.comp.postRenderState(container, g);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean canExecute()
	{
		return true;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean isFinished()
	{
		return this.finished >= 0;
	}

	/**
	 * Initializes the composite
	 */
	private void initComp()
	{
		// If there already is a composite (i.e. this dialog has already been
		// executed earlier) dispose it
		if (this.comp != null)
			this.comp.dispose();

		TextResource texts = ResourceProvider.getInstance().get("MainTexts",
				ResourceType.TEXT);

		// Initialize composite
		int width = 500;
		int height = 106;
		int x = (GameHandler.getInstance().getContainer().getWidth() - width) / 2;
		int y = GameHandler.getInstance().getContainer().getHeight() - height
				- 16;
		this.comp = new Composite(new Rectangle(x, y, width, height),
				"TextHeading", "defaultSkin", 1, Widget.NO_BG);
		for (IDialogNode choice : this.children)
		{
			if (choice instanceof DialogChoiceNode)
			{
				final DialogChoiceNode c = (DialogChoiceNode) choice;
				if (c.canExecute())
				{
					final ButtonWidget button = new ButtonWidget(comp,
							texts.replaceKeys(c.getMsg()), 1, Widget.NONE);
					button.setTextColor(Color.black);
					// Add a selection listener so we can notice what decision
					// the player has made
					button.addSelectionEventListener(new SelectionEventListener()
					{
						@Override
						public void selected(final SelectionEvent evt)
						{
							DialogSelectionNode.this.finished = children
									.indexOf(c);
							GameHandler.getInstance().getContainer().getInput()
									.clearKeyPressedRecord();
						}
					});
				}
			}
			else
			{
				Log.warn("Misformatted dialog file! Dialog node of type "
						+ choice.getClass().getName()
						+ " inside of a selection node.");
			}
		}
		this.comp.layout();
	}

	/**
	 * @return the finish ID
	 */
	public int getFinishID()
	{
		return this.finished;
	}
}
