/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.dialog;

import game.states.map.entity.NonPlayerCharacter;

import java.util.List;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.state.StateBasedGame;

/**
 * This defines a general interface for dialog nodes such as messages or
 * selection-boxes
 * 
 * @author Drag-On
 * 
 */
public interface IDialogNode
{
	/**
	 * Initializes this and all child nodes
	 * 
	 * @param npc
	 *            The npc the dialog belongs to
	 */
	public void init(NonPlayerCharacter npc);

	/**
	 * Updates the dialog node. Called once per update cycle and right after
	 * initialization
	 * 
	 * @param container
	 *            Game container
	 * @param game
	 *            Game handler
	 * @param delta
	 *            Delta time since last update
	 */
	public void update(final GameContainer container,
			final StateBasedGame game, final int delta);

	/**
	 * Renders the dialog node. Called once per frame
	 * 
	 * @param container
	 *            Game container
	 * @param game
	 *            Game handler
	 * @param g
	 *            Graphics context
	 */
	public void render(final GameContainer container,
			final StateBasedGame game, final Graphics g);

	/**
	 * Determines if this dialog node can be executed at all
	 * 
	 * @return <code>true</code> in case it can be executed, otherwise
	 *         <code>false</code>.
	 */
	public boolean canExecute();

	/**
	 * @return <code>true</code> in case this dialog node can be closed,
	 *         otherwise <code>false</code>.
	 */
	public boolean isFinished();

	/**
	 * @return A list of child dialog nodes to handle after this one has
	 *         finished
	 */
	public List<IDialogNode> getChildNodes();

	/**
	 * @return The parent of this dialog node or <code>null</code> in case this
	 *         is the root
	 */
	public IDialogNode getParent();

	/**
	 * @param parent
	 *            Parent to set
	 */
	public void setParent(IDialogNode parent);

	/**
	 * @return The NPC this dialog belongs to
	 */
	public NonPlayerCharacter getNPC();
}
