/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.entity;

import game.resources.ResourceType;
import game.states.map.Viewport;
import game.states.map.zbufferlist.IZBufferListItem;

import org.lwjgl.util.Point;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.SpriteSheet;
import org.newdawn.slick.state.StateBasedGame;

/**
 * This is an abstract game entity which has a two-dimensional extent and can be
 * drawn.
 * 
 * @author Drag-On
 * 
 */
public abstract class DrawableGameEntity extends GameEntity implements
		IZBufferListItem
{
	/**
	 * Serial Version UID
	 */
	private static final long serialVersionUID = 1997683413079890444L;

	/**
	 * ID of the resource to use for this entity
	 */
	protected String resourceId;

	/**
	 * Next element in the list
	 */
	private IZBufferListItem next = null;

	/**
	 * Previous element in the list
	 */
	private IZBufferListItem previous = null;

	/**
	 * Constructor
	 * 
	 * @param x
	 *            X coordinate of this object in pixels
	 * @param y
	 *            Y coordinate of this object in pixels
	 * @param width
	 *            Width of this object in pixels
	 * @param height
	 *            Height of this object in pixels
	 * @param resourceId
	 *            ID of the resource to use
	 */
	public DrawableGameEntity(final int x, final int y, final int width,
			final int height, final String resourceId)
	{
		super(x, y, width, height);
		this.resourceId = resourceId;
	}

	/**
	 * Updates this object.
	 * 
	 * @param gc
	 *            GameContainer of the game
	 * @param state
	 *            Current game state
	 * @param delta
	 *            Delta time since last update
	 */
	@Override
	public abstract void update(final GameContainer gc,
			final StateBasedGame state, final int delta);

	/**
	 * Render this game entity to where it belongs
	 * 
	 * @param container
	 *            Actual game container
	 * @param game
	 *            Game object
	 * @param g
	 *            Graphics context
	 * @param viewport
	 *            Current map viewport
	 * @param lockedRes
	 *            Locked resource for rendering
	 */
	public abstract void render(final GameContainer container,
			final StateBasedGame game, final Graphics g, Viewport viewport,
			SpriteSheet lockedRes);

	/**
	 * {@inheritDoc}
	 */
	public Point getPos()
	{
		return new Point(getX(), getY());
	}

	/**
	 * {@inheritDoc}
	 */
	public Point getSize()
	{
		return new Point(getWidth(), getHeight());
	}

	/**
	 * @return The z coordinate used for rendering
	 */
	@Override
	public int getZ()
	{
		return (int) (this.y + this.height);
	}

	/**
	 * @return the resourceId
	 */
	public String getResourceId()
	{
		return resourceId;
	}

	/**
	 * @param resourceId
	 *            the resourceId to set
	 */
	public void setResourceId(final String resourceId)
	{
		this.resourceId = resourceId;
	}

	/**
	 * @return The resource type the resource ID of this entity refers to
	 */
	public abstract ResourceType getResourceType();

	/**
	 * {@inheritDoc}
	 */
	@Override
	public IZBufferListItem getNext()
	{
		return this.next;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setNext(final IZBufferListItem next)
	{
		this.next = next;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public IZBufferListItem getPrevious()
	{
		return this.previous;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setPrevious(final IZBufferListItem previous)
	{
		this.previous = previous;
	}
}
