/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		 \u00a9 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.entity;

import game.resources.ResourceProvider;
import game.resources.ResourceType;
import game.states.inventory.InventoryItem;
import game.states.map.Viewport;

import java.util.Random;

import org.lwjgl.util.Rectangle;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.SpriteSheet;
import org.newdawn.slick.state.StateBasedGame;

/**
 * A collectible item
 * 
 * @author Drag-On
 * 
 */
public class ItemGameEntity extends CollidableGameEntity
{
	/**
	 * Serial Version UID
	 */
	private static final long serialVersionUID = 5777673102185511757L;

	/**
	 * The item object that is represented by this entity
	 */
	private final InventoryItem item;

	/**
	 * Y bobble value
	 */
	private float yBobble = 0;

	/**
	 * Indicating that the item is currently bobbling up
	 */
	private boolean bubbleUp = true;

	/**
	 * Constructor
	 * 
	 * @param x
	 *            X coordinate
	 * @param y
	 *            Y coordinate
	 * @param item
	 *            Item to display
	 */
	public ItemGameEntity(final int x, final int y, final InventoryItem item)
	{
		super(x, y, 32, 32, item.getResource().getParameter("resourceID"));
		this.item = item;
		Random rand = new Random();
		this.yBobble = rand.nextFloat() * 10;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void onCollision(final float lastX, final float lastY,
			final Rectangle collisionRect, final CollidableGameEntity other)
	{
		/*
		 * if (other instanceof PlayerCharacter) { MapState mapState =
		 * (MapState) GameHandler.getInstance().getState(
		 * GameHandler.STATE_MAP);
		 * mapState.getMapData().removeDrawableEntity(this);
		 * mapState.getMapData().getPlayerCharacter().getInventory()
		 * .addItem(this.item.getResourceID());
		 * 
		 * // Player pick up sfx AudioPlayer.playSFX("Pick"); }
		 */
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void update(final GameContainer gc, final StateBasedGame state,
			final int delta)
	{
		super.update(gc, state, delta);

		if (this.bubbleUp && this.yBobble < 10)
		{
			this.yBobble += 0.01f * delta;
		}
		else if (this.bubbleUp && this.yBobble >= 10)
		{
			this.bubbleUp = false;
		}
		if (!this.bubbleUp && this.yBobble > 0)
		{
			this.yBobble -= 0.01f * delta;
		}
		else if (!this.bubbleUp && this.yBobble <= 0)
		{
			this.bubbleUp = true;
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void render(final GameContainer container,
			final StateBasedGame game, final Graphics g,
			final Viewport viewport, final SpriteSheet lockedRes)
	{
		Image sprite = lockedRes.getSubImage(item.getResource()
				.getParameterAsInt("tileX"), item.getResource()
				.getParameterAsInt("tileY"));
		sprite.draw(getX() - viewport.getPosition().getX(), getY() - yBobble
				- viewport.getPosition().getY(), 1);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public ResourceType getResourceType()
	{
		return ResourceType.SPRITESHEET;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void requestResources()
	{
		ResourceProvider.getInstance().get(
				this.item.getResource().getParameter("resourceID"),
				ResourceType.SPRITESHEET);
	}

	/**
	 * @return the item
	 */
	public InventoryItem getItem()
	{
		return item;
	}
}
