/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.entity;

import game.resources.DialogResource;
import game.resources.ResourceProvider;
import game.resources.ResourceType;
import game.states.map.dialog.IDialogNode;

/**
 * Class used for NPCs the player can interact with
 * 
 * @author Drag-On
 * 
 */
public class NonPlayerCharacter extends BaseCharacter
{
	/**
	 * Serial Version UID
	 */
	private static final long serialVersionUID = -8676855801380467153L;

	/**
	 * Resource ID of the dialog to use for this NPC
	 */
	private String dialogID = null;

	/**
	 * Name of this character
	 * 
	 * @deprecated This is only here for compatibility issues with old map
	 *             files!
	 */
	@Deprecated
	private String name = "";

	/**
	 * Constructor
	 * 
	 * @param x
	 *            Initial x coordinate on map in pixels
	 * @param y
	 *            Initial y coordinate on map in pixels
	 * @param resId
	 *            ID of the resource to use for this character
	 * @param dialogID
	 *            Resource ID of the dialog to use for this character or
	 *            <code>null</code> in case there is no dialog to use for this
	 *            character
	 */
	public NonPlayerCharacter(final int x, final int y, final String resId,
			final String dialogID)
	{
		super(x, y, resId);
		this.dialogID = dialogID;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void requestResources()
	{
		super.requestResources();

		if (this.dialogID != null && !dialogID.trim().isEmpty())
		{
			ResourceProvider.getInstance().get(dialogID, ResourceType.DIALOG);
		}
	}

	/**
	 * @return the rootDialogNode
	 */
	public IDialogNode getRootDialogNode()
	{
		if (this.dialogID != null && this.dialogID != "")
		{
			DialogResource res = ResourceProvider.getInstance().get(
					this.dialogID, ResourceType.DIALOG, true);
			return res.getDialogRoot();
		}
		else
			return null;
	}

	/**
	 * @return the dialogID
	 */
	public String getDialogID()
	{
		return dialogID;
	}

	/**
	 * @param dialogID
	 *            the dialogID to set
	 */
	public void setDialogID(final String dialogID)
	{
		this.dialogID = dialogID;
	}

	/**
	 * @return the name
	 */
	@Override
	public String getName()
	{
		return this.name != "" ? this.name : super.getName();
	}

	/**
	 * @param name
	 *            the name to set
	 */
	@Override
	public void setName(final String name)
	{
		this.name = name;
		super.setName(name);
	}
}
