/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.entity.triggers;

import game.states.map.entity.GameEntity;
import game.states.map.entity.PlayerCharacter;
import game.states.map.entity.TriggerZoneEntity;

import org.newdawn.slick.util.Log;

/**
 * This is a generic trigger which teleports the player to a new map. The new
 * map id is taken from additional data in the shape: "mapID,tileX,tileY".
 * 
 * @author Drag-On
 * 
 */
public class TeleportTrigger extends TriggerZoneEntity
{
	/**
	 * Serial Version UID
	 */
	private static final long serialVersionUID = -5851475044442261330L;

	/**
	 * Constructor
	 * 
	 * @param x
	 *            X coordinate on map in pixels
	 * @param y
	 *            Y coordinate on map in pixels
	 * @param width
	 *            Width in pixels
	 * @param height
	 *            Height in pixels
	 */
	public TeleportTrigger(final int x, final int y, final int width,
			final int height)
	{
		super(x, y, width, height);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void onTrigger(final GameEntity activator)
	{
		super.onTrigger(activator);

		try
		{
			String[] data = getAdditionalData().split(",");
			final int mapID = Integer.parseInt(data[0]);
			final int teleportX = Integer.parseInt(data[1]);
			final int teleportY = Integer.parseInt(data[2]);

			getMap().teleportPlayer(mapID, teleportX, teleportY);
		}
		catch (Exception e)
		{
			Log.error("Teleport trigger on map " + getMap().getUID() + ": \""
					+ getMap().getName()
					+ "\" has no sufficient additional data.", e);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void onEntityLeft(final GameEntity activator)
	{
		super.onEntityLeft(activator);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean canTrigger(final GameEntity activator)
	{
		return super.canTrigger(activator)
				&& activator instanceof PlayerCharacter;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean deleteOnTrigger()
	{
		return false;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void requestResources()
	{
		// No resources needed
	}
}
