/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.entity.triggers.map;

import game.GameHandler;
import game.states.MapState;
import game.states.map.MapData;

/**
 * A map trigger entity which will be called every update cycle
 * 
 * @author Drag-On
 * 
 */
public abstract class AbstractMapTriggerEntity
{
	/**
	 * String containing some user data to pass to the map trigger
	 */
	private String additionalData = null;

	/**
	 * Called upon map initialization
	 */
	public abstract void onInit();

	/**
	 * This method will be executed once every update cycle
	 * 
	 * @param delta
	 *            Delta time since last update
	 */
	public abstract void onTrigger(final int delta);

	/**
	 * Provides access to the map object
	 * 
	 * @return The current map object
	 */
	protected MapData getMap()
	{
		return ((MapState) GameHandler.getInstance().getState(
				GameHandler.STATE_MAP)).getMapData();
	}

	/**
	 * @return the additionalData
	 */
	public String getAdditionalData()
	{
		return additionalData;
	}

	/**
	 * @param additionalData
	 *            the additionalData to set
	 */
	public void setAdditionalData(final String additionalData)
	{
		this.additionalData = additionalData;
	}
}
