/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.map.zbufferlist;

import game.resources.TileData;

/**
 * @author Drag-On
 * 
 */
public class ZBufferListTileItem implements IZBufferListItem
{
	/**
	 * Map tile coordinates
	 */
	private final int mapX, mapY, mapZ;

	/**
	 * Tile information
	 */
	private final TileData tileData;

	/**
	 * Resource ID of the graphics file
	 */
	private final String resourceID;

	/**
	 * Next element in the list
	 */
	private IZBufferListItem next = null;

	/**
	 * Previous element in the list
	 */
	private IZBufferListItem previous = null;

	/**
	 * Constructor
	 * 
	 * @param mapX
	 *            X tile coordinate on map
	 * @param mapY
	 *            Y tile coordinate on map
	 * @param mapZ
	 *            Z tile coordinate
	 * @param tileData
	 *            {@link TileData} object
	 * @param resId
	 *            Resource ID of the tileset where the tile is from
	 */
	public ZBufferListTileItem(final int mapX, final int mapY, final int mapZ,
			final TileData tileData, final String resId)
	{
		this.mapX = mapX;
		this.mapY = mapY;
		this.mapZ = mapZ;
		this.tileData = tileData;
		this.resourceID = resId;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int getX()
	{
		return this.mapX;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int getY()
	{
		return this.mapY;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int getZ()
	{
		return this.mapZ;
	}

	/**
	 * @return the tileData
	 */
	public TileData getTileData()
	{
		return tileData;
	}

	/**
	 * @return the resourceID
	 */
	public String getResourceID()
	{
		return resourceID;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public IZBufferListItem getNext()
	{
		return this.next;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setNext(final IZBufferListItem next)
	{
		this.next = next;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public IZBufferListItem getPrevious()
	{
		return this.previous;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setPrevious(final IZBufferListItem previous)
	{
		this.previous = previous;
	}
}
