/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.questlog;

import game.resources.QuestResource;
import game.resources.ResourceProvider;
import game.resources.ResourceType;
import game.resources.TextResource;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * One entry in the questlog
 * 
 * @author Drag-On
 * 
 */
public class QuestlogItem implements Serializable
{
	/**
	 * Serial Version UID
	 */
	private static final long serialVersionUID = -5572240665764568979L;

	/**
	 * Unique identifier of this quest
	 */
	private final String questID;

	/**
	 * Open sub quest IDs
	 */
	private final List<String> open;

	/**
	 * Finished sub quest IDs
	 */
	private final List<String> closed;

	/**
	 * Marks this quest as solved
	 */
	private boolean finished;

	/**
	 * Constructor
	 * 
	 * @param questID
	 *            ID of the quest
	 */
	public QuestlogItem(final String questID)
	{
		this.questID = questID;
		this.open = new ArrayList<String>();
		this.closed = new ArrayList<String>();
		this.finished = false;
	}

	/**
	 * @return the questID
	 */
	public String getQuestID()
	{
		return questID;
	}

	/**
	 * @return the open
	 */
	public List<String> getOpen()
	{
		return open;
	}

	/**
	 * Adds a new sub quest to this entry
	 * 
	 * @param id
	 *            Internal ID of the sub quest
	 */
	public void addSubQuest(final String id)
	{
		if (!this.open.contains(id))
			this.open.add(id);
	}

	/**
	 * @return the closed
	 */
	public List<String> getClosed()
	{
		return closed;
	}

	/**
	 * Marks a specific sub quest as finished
	 * 
	 * @param id
	 *            Internal ID of the sub quest
	 */
	public void closeSubQuest(final String id)
	{
		this.open.remove(id);
		this.closed.add(id);
	}

	/**
	 * @return the finished
	 */
	public boolean isFinished()
	{
		return finished;
	}

	/**
	 * Marks this quest as solved
	 */
	public void finish()
	{
		this.finished = true;
	}

	/**
	 * @return Quest name
	 */
	public String getName()
	{
		QuestResource q = ResourceProvider.getInstance().get(getQuestID(),
				ResourceType.QUEST, true);
		TextResource texts = ResourceProvider.getInstance().get("MainTexts",
				ResourceType.TEXT, true);
		return texts.replaceKeys(q.getName());
	}

	/**
	 * @return Quest description
	 */
	public String getDescription()
	{
		QuestResource q = ResourceProvider.getInstance().get(getQuestID(),
				ResourceType.QUEST, true);
		TextResource texts = ResourceProvider.getInstance().get("MainTexts",
				ResourceType.TEXT, true);
		return texts.replaceKeys(q.getDescription());
	}
}
