/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		 \u00a9 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.states.variables;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

/**
 * Game variables of any kind that may be manipulated by dialogs and triggers.
 * 
 * @author Drag-On
 * 
 */
public class GameVariables implements Serializable
{
	/**
	 * Serial Version UID
	 */
	private static final long serialVersionUID = -4860959717972299532L;

	/**
	 * A map containing all the variable name-value bindings
	 */
	private final Map<String, Object> variables;

	/**
	 * Constructor
	 */
	public GameVariables()
	{
		this.variables = new HashMap<String, Object>();
	}

	/**
	 * Sets a game variable
	 * 
	 * @param name
	 *            Name of the variable to set
	 * @param value
	 *            New value of the variable with the specified name
	 */
	public void set(final String name, final Object value)
	{
		this.variables.put(name, value);
	}

	/**
	 * Gets a game variable by it's name
	 * 
	 * @param name
	 *            Name of the variable to get
	 * @return The variable's value or <code>null</code> in case there is no
	 *         variable with this name known
	 */
	public Object get(final String name)
	{
		return this.variables.get(name);
	}
}
