/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		© 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.widget;

import game.widget.event.SelectionEvent;

import org.lwjgl.util.Rectangle;

/**
 * A list item which can be hold by {@link ListWidget} instances.
 * 
 * @author Drag-On
 * 
 */
public class ListItemWidget extends ButtonWidget
{
	/**
	 * Coordinates of this item in the list. This will be set automatically when
	 * this item is added to a list.
	 */
	private int x, y;

	/**
	 * Constructor
	 * 
	 * @param parent
	 *            Parent list
	 * @param text
	 *            Text content of this item
	 * @param style
	 *            Style value
	 */
	public ListItemWidget(final ListWidget parent, final String text,
			final int style)
	{
		super(parent, text, parent.computeItemColumnSpan(), style);
	}

	/**
	 * @return the parent
	 */
	@Override
	public ListWidget getParent()
	{
		return (ListWidget) super.getParent();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void fireSelectionEvent(final SelectionEvent evt)
	{
		evt.setX(x);
		evt.setY(y);
		super.fireSelectionEvent(evt);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Rectangle getAbsoluteHoverBounds()
	{
		Rectangle bounds = getAbsoluteBounds();
		if ((getParent().getStyle() & Widget.FULL_SELECTION) != 0)
		{
			Rectangle parentBounds = getParent().getAbsoluteBounds();
			if (parentBounds.getX() + getParent().getMarginLeft() < bounds
					.getX())
				bounds.setX(parentBounds.getX() + getParent().getMarginLeft());
			if (parentBounds.getWidth() - getParent().getMarginLeft()
					- getParent().getMarginRight() > bounds.getWidth())
				bounds.setWidth(parentBounds.getWidth()
						- getParent().getMarginLeft()
						- getParent().getMarginRight());
		}
		return bounds;
	}

	/**
	 * Sets the coordinates of this item inside the list. Should be called only
	 * by the list containing this item!
	 * 
	 * @param x
	 *            X coordinate
	 * @param y
	 *            Y coordinate
	 */
	protected void setCoordinates(final int x, final int y)
	{
		this.x = x;
		this.y = y;
	}
}
