/******************************************************
 ***
 ** Project 'Roleplay'
 **		Hochschule Furtwangen University
 **		 \u00a9 2013
 ***
 *** 
 ** GameLib
 ** 	Drag-On
 ***
 *****************************************************/
package game.widget;

import game.GameHandler;

import org.lwjgl.input.Keyboard;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Input;
import org.newdawn.slick.KeyListener;
import org.newdawn.slick.state.StateBasedGame;

/**
 * A widget used for user input from keyboard
 * 
 * @author Drag-On
 * 
 */
public class TextInputWidget extends TextWidget
{
	/**
	 * Flag determining if this control is currently active
	 */
	private boolean active = false;

	/**
	 * Current position of the cursor
	 */
	private int cursorPos = 0;

	/**
	 * Key listener reference
	 */
	private KeyListener keyListener = null;

	/**
	 * Constructor
	 * 
	 * @param parent
	 *            Parent composite
	 * @param columnSpan
	 *            Amount of columns for this control to span on it's parent
	 * @param style
	 *            Style value
	 */
	public TextInputWidget(final Composite parent, final int columnSpan,
			final int style)
	{
		super(parent, "|", columnSpan, style);
		GameHandler.getInstance().getContainer().getInput()
				.addKeyListener(this.keyListener = new KeyListener()
				{
					@Override
					public void setInput(final Input input)
					{
					}

					@Override
					public boolean isAcceptingInput()
					{
						return TextInputWidget.this.active;
					}

					@Override
					public void inputStarted()
					{
					}

					@Override
					public void inputEnded()
					{
						// Input is only reactivated in case this control is
						// updated
						TextInputWidget.this.active = false;
					}

					@Override
					public void keyReleased(final int key, final char c)
					{
						if (key == Input.KEY_LEFT && cursorPos > 0)
						{
							TextInputWidget.this
									.setTextInternal(getTextInternal()
											.substring(0, cursorPos)
											+ getTextInternal().substring(
													cursorPos + 1));
							cursorPos--;
							TextInputWidget.this
									.setTextInternal(getTextInternal()
											.substring(0, cursorPos)
											+ "|"
											+ getTextInternal().substring(
													cursorPos));
						}
						else if (key == Input.KEY_RIGHT
								&& cursorPos < getTextInternal().length() - 1)
						{
							TextInputWidget.this
									.setTextInternal(getTextInternal()
											.substring(0, cursorPos)
											+ getTextInternal().substring(
													cursorPos + 1));
							cursorPos++;
							TextInputWidget.this
									.setTextInternal(getTextInternal()
											.substring(0, cursorPos)
											+ "|"
											+ getTextInternal().substring(
													cursorPos));
						}
						else if (key == Input.KEY_DELETE
								&& cursorPos < getTextInternal().length() - 1)
						{
							TextInputWidget.this
									.setTextInternal(getTextInternal()
											.substring(0, cursorPos + 1)
											+ getTextInternal().substring(
													cursorPos + 2));
						}
						else if (key == Input.KEY_BACK && cursorPos > 0)
						{
							cursorPos--;
							TextInputWidget.this
									.setTextInternal(getTextInternal()
											.substring(0, cursorPos)
											+ getTextInternal().substring(
													cursorPos + 1));
						}
						else if (c != Keyboard.CHAR_NONE
								&& key != Input.KEY_ESCAPE
								&& key != Input.KEY_BACK)
						{
							TextInputWidget.this
									.setTextInternal(getTextInternal()
											.substring(0, cursorPos)
											+ c
											+ getTextInternal().substring(
													cursorPos));
							cursorPos++;
						}
					}

					@Override
					public void keyPressed(final int key, final char c)
					{
					}
				});
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void update(final GameContainer gc, final StateBasedGame sb,
			final int delta)
	{
		// Activate input
		this.active = true;

		super.update(gc, sb, delta);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void dispose()
	{
		GameHandler.getInstance().getContainer().getInput()
				.removeKeyListener(keyListener);
	}

	/**
	 * Internal method for setting the text without needing to worry about the
	 * cursor
	 * 
	 * @param text
	 *            Text to set
	 */
	private void setTextInternal(final String text)
	{
		super.setText(text);
	}

	/**
	 * Internal method for getting the text without needing to worry about the
	 * cursor
	 * 
	 * @return The text
	 */
	private String getTextInternal()
	{
		return super.getText();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setText(final String text)
	{
		if (text.contains("|"))
			super.setText(text);
		else
		{
			super.setText(text + "|");
			this.cursorPos = text.length();
		}
	}

	/**
	 * @return the text
	 */
	@Override
	public String getText()
	{
		String text = super.getText().substring(0, cursorPos)
				+ super.getText().substring(cursorPos + 1);
		return text;
	}
}
