package data.container.binaryheap;

import junit.framework.TestCase;

import org.junit.Test;

public class BinaryHeapTest extends TestCase
{
	public class MyInteger implements Comparable<MyInteger>
	{
		public int i = 0;

		public MyInteger(int i)
		{
			this.i = i;
		}

		@Override
		public int compareTo(MyInteger o)
		{
			return i < o.i ? -1 : 1;
		}

		@Override
		public String toString()
		{
			return String.valueOf(i);
		}
	}

	@Test
	public void testAdd()
	{
		BinaryHeap<Integer> heap = new BinaryHeap<Integer>(Integer.class, 8);

		int num = 10;
		int index = heap.add(num);
		assertEquals(1, heap.getSize());
		assertEquals((Integer) num, heap.getFirst());
		assertEquals((Integer) num, heap.get(index));

		num = 30;
		index = heap.add(num);
		assertEquals(2, heap.getSize());
		assertEquals((Integer) 10, heap.getFirst());
		assertEquals((Integer) num, heap.get(index));

		num = 20;
		index = heap.add(num);
		assertEquals(3, heap.getSize());
		assertEquals((Integer) 10, heap.getFirst());
		assertEquals((Integer) num, heap.get(index));

		num = 34;
		index = heap.add(num);
		assertEquals(4, heap.getSize());
		assertEquals((Integer) 10, heap.getFirst());
		assertEquals((Integer) num, heap.get(index));

		num = 38;
		index = heap.add(num);
		assertEquals(5, heap.getSize());
		assertEquals((Integer) 10, heap.getFirst());
		assertEquals((Integer) num, heap.get(index));

		num = 30;
		index = heap.add(num);
		assertEquals(6, heap.getSize());
		assertEquals((Integer) 10, heap.getFirst());
		assertEquals((Integer) num, heap.get(index));

		num = 24;
		index = heap.add(num);
		assertEquals(7, heap.getSize());
		assertEquals((Integer) 10, heap.getFirst());
		assertEquals((Integer) num, heap.get(index));

		num = 17;
		index = heap.add(num);
		assertEquals(8, heap.getSize());
		assertEquals((Integer) 10, heap.getFirst());
		assertEquals((Integer) num, heap.get(index));

		// This should also work in case we exceed the size hint
		num = 3;
		index = heap.add(num);
		assertEquals(9, heap.getSize());
		assertEquals((Integer) 3, heap.getFirst());
		assertEquals((Integer) 10, heap.get(1));
		assertEquals((Integer) num, heap.get(index));
	}

	@Test
	public void testRemove()
	{
		BinaryHeap<Integer> heap = new BinaryHeap<Integer>(Integer.class, 8);

		// Add some elements
		heap.add(10);
		heap.add(30);
		heap.add(20);
		heap.add(34);
		heap.add(38);
		heap.add(30);
		heap.add(24);
		heap.add(17);

		assertEquals((Integer) 10, heap.removeFirst());
		assertEquals(7, heap.getSize());
		assertEquals((Integer) 17, heap.getFirst());
		assertEquals((Integer) 34, heap.get(3));

		assertEquals((Integer) 17, heap.removeFirst());
		assertEquals(6, heap.getSize());
		assertEquals((Integer) 20, heap.getFirst());
		assertEquals((Integer) 24, heap.get(2));

		// Try to remove more items than there are in the heap
		for (int i = 0; i < 7; i++)
		{
			heap.removeFirst();
		}
	}

	@Test
	public void testUpdate()
	{
		BinaryHeap<MyInteger> heap = new BinaryHeap<MyInteger>(MyInteger.class,
				8);

		MyInteger testInt = new MyInteger(34);

		// Add some elements
		heap.add(new MyInteger(10));
		heap.add(new MyInteger(17));
		heap.add(new MyInteger(20));
		heap.add(new MyInteger(24));
		heap.add(new MyInteger(30));
		heap.add(new MyInteger(30));
		heap.add(testInt);
		heap.add(new MyInteger(38));

		assertEquals(10, heap.get(0).i);
		assertEquals(17, heap.get(1).i);
		assertEquals(20, heap.get(2).i);
		assertEquals(24, heap.get(3).i);
		assertEquals(30, heap.get(4).i);
		assertEquals(30, heap.get(5).i);
		assertEquals(34, heap.get(6).i);
		assertEquals(38, heap.get(7).i);

		testInt.i = 19;

		heap.update(testInt);

		assertEquals(10, heap.get(0).i);
		assertEquals(17, heap.get(1).i);
		assertEquals(19, heap.get(2).i);
		assertEquals(24, heap.get(3).i);
		assertEquals(30, heap.get(4).i);
		assertEquals(30, heap.get(5).i);
		assertEquals(20, heap.get(6).i);
		assertEquals(38, heap.get(7).i);

		testInt.i = 25;

		heap.update(testInt);

		assertEquals(10, heap.get(0).i);
		assertEquals(17, heap.get(1).i);
		assertEquals(20, heap.get(2).i);
		assertEquals(24, heap.get(3).i);
		assertEquals(30, heap.get(4).i);
		assertEquals(30, heap.get(5).i);
		assertEquals(25, heap.get(6).i);
		assertEquals(38, heap.get(7).i);

		testInt.i = 6;

		heap.update(testInt);

		assertEquals(6, heap.get(0).i);
		assertEquals(17, heap.get(1).i);
		assertEquals(10, heap.get(2).i);
		assertEquals(24, heap.get(3).i);
		assertEquals(30, heap.get(4).i);
		assertEquals(30, heap.get(5).i);
		assertEquals(20, heap.get(6).i);
		assertEquals(38, heap.get(7).i);

		testInt.i = 40;

		heap.update(testInt);

		assertEquals(10, heap.get(0).i);
		assertEquals(17, heap.get(1).i);
		assertEquals(20, heap.get(2).i);
		assertEquals(24, heap.get(3).i);
		assertEquals(30, heap.get(4).i);
		assertEquals(30, heap.get(5).i);
		assertEquals(40, heap.get(6).i);
		assertEquals(38, heap.get(7).i);
	}
}
