package game.states.map;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import junit.framework.TestCase;

import org.junit.Test;
import org.lwjgl.util.Point;
import org.lwjgl.util.Rectangle;
import org.newdawn.slick.GameContainer;

public class ViewportTest extends TestCase
{
	@Test
	public void testGetVisibleTiles()
	{
		final GameContainer container = mock(GameContainer.class);
		when(container.getWidth()).thenReturn(800);
		when(container.getHeight()).thenReturn(600);

		// Test it with x = 0 px, y = 0 px
		final Viewport v = new Viewport(container, 0, 0);
		Rectangle rect = v.getVisibleTiles();
		assertEquals(0, rect.getX());
		assertEquals(0, rect.getY());
		assertEquals(25, rect.getWidth());
		assertEquals(19, rect.getHeight());

		// x = 5 px, y = 160 px
		v.getPosition().setLocation(5, 160);
		rect = v.getVisibleTiles();
		assertEquals(0, rect.getX());
		assertEquals(5, rect.getY());
		assertEquals(26, rect.getWidth());
		assertEquals(19, rect.getHeight());

		// x = -33 px, y = -64 px
		v.getPosition().setLocation(-33, -64);
		rect = v.getVisibleTiles();
		assertEquals(-2, rect.getX());
		assertEquals(-2, rect.getY());
		assertEquals(26, rect.getWidth());
		assertEquals(19, rect.getHeight());
	}

	@Test
	public void testGetTileCoordinates()
	{
		final GameContainer container = mock(GameContainer.class);
		when(container.getWidth()).thenReturn(800);
		when(container.getHeight()).thenReturn(600);

		// Test it with x = 0 px, y = 0 px
		final Viewport v = new Viewport(container, 0, 0);
		Point p = v.getTileCoordinates(new Point(0, 0));
		assertEquals(0, p.getX());
		assertEquals(0, p.getY());

		// x = 31 px, y = 0 px
		v.moveLeftRight(31);
		p = v.getTileCoordinates(new Point(0, 0));
		assertEquals(0, p.getX());
		assertEquals(0, p.getY());

		// x = 33 px, y = 0 px
		v.moveLeftRight(1);
		p = v.getTileCoordinates(new Point(0, 0));
		assertEquals(1, p.getX());
		assertEquals(0, p.getY());

		// x = 33 px, y = 0 px
		v.moveLeftRight(1);
		p = v.getTileCoordinates(new Point(31, 0));
		assertEquals(2, p.getX());
		assertEquals(0, p.getY());
	}
}
