package game.states.map.ai.pathfinding;

import java.util.List;

import junit.framework.TestCase;

import org.junit.Test;
import org.lwjgl.util.Point;

public class AStarPathfinderTest extends TestCase
{
	@Test
	public void testFindPath()
	{
		// Initialize some basic obstacles
		AStarPathfinder pathfinder = new AStarPathfinder(new IPathfindingMap()
		{
			@Override
			public boolean isPassable(int x, int y)
			{
				return !(x == 4 && (y == 1 || y == 2 || y == 3)) && x >= 0
						&& x < getWidth() && y >= 0 && y < getHeight();
			}

			@Override
			public int getWidth()
			{
				return 8;
			}

			@Override
			public int getHeight()
			{
				return 6;
			}

			@Override
			public int getNodePenalty(int x, int y)
			{
				return 0;
			}
		});

		// Try to find the path around the obstacle
		List<Point> path = pathfinder
				.findPath(new Point(2, 2), new Point(6, 2));
		assertEquals(4, path.size());

		// Initialize a harder parcour
		pathfinder.update(new IPathfindingMap()
		{
			@Override
			public boolean isPassable(int x, int y)
			{
				return !(x == 4 && (y == 1 || y == 2 || y == 3))
						&& !(y == 1 && (x == 2 || x == 3))
						&& !(x == 3 && y == 4) && x >= 0 && x < getWidth()
						&& y >= 0 && y < getHeight();
			}

			@Override
			public int getWidth()
			{
				return 8;
			}

			@Override
			public int getHeight()
			{
				return 6;
			}

			@Override
			public int getNodePenalty(int x, int y)
			{
				return 0;
			}
		});

		// Try to find the path around the obstacle
		path = pathfinder.findPath(new Point(2, 2), new Point(6, 2));
		assertEquals(6, path.size());

		// Try another one
		pathfinder.update(new IPathfindingMap()
		{
			@Override
			public boolean isPassable(int x, int y)
			{
				return !(x == 4 && (y == 1 || y == 2 || y == 3))
						&& !(y == 1 && (x == 2 || x == 3))
						&& !(x == 3 && y == 4) && !(x == 1 && y == 0) && x >= 0
						&& x < getWidth() && y >= 0 && y < getHeight();
			}

			@Override
			public int getWidth()
			{
				return 8;
			}

			@Override
			public int getHeight()
			{
				return 6;
			}

			@Override
			public int getNodePenalty(int x, int y)
			{
				return 0;
			}
		});

		// Try to find the path around the obstacle
		path = pathfinder.findPath(new Point(2, 2), new Point(6, 2));
		assertEquals(6, path.size());

		// Try to find a path to a not passable place
		path = pathfinder.findPath(new Point(2, 2), new Point(4, 1));
		assertNull(path);
	}
}
