package game.states.map.entities;

import game.states.map.entity.BattleCharacter;

import java.util.Set;

import junit.framework.TestCase;

import org.junit.Test;
import org.lwjgl.util.Point;

public class BattleCharacterTest extends TestCase
{
	public static class TestBattleCharacter extends BattleCharacter
	{
		public TestBattleCharacter(int x, int y)
		{
			super(x, y, 32, 32);
		}

		@Override
		public int getFootX()
		{
			return getX();
		}

		@Override
		public int getFootY()
		{
			return getY();
		}

		@Override
		public Set<Point> getTilesOnLine(final int targetX, final int targetY)
		{
			return super.getTilesOnLine(targetX, targetY);
		}
	}

	@Test
	public void testGetTilesOnLine()
	{
		TestBattleCharacter chara = new TestBattleCharacter(12 * 32, 10 * 32);

		// Check straight right
		Set<Point> tiles = chara.getTilesOnLine(14 * 32, 10 * 32);
		assertNotNull(tiles);
		assertEquals(2, tiles.size());
		assertTrue(tiles.contains(new Point(12, 10)));
		assertTrue(tiles.contains(new Point(13, 10)));

		// Straight down
		tiles = chara.getTilesOnLine(12 * 32, 13 * 32);
		assertNotNull(tiles);
		assertEquals(3, tiles.size());
		assertTrue(tiles.contains(new Point(12, 10)));
		assertTrue(tiles.contains(new Point(12, 11)));
		assertTrue(tiles.contains(new Point(12, 12)));

		// 45 degree
		tiles = chara.getTilesOnLine(15 * 32, 13 * 32);
		assertNotNull(tiles);
		assertEquals(7, tiles.size());
		assertTrue(tiles.contains(new Point(12, 10)));
		assertTrue(tiles.contains(new Point(12, 11)));
		assertTrue(tiles.contains(new Point(13, 11)));
		assertTrue(tiles.contains(new Point(13, 12)));
		assertTrue(tiles.contains(new Point(14, 12)));
		assertTrue(tiles.contains(new Point(14, 13)));
		assertTrue(tiles.contains(new Point(15, 13)));

		// Steep
		tiles = chara.getTilesOnLine(15 * 32, 15 * 32);
		assertNotNull(tiles);
		assertEquals(9, tiles.size());
		assertTrue(tiles.contains(new Point(12, 10)));
		assertTrue(tiles.contains(new Point(12, 11)));
		assertTrue(tiles.contains(new Point(13, 11)));
		assertTrue(tiles.contains(new Point(13, 12)));
		assertTrue(tiles.contains(new Point(13, 13)));
		assertTrue(tiles.contains(new Point(14, 13)));
		assertTrue(tiles.contains(new Point(14, 14)));
		assertTrue(tiles.contains(new Point(15, 14)));
		assertTrue(tiles.contains(new Point(15, 15)));
	}
}
